@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2015-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -33,8 +33,11 @@
  */
 
 #include "color_palette_widget.h"
-#include "../src/animation_flight.hpp"
-#include "../src/animation_keyframes.hpp"
+
+#include <QColorDialog>
+
+#include "src/animation_flight.hpp"
+#include "src/animation_keyframes.hpp"
 
 ColorPaletteWidget::ColorPaletteWidget(QWidget *parent)
 		: QWidget(parent), CommonMyWidgetWrapper(this)
@@ -66,7 +69,7 @@ void ColorPaletteWidget::paintEvent(QPaintEvent *event)
 		for (int x = 0; x < this->width(); x++)
 		{
 			sRGB color = palette.IndexToColour(x * 16 + paletteOffset * 256);
-			int offset = paletteOffset * 16;
+			const int offset = paletteOffset * 16;
 			if ((x + offset) % 16 == 8)
 			{
 				color.R += 32;
@@ -103,18 +106,20 @@ void ColorPaletteWidget::mousePressEvent(QMouseEvent *event)
 {
 	QWidget::mousePressEvent(event);
 
-	QPoint point = event->pos();
-	Qt::MouseButton button = event->button();
+	const QPoint point = event->pos();
+	const Qt::MouseButton button = event->button();
 
 	if (button == Qt::LeftButton)
 	{
-		int x = point.x() + 8 + paletteOffset * 16;
+		const int x = point.x() + 8 + paletteOffset * 16;
 		int index = (x / 16) % palette.GetSize();
 		QColorDialog colorDialog(this);
+		colorDialog.setOption(QColorDialog::DontUseNativeDialog);
 		sRGB colorRGB = palette.GetColor(index);
 		QColor color(colorRGB.R, colorRGB.G, colorRGB.B);
 		colorDialog.setCurrentColor(color);
-		colorDialog.setWindowTitle(QString("Edit color # ") + QString::number(index + 1));
+		colorDialog.setWindowTitle(
+			tr("Edit color #%1 of %2").arg(QString::number(index + 1)).arg(parameterName));
 		if (colorDialog.exec() == QDialog::Accepted)
 		{
 			color = colorDialog.currentColor();
@@ -124,7 +129,7 @@ void ColorPaletteWidget::mousePressEvent(QMouseEvent *event)
 	}
 }
 
-cColorPalette ColorPaletteWidget::GetDefault(void)
+cColorPalette ColorPaletteWidget::GetDefault()
 {
 	if (parameterContainer && !gotDefault)
 	{
