@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -32,28 +32,26 @@
  * ColorPaletteWidget class - promoted QWidget for displaying color palette
  */
 
-#ifndef COLOR_PALETTE_WIDGET_H_
-#define COLOR_PALETTE_WIDGET_H_
+#ifndef MANDELBULBER2_QT_COLOR_PALETTE_WIDGET_H_
+#define MANDELBULBER2_QT_COLOR_PALETTE_WIDGET_H_
 
-#include "../qt/common_my_widget_wrapper.h"
-#include "../src/color_palette.hpp"
-#include "../src/parameters.hpp"
 #include <QColorDialog>
-#include <QMenu>
 #include <QMouseEvent>
-#include <QPainter>
-#include <QPoint>
 #include <QWidget>
 #include <QtCore>
 
+#include "common_my_widget_wrapper.h"
+
+#include "src/color_palette.hpp"
+
 class ColorPaletteWidget : public QWidget, public CommonMyWidgetWrapper
 {
 	Q_OBJECT
 
 public:
-	ColorPaletteWidget(QWidget *parent = 0);
+	ColorPaletteWidget(QWidget *parent = nullptr);
 	void SetPalette(const cColorPalette &_palette);
-	cColorPalette GetPalette(void) { return palette; }
+	cColorPalette GetPalette() const { return palette; }
 	void SetOffset(double offset);
 
 private:
@@ -61,16 +59,16 @@ private:
 	double paletteOffset;
 	cColorPalette GetDefault();
 	cColorPalette defaultValue;
-	void paintEvent(QPaintEvent *event);
+	void paintEvent(QPaintEvent *event) override;
 
 	// methods to define from CommonMyWidgetWrapper
-	virtual void resetToDefault();
-	virtual QString getDefaultAsString();
-	virtual QString getFullParameterName();
+	void resetToDefault() override;
+	QString getDefaultAsString() override;
+	QString getFullParameterName() override;
 
 protected:
-	void mousePressEvent(QMouseEvent *event);
-	void contextMenuEvent(QContextMenuEvent *event);
+	void mousePressEvent(QMouseEvent *event) override;
+	void contextMenuEvent(QContextMenuEvent *event) override;
 };
 
-#endif /* COLOR_PALETTE_H_ */
+#endif /* MANDELBULBER2_QT_COLOR_PALETTE_WIDGET_H_ */
