@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -27,7 +27,7 @@
  *
  * ###########################################################################
  *
- * Authors: Krzysztof Marczak (buddhi1980@gmail.com)
+ * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Robert Pancoast (RobertPancoast77@gmail.com)
  *
  * CVector2, CVector3, CVector4 and CMatrix33 classes - vector and matrix
  * manipulation
@@ -37,15 +37,18 @@
  * rotation in 3d space.
  */
 
-#ifndef ALGEBRA_H_
-#define ALGEBRA_H_
+#ifndef MANDELBULBER2_SRC_ALGEBRA_HPP_
+#define MANDELBULBER2_SRC_ALGEBRA_HPP_
 
 #ifndef _USE_MATH_DEFINES
 #define _USE_MATH_DEFINES
 #endif
 
+#include <gsl/gsl_sys.h>
+
 #include <QString>
-#include <math.h>
+
+#include "include_header_wrapper.hpp"
 
 /************************* vector 3D **********************/
 class CVector3
@@ -77,9 +80,16 @@ public:
 	}
 	inline CVector3(const double vector[3])
 	{
-		x = vector[0];
-		y = vector[1];
-		z = vector[2];
+		if (vector)
+		{
+			x = vector[0];
+			y = vector[1];
+			z = vector[2];
+		}
+		else
+		{
+			qFatal("Assigned nullptr to CVector3 as initialization!");
+		}
 	}
 	inline CVector3 operator+(const CVector3 &vector) const
 	{
@@ -160,11 +170,15 @@ public:
 		z /= vector.z;
 		return *this;
 	}
-	inline bool operator==(const CVector3 &vector)
+	inline bool operator==(const CVector3 &vector) const
 	{
 		return x == vector.x && y == vector.y && z == vector.z;
 	}
 	inline double Length() const { return sqrt(x * x + y * y + z * z); }
+	inline double LengthPow(double p) const
+	{
+		return pow(pow(x, p) + pow(y, p) + pow(z, p), 1.0 / p);
+	}
 	inline double Dot(const CVector3 &vector) const
 	{
 		return x * vector.x + y * vector.y + z * vector.z;
@@ -177,7 +191,7 @@ public:
 		c.z = x * v.y - y * v.x;
 		return c;
 	}
-	inline CVector3 Abs()
+	inline CVector3 Abs() const
 	{
 		CVector3 c;
 		c.x = fabs(x);
@@ -187,7 +201,7 @@ public:
 	}
 	inline double Normalize() // returns normalization factor
 	{
-		double norm = 1.0 / Length();
+		const double norm = 1.0 / Length();
 		x = x * norm;
 		y = y * norm;
 		z = z * norm;
@@ -195,17 +209,21 @@ public:
 	}
 	inline double GetAlpha() const { return atan2(y, x); }
 	inline double GetBeta() const { return atan2(z, sqrt(x * x + y * y)); }
-	bool IsNotANumber()
+	inline bool IsNotANumber() const
 	{
-		return x == (x + 1e60) / 0.0 || y == (y + 1e60) / 0.0 || z == (z + 1e60) / 0.0;
+		// Check for NaN
+		// single "&" is intended: since "most of times" gsl_finite is true,
+		// always evaluating xyz at once allows cpu to perform faster
+		// than evaluating each dimension sequentially.
+		return !(gsl_finite(x) & gsl_finite(y) & gsl_finite(z));
 	}
-	CVector3 RotateAroundVectorByAngle(CVector3 axis, double angle);
+	CVector3 RotateAroundVectorByAngle(CVector3 axis, double angle) const;
 	QString Debug() const
 	{
 		return QString("[%1, %2, %3]").arg(QString::number(x), QString::number(y), QString::number(z));
 	}
 
-	double itemByName(char item);
+	double itemByName(char item) const;
 
 	double x;
 	double y;
@@ -219,7 +237,7 @@ inline CVector3 operator*(double scalar, CVector3 vector)
 
 inline CVector3 operator/(double scalar, CVector3 vector)
 {
-	return CVector3(vector.x / scalar, vector.y / scalar, vector.z / scalar);
+	return CVector3(scalar / vector.x, scalar / vector.y, scalar / vector.z);
 }
 inline CVector3 fabs(CVector3 v)
 {
@@ -229,6 +247,35 @@ inline CVector3 fabs(CVector3 v)
 	return v;
 }
 
+/******************* Structured vector 3D ****************/
+class StructuredCVector3
+{
+public:
+	inline StructuredCVector3()
+	{
+		for (int n = 0; n < 8; n++)
+		{
+			x[n] = 0.0;
+			y[n] = 0.0;
+			z[n] = 0.0;
+		}
+	}
+
+	inline StructuredCVector3(CVector3 vectors[])
+	{
+		for (int n = 0; n < 8; n++)
+		{
+			x[n] = vectors[n].x;
+			y[n] = vectors[n].y;
+			z[n] = vectors[n].z;
+		}
+	}
+
+	double x[8];
+	double y[8];
+	double z[8];
+};
+
 /************************* vector 4D **********************/
 class CVector4
 {
@@ -281,6 +328,11 @@ public:
 	{
 		return CVector4(x * v.x, y * v.y, z * v.z, w * v.w);
 	}
+	inline CVector4 operator*(const CVector3 &v) const
+	{
+		return CVector4(x * v.x, y * v.y, z * v.z, w);
+	}
+
 	inline CVector4 operator/(const double &s) const { return CVector4(x / s, y / s, z / s, w / s); }
 	inline CVector4 &operator=(const CVector4 &v)
 	{
@@ -322,6 +374,14 @@ public:
 		w *= v.w;
 		return *this;
 	}
+	inline CVector4 &operator*=(const CVector3 &v)
+	{
+		x *= v.x;
+		y *= v.y;
+		z *= v.z;
+		w *= 1.0;
+		return *this;
+	}
 	inline CVector4 &operator/=(const double &s)
 	{
 		x /= s;
@@ -330,10 +390,13 @@ public:
 		w /= s;
 		return *this;
 	}
-	inline bool operator==(const CVector4 &v) { return x == v.x && y == v.y && z == v.z && w == v.w; }
+	inline bool operator==(const CVector4 &v) const
+	{
+		return x == v.x && y == v.y && z == v.z && w == v.w;
+	}
 	inline double Length() const { return sqrt(x * x + y * y + z * z + w * w); }
 	inline double Dot(const CVector4 &v) const { return x * v.x + y * v.y + z * v.z + w * v.w; }
-	inline CVector4 Abs()
+	inline CVector4 Abs() const
 	{
 		CVector4 c;
 		c.x = fabs(x);
@@ -342,11 +405,18 @@ public:
 		c.w = fabs(w);
 		return c;
 	}
-	inline CVector3 GetXYZ() { return CVector3(x, y, z); }
+	inline void GetXYZWInto(CVector3 &v, double &wOutput) const
+	{
+		v.x = x;
+		v.y = y;
+		v.z = z;
+		wOutput = this->w;
+	}
+	inline CVector3 GetXYZ() const { return CVector3(x, y, z); }
 
 	inline double Normalize()
 	{
-		double norm = 1.0 / Length();
+		const double norm = 1.0 / Length();
 		x = x * norm;
 		y = y * norm;
 		z = z * norm;
@@ -364,13 +434,25 @@ public:
 		if (v.Length() == 0.0) return *this;
 		return (((*this - v * 0.5) % v) + v) % v - v * 0.5;
 	}
+
+	inline bool IsNotANumber() const
+	{
+		// Check for NaN
+		// single "&" is intended: since "most of times" gsl_finite is true,
+		// always evaluating xyz at once allows cpu to perform faster
+		// than evaluating each dimension sequentially.
+		return !(gsl_finite(x) & gsl_finite(y) & gsl_finite(z) & gsl_finite(w));
+	}
+
+	CVector4 RotateAroundVectorByAngle(CVector3 axis, double angle) const;
+
 	QString Debug() const
 	{
 		return QString("[%1, %2, %3, %4]")
 			.arg(QString::number(x), QString::number(y), QString::number(z), QString::number(w));
 	}
 
-	double itemByName(char item);
+	double itemByName(char item) const;
 
 	double x, y, z, w;
 };
@@ -382,7 +464,7 @@ inline CVector4 operator*(double scalar, CVector4 vector)
 
 inline CVector4 operator/(double scalar, CVector4 vector)
 {
-	return CVector4(vector.x / scalar, vector.y / scalar, vector.z / scalar, vector.w / scalar);
+	return CVector4(scalar / vector.x, scalar / vector.y, scalar / vector.z, scalar / vector.w);
 }
 
 inline CVector4 fabs(CVector4 v)
@@ -447,6 +529,16 @@ public:
 		return *this;
 	}
 	inline double Length() const { return sqrt(x * x + y * y); }
+	inline double LengthPow(double p) const { return pow(pow(x, p) + pow(y, p), 1.0 / p); }
+	inline CVector2 Deadband() const
+	{
+		CVector2 c;
+		c.x = x;
+		c.y = y;
+		if (fabs(x) < 0.1) c.x = 0;
+		if (fabs(y) < 0.1) c.y = 0;
+		return c;
+	}
 	inline double Dot(const CVector2 &vector) const { return x * vector.x + y * vector.y; }
 	inline CVector2 Abs()
 	{
@@ -467,7 +559,7 @@ public:
 	}
 	inline double Normalize() // returns normalization factor
 	{
-		double norm = 1.0 / Length();
+		const double norm = 1.0 / Length();
 		x = x * norm;
 		y = y * norm;
 		return norm;
@@ -487,7 +579,7 @@ inline CVector2<T> operator*(T scalar, CVector2<T> vector)
 template <typename T>
 inline CVector2<T> operator/(T scalar, CVector2<T> vector)
 {
-	return CVector2<T>(vector.x / scalar, vector.y / scalar);
+	return CVector2<T>(scalar / vector.x, scalar / vector.y);
 }
 
 template <typename T>
@@ -529,6 +621,7 @@ public:
 	void RotateZ(double angle);
 	void Null();
 	CVector3 RotateVector(const CVector3 &vector) const;
+	CVector4 RotateVector(const CVector4 &vector) const;
 	double GetAlfa() const;
 	double GetBeta() const;
 	double GetGamma() const;
@@ -537,12 +630,76 @@ public:
 	void SetRotation(CVector3 rotation);
 	void SetRotation2(CVector3 rotation);
 	void SetRotation3(CVector3 rotation);
-	CRotationMatrix Transpose(void) const;
-	CMatrix33 GetMatrix() { return matrix; }
+	CRotationMatrix Transpose() const;
+	CMatrix33 GetMatrix() const { return matrix; }
 
 private:
 	CMatrix33 matrix;
 	bool zero;
 };
 
-#endif /* ALGEBRA_H_ */
+/************************* matrix 4x4 (fast) *****************/
+class CMatrix44
+{
+public:
+	CMatrix44();
+	CMatrix44(const CMatrix44 &matrix);
+	CMatrix44(const CVector4 &v1, const CVector4 &v2, const CVector4 &v3, const CVector4 &v4);
+	CMatrix44 operator*(const CMatrix44 &matrix) const;
+	CVector4 operator*(const CVector4 &vector) const;
+	CMatrix44 &operator=(const CMatrix44 &);
+	double m11;
+	double m12;
+	double m13;
+	double m14;
+	double m21;
+	double m22;
+	double m23;
+	double m24;
+	double m31;
+	double m32;
+	double m33;
+	double m34;
+	double m41;
+	double m42;
+	double m43;
+	double m44;
+};
+/************************* rotation matrix 44 *******************/
+class CRotationMatrix44
+{
+public:
+	CRotationMatrix44();
+	void RotateXY(double angle);
+	void RotateYZ(double angle);
+	void RotateXZ(double angle);
+	void RotateXW(double angle);
+	void RotateYW(double angle);
+	void RotateZW(double angle);
+	void Null();
+	CVector4 RotateVector(const CVector4 &vector) const; // ....................??
+	double GetAlfa() const;
+	double GetBeta() const;
+	double GetGamma() const;
+	double GetDelta() const;
+	double GetEpsilon() const;
+	double GetZeta() const;
+	void SetRotation(double angles[6]);
+	void SetRotation(
+		double alpha, double beta, double gamma, double delta, double epsilon, double zeta);
+	// void SetRotation(CVector4 rotation);
+	// void SetRotation2(CVector4 rotation);
+	// void SetRotation3(CVector4 rotation);
+
+	void SetRotation44a(CVector3 rotation);
+	void SetRotation44b(CVector3 rotation);
+
+	CRotationMatrix44 Transpose() const;
+	CMatrix44 GetMatrix() const { return matrix; }
+
+private:
+	CMatrix44 matrix;
+	bool zero;
+};
+
+#endif /* MANDELBULBER2_SRC_ALGEBRA_HPP_ */
