@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -27,7 +27,7 @@
  *
  * ###########################################################################
  *
- * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Sebastian Jennen
+ * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Sebastian Jennen (jenzebas@gmail.com)
  *
  * Functions for flight animation.
  *
@@ -38,13 +38,28 @@
  * slots to change the camera acceleration.
  */
 
-#ifndef SRC_ANIMATION_FLIGHT_HPP_
-#define SRC_ANIMATION_FLIGHT_HPP_
+#ifndef MANDELBULBER2_SRC_ANIMATION_FLIGHT_HPP_
+#define MANDELBULBER2_SRC_ANIMATION_FLIGHT_HPP_
 
+#include <qmessagebox.h>
+
+#include "algebra.hpp"
 #include "animation_frames.hpp"
-#include "cimage.hpp"
 #include "error_message.hpp"
-#include "interface.hpp"
+#include "progress_text.hpp"
+#include "statistics.h"
+
+// forward declarations
+class cImage;
+class cInterface;
+class cFractalContainer;
+class cParameterContainer;
+class MyTableWidgetAnim;
+
+namespace Ui
+{
+class cDockAnimation;
+}
 
 class cFlightAnimation : public QObject
 {
@@ -59,18 +74,18 @@ public:
 
 	cFlightAnimation(cInterface *_interface, cAnimationFrames *_frames, cImage *_image,
 		QWidget *_imageWidget, cParameterContainer *_params, cFractalContainer *_fractal,
-		QObject *parent = 0);
+		QObject *parent = nullptr);
 	void RecordFlight(bool continueRecording);
 	bool RenderFlight(bool *stopRequest);
-	void RenderFrame(int index);
+	void RenderFrame(int index) const;
 	void RefreshTable();
 	QString GetParameterName(int rowNumber);
-	void DeleteFramesFrom(int index);
-	void DeleteFramesTo(int index);
-	void UpdateThumbnailFromImage(int index);
+	void DeleteFramesFrom(int index) const;
+	void DeleteFramesTo(int index) const;
+	void UpdateThumbnailFromImage(int index) const;
 	void InterpolateForward(int row, int column);
-	QString GetFlightFilename(int index);
-	void UpdateLimitsForFrameRange(void);
+	QString GetFlightFilename(int index) const;
+	void UpdateLimitsForFrameRange() const;
 
 public slots:
 	bool slotRenderFlight();
@@ -79,20 +94,21 @@ private slots:
 	void slotRecordFlight();
 	void slotContinueRecording();
 	void slotFlightChangeSpeed(double amount);
+	void slotFlightSetSpeed(double amount);
 	void slotFlightStrafe(CVector2<double> _strafe);
 	void slotFlightYawAndPitch(CVector2<double> _yawAndPitch);
 	void slotFlightRotation(double direction);
 	void slotOrthogonalStrafe(bool _orthogonalStrafe);
-	void slotSelectAnimFlightImageDir();
+	void slotSelectAnimFlightImageDir() const;
 	void slotTableCellChanged(int row, int column);
-	void slotDeleteAllImages();
-	void slotShowAnimation();
+	void slotDeleteAllImages() const;
+	void slotShowAnimation() const;
 	void slotRecordPause();
 	void slotRefreshTable();
-	void slotExportFlightToKeyframes();
-	void slotMovedSliderFirstFrame(int value);
-	void slotMovedSliderLastFrame(int value);
-	void slotCellDoubleClicked(int row, int column);
+	void slotExportFlightToKeyframes() const;
+	void slotMovedSliderFirstFrame(int value) const;
+	void slotMovedSliderLastFrame(int value) const;
+	void slotCellDoubleClicked(int row, int column) const;
 
 private:
 	void PrepareTable();
@@ -101,7 +117,7 @@ private:
 		const cAnimationFrames::sParameterDescription &parameterDescription, int index);
 	int AddColumn(const cAnimationFrames::sAnimationFrame &frame, int indexOfExistingColumn = -1);
 	cInterface *mainInterface;
-	Ui::RenderWindow *ui;
+	Ui::cDockAnimation *ui;
 	cAnimationFrames *frames;
 	cImage *image;
 	QWidget *imageWidget;
@@ -115,9 +131,11 @@ private:
 	CVector2<double> yawAndPitch;
 	double rotationDirection;
 	bool orthogonalStrafe;
+	bool negativeFlightSpeed;
 	double linearSpeedSp;
 	// QList<cThumbnailWidget*> thumbnailWidgets;
 	bool recordPause;
+	QSize previewSize;
 
 signals:
 	void updateProgressAndStatus(const QString &text, const QString &progressText, double progress,
@@ -128,9 +146,10 @@ signals:
 	void QuestionMessage(const QString &questionTitle, const QString &questionText,
 		QMessageBox::StandardButtons buttons, QMessageBox::StandardButton *reply);
 	void showErrorMessage(
-		QString text, cErrorMessage::enumMessageType messageType, QWidget *parent = NULL);
+		QString text, cErrorMessage::enumMessageType messageType, QWidget *parent = nullptr);
+	void notifyRenderFlightRenderStatus(QString text, QString progressText);
 };
 
 extern cFlightAnimation *gFlightAnimation;
 
-#endif /* SRC_ANIMATION_FLIGHT_HPP_ */
+#endif /* MANDELBULBER2_SRC_ANIMATION_FLIGHT_HPP_ */
