@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -35,13 +35,18 @@
  * and exposes functions to modify this matrix.
  */
 
-#ifndef SRC_ANIMATION_FRAMES_HPP_
-#define SRC_ANIMATION_FRAMES_HPP_
+#ifndef MANDELBULBER2_SRC_ANIMATION_FRAMES_HPP_
+#define MANDELBULBER2_SRC_ANIMATION_FRAMES_HPP_
 
-#include "fractal_container.hpp"
-#include "parameters.hpp"
 #include <QtGui>
 
+#include "audio_track_collection.h"
+#include "parameters.hpp"
+
+// forward declarations
+class cFractalContainer;
+class cAudioTrack;
+
 class cAnimationFrames
 {
 public:
@@ -85,37 +90,59 @@ public:
 	virtual int GetUnrenderedTotal();
 	void Clear();
 	void ClearAll();
-	virtual void AddAnimatedParameter(
-		const QString &parameterName, const cOneParameter &defaultValue);
-	virtual bool AddAnimatedParameter(const QString &fullParameterName,
-		const cParameterContainer *param, const cFractalContainer *fractal);
+	virtual void AddAnimatedParameter(const QString &parameterName, const cOneParameter &defaultValue,
+		cParameterContainer *params = nullptr);
+	virtual bool AddAnimatedParameter(
+		const QString &fullParameterName, cParameterContainer *param, const cFractalContainer *fractal);
 	virtual void RemoveAnimatedParameter(const QString &fullParameterName);
 	QList<sParameterDescription> GetListOfUsedParameters() const { return listOfParameters; }
-	const cParameterContainer *ContainerSelector(QString containerName,
-		const cParameterContainer *params, const cFractalContainer *fractal) const;
-	cParameterContainer *ContainerSelector(
-		QString containerName, cParameterContainer *params, cFractalContainer *fractal) const;
+	static const cParameterContainer *ContainerSelector(
+		QString containerName, const cParameterContainer *params, const cFractalContainer *fractal);
+	static cParameterContainer *ContainerSelector(
+		QString containerName, cParameterContainer *params, cFractalContainer *fractal);
 	void DeleteFrames(int begin, int end);
 	void Override(QList<sAnimationFrame> _frames, QList<sParameterDescription> _listOfParameters)
 	{
 		frames = _frames;
 		listOfParameters = _listOfParameters;
 	}
-	QList<sAnimationFrame> GetFrames() { return frames; }
-	QList<sParameterDescription> GetListOfParameters() { return listOfParameters; }
-	void SetListOfParametersAndClear(QList<sParameterDescription> _listOfParameters)
-	{
-		listOfParameters = _listOfParameters;
-		frames.clear();
-	}
+	QList<sAnimationFrame> GetFrames() const { return frames; }
+	QList<sParameterDescription> GetListOfParameters() const { return listOfParameters; }
+	void SetListOfParametersAndClear(
+		QList<sParameterDescription> _listOfParameters, cParameterContainer *params);
+
 	int IndexOnList(QString parameterName, QString containerName);
 
-protected:
+	void AddAudioParameter(const QString &parameterName, enumVarType paramType,
+		const QString originalContainerName, cParameterContainer *params = nullptr);
 
+	void RemoveAudioParameter(
+		const sParameterDescription &parameter, cParameterContainer *params = nullptr);
+
+	void RemoveAllAudioParameters(cParameterContainer *params = nullptr);
+	void LoadAllAudioFiles(cParameterContainer *params = nullptr);
+
+	cAudioTrack *GetAudioPtr(const QString fullParameterName) const;
+
+	cOneParameter ApplyAudioAnimation(int frame, const cOneParameter &parameter,
+		const QString &parameterName, const cParameterContainer *params) const;
+
+	template <typename T>
+	T ApplyAudioAnimationOneComponent(int frame, T oldVal, const QString &fullParameterNameWithSuffix,
+		const cParameterContainer *params) const;
+
+	void RegenerateAudioTracks(cParameterContainer *param);
+	void RefreshAllAudioTracks(cParameterContainer *param);
+
+	virtual void setAudioParameterPrefix();
+	static void WipeFramesFromFolder(QString folder);
+
+protected:
 	QList<sAnimationFrame> frames;
 	QList<sParameterDescription> listOfParameters;
+	cAudioTrackCollection audioTracks;
 };
 
 extern cAnimationFrames *gAnimFrames;
 
-#endif /* SRC_ANIMATION_FRAMES_HPP_ */
+#endif /* MANDELBULBER2_SRC_ANIMATION_FRAMES_HPP_ */
