@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2015-18 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -27,66 +27,87 @@
  *
  * ###########################################################################
  *
- * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Sebastian Jennen
+ * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Sebastian Jennen (jenzebas@gmail.com)
  *
  * Functions for flight animation.
  *
  * cKeyframeAnimation contains all functionality for editing and rendering of
  * a keyframe animation. The class holds a cKeyframes to store
  * the parameters of the keyframes and a table to display the frames in a widget.
- * The class can render interpolated subframes and validate their path (check
+ * The class can render interpolated sub-frames and validate their path (check
  * for collision). It exposes slots to manipulate the keyframes.
  */
 
-#ifndef SRC_ANIMATION_KEYFRAMES_HPP_
-#define SRC_ANIMATION_KEYFRAMES_HPP_
+#ifndef MANDELBULBER2_SRC_ANIMATION_KEYFRAMES_HPP_
+#define MANDELBULBER2_SRC_ANIMATION_KEYFRAMES_HPP_
+
+#include <qmessagebox.h>
 
-#include "cimage.hpp"
 #include "error_message.hpp"
-#include "interface.hpp"
 #include "keyframes.hpp"
+#include "progress_text.hpp"
+#include "statistics.h"
+
+// forward declarations
+class cImage;
+class cInterface;
+class cFractalContainer;
+class cParameterContainer;
+class MyTableWidgetKeyframes;
+class RenderedImage;
+
+namespace Ui
+{
+class cDockAnimation;
+}
 
 class cKeyframeAnimation : public QObject
 {
 	Q_OBJECT
 
 public:
+	static const int reservedColumns = 1;
+	static const int animSoundColumn = 0;
+
 	cKeyframeAnimation(cInterface *_interface, cKeyframes *_frames, cImage *_image,
 		QWidget *_imageWidget, cParameterContainer *_params, cFractalContainer *_fractal,
-		QObject *parent = 0);
+		QObject *parent = nullptr);
 
 	bool RenderKeyframes(bool *stopRequest);
-	void RenderFrame(int index);
+	void RenderFrame(int index) const;
 	void RefreshTable();
 	QString GetParameterName(int rowNumber);
-	void DeleteKeyframe(int index);
-	void DeleteFramesFrom(int index);
-	void DeleteFramesTo(int index);
+	void DeleteKeyframe(int index) const;
+	void DeleteFramesFrom(int index) const;
+	void DeleteFramesTo(int index) const;
 	void InterpolateForward(int row, int column);
-	parameterContainer::enumMorphType GetMorphType(int row);
+	parameterContainer::enumMorphType GetMorphType(int row) const;
 	void ChangeMorphType(int row, parameterContainer::enumMorphType morphType);
 	QList<int> CheckForCollisions(double minDist, bool *stopRequest);
+	void UpdateActualCameraPosition(const CVector3 &cameraPosition);
 
 public slots:
-	void UpdateLimitsForFrameRange();
+	void UpdateLimitsForFrameRange() const;
 	bool slotRenderKeyframes();
 
 private slots:
 	void slotAddKeyframe();
 	void slotInsertKeyframe();
-	void slotDeleteKeyframe();
+	void slotDeleteKeyframe() const;
 	void slotModifyKeyframe();
-	void slotSelectKeyframeAnimImageDir();
+	void slotSelectKeyframeAnimImageDir() const;
 	void slotTableCellChanged(int row, int column);
-	void slotDeleteAllImages();
-	void slotShowAnimation();
+	void slotDeleteAllImages() const;
+	void slotShowAnimation() const;
 	void slotRefreshTable();
 	void slotExportKeyframesToFlight();
-	void slotMovedSliderFirstFrame(int value);
+	void slotMovedSliderFirstFrame(int value) const;
 	void slotMovedSliderLastFrame(int value);
 	void slotValidate();
-	void slotCellDoubleClicked(int row, int column);
+	void slotCellDoubleClicked(int row, int column) const;
+	void slotCellClicked(int row, int column) const;
 	void slotSetConstantTargetDistance();
+	void slotUpdateAnimationPathSelection();
 
 private:
 	void PrepareTable();
@@ -94,21 +115,28 @@ private:
 	int AddVariableToTable(
 		const cAnimationFrames::sParameterDescription &parameterDescription, int index);
 	int AddColumn(const cAnimationFrames::sAnimationFrame &frame, int index = -1);
+	void AddRow(int row, const QString &fullParameterName, int index);
 	void NewKeyframe(int index);
-	QString GetKeyframeFilename(int index, int subIndex);
-	QColor MorphType2Color(parameterContainer::enumMorphType morphType);
+	QString GetKeyframeFilename(int index, int subIndex) const;
+	static QColor MorphType2Color(parameterContainer::enumMorphType morphType);
+	void AddAnimSoundColumn() const;
+	void UpdateAnimationPath() const;
+	void UpdateCameraDistanceInformation() const;
 
 	cInterface *mainInterface;
-	Ui::RenderWindow *ui;
+	Ui::cDockAnimation *ui;
 	cKeyframes *keyframes;
 	cImage *image;
-	QWidget *imageWidget;
+	RenderedImage *imageWidget;
 	cParameterContainer *params;
 	cFractalContainer *fractalParams;
 	QStringList tableRowNames;
 	QVector<int> parameterRows; // position of parameter in table
 	QVector<int> rowParameter;	// index of parameter in row
 	MyTableWidgetKeyframes *table;
+	bool lastToRenderMax = false;
+	QSize previewSize;
+	CVector3 actualCameraPosition;
 
 signals:
 	void updateProgressAndStatus(const QString &text, const QString &progressText, double progress,
@@ -119,9 +147,10 @@ signals:
 	void QuestionMessage(const QString &questionTitle, const QString &questionText,
 		QMessageBox::StandardButtons buttons, QMessageBox::StandardButton *reply);
 	void showErrorMessage(
-		QString text, cErrorMessage::enumMessageType messageType, QWidget *parent = NULL);
+		QString text, cErrorMessage::enumMessageType messageType, QWidget *parent = nullptr);
+	void notifyRenderKeyframeRenderStatus(QString text, QString progressText);
 };
 
 extern cKeyframeAnimation *gKeyframeAnimation;
 
-#endif /* SRC_ANIMATION_KEYFRAMES_HPP_ */
+#endif /* MANDELBULBER2_SRC_ANIMATION_KEYFRAMES_HPP_ */
