@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2016-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -47,19 +47,19 @@ cAutomatedWidgets::cAutomatedWidgets(QObject *parent) : QObject(parent)
 {
 }
 
-void cAutomatedWidgets::slotSliderMoved(int value)
+void cAutomatedWidgets::slotSliderMoved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
+	const QString sliderName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinbox_") + parameterName;
+	const QString spinBoxName = QString("spinbox_") + parameterName;
 
 	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 	if (spinBox)
 	{
-		double decimals = spinBox->decimals();
-		double divider = pow(10.0, decimals);
+		const double decimals = spinBox->decimals();
+		const double divider = pow(10.0, decimals);
 		spinBox->setValue(value / divider);
 	}
 	else
@@ -68,20 +68,20 @@ void cAutomatedWidgets::slotSliderMoved(int value)
 	}
 }
 
-void cAutomatedWidgets::slotDoubleSpinBoxChanged(double value)
+void cAutomatedWidgets::slotDoubleSpinBoxChanged(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
+	const QString spinBoxName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString sliderName = QString("slider_") + parameterName;
+	const QString sliderName = QString("slider_") + parameterName;
 	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
 	if (slider)
 	{
 		QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox *>(this->sender());
-		double decimals = spinBox->decimals();
-		double multiplier = pow(10.0, decimals);
-		slider->setValue(value * multiplier);
+		const double decimals = spinBox->decimals();
+		const double multiplier = pow(10.0, decimals);
+		slider->setValue(int(value * multiplier));
 	}
 	else
 	{
@@ -90,13 +90,13 @@ void cAutomatedWidgets::slotDoubleSpinBoxChanged(double value)
 	}
 }
 
-void cAutomatedWidgets::slotIntSliderMoved(int value)
+void cAutomatedWidgets::slotIntSliderMoved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
+	const QString sliderName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinboxName = QString("spinboxInt_") + parameterName;
+	const QString spinboxName = QString("spinboxInt_") + parameterName;
 
 	QSpinBox *spinbox = this->sender()->parent()->findChild<QSpinBox *>(spinboxName);
 	if (spinbox)
@@ -110,13 +110,13 @@ void cAutomatedWidgets::slotIntSliderMoved(int value)
 	}
 }
 
-void cAutomatedWidgets::slotIntSpinBoxChanged(int value)
+void cAutomatedWidgets::slotIntSpinBoxChanged(int value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
+	const QString spinBoxName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString sliderName = QString("sliderInt_") + parameterName;
+	const QString sliderName = QString("sliderInt_") + parameterName;
 
 	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
 	if (slider)
@@ -130,20 +130,20 @@ void cAutomatedWidgets::slotIntSpinBoxChanged(int value)
 	}
 }
 
-void cAutomatedWidgets::slotSlider3Moved(int value)
+void cAutomatedWidgets::slotSlider3Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
+	const QString sliderName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinbox3_") + parameterName;
+	const QString spinBoxName = QString("spinbox3_") + parameterName;
 
 	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 
 	if (spinBox)
 	{
-		double decimals = spinBox->decimals();
-		double divider = pow(10.0, decimals);
+		const double decimals = spinBox->decimals();
+		const double divider = pow(10.0, decimals);
 		spinBox->setValue(value / divider);
 	}
 	else
@@ -152,20 +152,20 @@ void cAutomatedWidgets::slotSlider3Moved(int value)
 	}
 }
 
-void cAutomatedWidgets::slotSlider4Moved(int value)
+void cAutomatedWidgets::slotSlider4Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
+	const QString sliderName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinbox4_") + parameterName;
+	const QString spinBoxName = QString("spinbox4_") + parameterName;
 
 	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 
 	if (spinBox)
 	{
-		double decimals = spinBox->decimals();
-		double divider = pow(10.0, decimals);
+		const double decimals = spinBox->decimals();
+		const double divider = pow(10.0, decimals);
 		spinBox->setValue(value / divider);
 	}
 	else
@@ -174,22 +174,22 @@ void cAutomatedWidgets::slotSlider4Moved(int value)
 	}
 }
 
-void cAutomatedWidgets::slotSpinBox3Changed(double value)
+void cAutomatedWidgets::slotSpinBox3Changed(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
+	const QString spinBoxName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString sliderName = QString("slider3_") + parameterName;
+	const QString sliderName = QString("slider3_") + parameterName;
 
 	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
 
 	if (slider)
 	{
 		QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox *>(this->sender());
-		double decimals = spinBox->decimals();
-		double multiplier = pow(10.0, decimals);
-		slider->setValue(value * multiplier);
+		const double decimals = spinBox->decimals();
+		const double multiplier = pow(10.0, decimals);
+		slider->setValue(int(value * multiplier));
 	}
 	else
 	{
@@ -197,22 +197,22 @@ void cAutomatedWidgets::slotSpinBox3Changed(double value)
 	}
 }
 
-void cAutomatedWidgets::slotSpinBox4Changed(double value)
+void cAutomatedWidgets::slotSpinBox4Changed(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
+	const QString spinBoxName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString sliderName = QString("slider4_") + parameterName;
+	const QString sliderName = QString("slider4_") + parameterName;
 
 	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
 
 	if (slider)
 	{
 		QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox *>(this->sender());
-		double decimals = spinBox->decimals();
-		double multiplier = pow(10.0, decimals);
-		slider->setValue(value * multiplier);
+		const double decimals = spinBox->decimals();
+		const double multiplier = pow(10.0, decimals);
+		slider->setValue(int(value * multiplier));
 	}
 	else
 	{
@@ -220,13 +220,13 @@ void cAutomatedWidgets::slotSpinBox4Changed(double value)
 	}
 }
 
-void cAutomatedWidgets::slotDial3Moved(int value)
+void cAutomatedWidgets::slotDial3Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
+	const QString sliderName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinboxd3_") + parameterName;
+	const QString spinBoxName = QString("spinboxd3_") + parameterName;
 
 	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 	if (spinBox)
@@ -239,13 +239,13 @@ void cAutomatedWidgets::slotDial3Moved(int value)
 	}
 }
 
-void cAutomatedWidgets::slotDial4Moved(int value)
+void cAutomatedWidgets::slotDial4Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
+	const QString sliderName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinboxd4_") + parameterName;
+	const QString spinBoxName = QString("spinboxd4_") + parameterName;
 
 	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 	if (spinBox)
@@ -258,18 +258,18 @@ void cAutomatedWidgets::slotDial4Moved(int value)
 	}
 }
 
-void cAutomatedWidgets::slotSpinBoxD3Changed(double value)
+void cAutomatedWidgets::slotSpinBoxD3Changed(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
+	const QString spinBoxName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString dialName = QString("dial3_") + parameterName;
+	const QString dialName = QString("dial3_") + parameterName;
 
 	QDial *dial = this->sender()->parent()->findChild<QDial *>(dialName);
 	if (dial)
 	{
-		dial->setValue(value * 100.0);
+		dial->setValue(int(value * 100.0));
 	}
 	else
 	{
@@ -277,18 +277,18 @@ void cAutomatedWidgets::slotSpinBoxD3Changed(double value)
 	}
 }
 
-void cAutomatedWidgets::slotSpinBoxD4Changed(double value)
+void cAutomatedWidgets::slotSpinBoxD4Changed(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
+	const QString spinBoxName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString dialName = QString("dial4_") + parameterName;
+	const QString dialName = QString("dial4_") + parameterName;
 
 	QDial *dial = this->sender()->parent()->findChild<QDial *>(dialName);
 	if (dial)
 	{
-		dial->setValue(value * 100.0);
+		dial->setValue(int(value * 100.0));
 	}
 	else
 	{
@@ -296,13 +296,13 @@ void cAutomatedWidgets::slotSpinBoxD4Changed(double value)
 	}
 }
 
-void cAutomatedWidgets::slotDialMoved(int value)
+void cAutomatedWidgets::slotDialMoved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
+	const QString sliderName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinboxd_") + parameterName;
+	const QString spinBoxName = QString("spinboxd_") + parameterName;
 
 	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 	if (spinBox)
@@ -315,18 +315,18 @@ void cAutomatedWidgets::slotDialMoved(int value)
 	}
 }
 
-void cAutomatedWidgets::slotSpinBoxDChanged(double value)
+void cAutomatedWidgets::slotSpinBoxDChanged(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
+	const QString spinBoxName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString dialName = QString("dial_") + parameterName;
+	const QString dialName = QString("dial_") + parameterName;
 
 	QDial *dial = this->sender()->parent()->findChild<QDial *>(dialName);
 	if (dial)
 	{
-		dial->setValue(value * 100.0);
+		dial->setValue(int(value * 100.0));
 	}
 	else
 	{
@@ -334,19 +334,19 @@ void cAutomatedWidgets::slotSpinBoxDChanged(double value)
 	}
 }
 
-void cAutomatedWidgets::slotLogSliderMoved(int value)
+void cAutomatedWidgets::slotLogSliderMoved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
+	const QString sliderName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString lineEditName = QString("logedit_") + parameterName;
+	const QString lineEditName = QString("logedit_") + parameterName;
 
 	QLineEdit *lineEdit = this->sender()->parent()->findChild<QLineEdit *>(lineEditName);
 	if (lineEdit)
 	{
-		double dValue = pow(10.0, value / 100.0);
-		QString text = QString("%L1").arg(dValue);
+		const double dValue = pow(10.0, value / 100.0);
+		const QString text = QString("%L1").arg(dValue);
 		lineEdit->setText(text);
 	}
 	else
@@ -356,43 +356,43 @@ void cAutomatedWidgets::slotLogSliderMoved(int value)
 	}
 }
 
-void cAutomatedWidgets::slotLogSliderVect3Moved(int value)
+void cAutomatedWidgets::slotLogSliderVect3Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
+	const QString sliderName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString lineEditName = QString("logvect3_") + parameterName;
+	const QString lineEditName = QString("logvect3_") + parameterName;
 
 	QLineEdit *lineEdit = this->sender()->parent()->findChild<QLineEdit *>(lineEditName);
 	if (lineEdit)
 	{
-		double dValue = pow(10.0, value / 100.0);
-		QString text = QString("%L1").arg(dValue);
+		const double dValue = pow(10.0, value / 100.0);
+		const QString text = QString("%L1").arg(dValue);
 		lineEdit->setText(text);
 	}
 	else
 	{
-		qWarning() << "slotLogSlidervect3Moved() error: lineEdit " << lineEditName << " doesn't exists"
+		qWarning() << "slotLogSliderVect3Moved() error: lineEdit " << lineEditName << " doesn't exists"
 							 << endl;
 	}
 }
 
-void cAutomatedWidgets::slotLogLineEditChanged(const QString &text)
+void cAutomatedWidgets::slotLogLineEditChanged(const QString &text) const
 {
 	using namespace std;
-	QString lineEditName = this->sender()->objectName();
+	const QString lineEditName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(lineEditName, &parameterName, &type);
-	QString sliderName = QString("logslider_") + parameterName;
+	const QString sliderName = QString("logslider_") + parameterName;
 
 	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
 	if (slider)
 	{
-		double value = systemData.locale.toDouble(text);
+		const double value = systemData.locale.toDouble(text);
 		if (value > 0.0)
 		{
-			int sliderPosition = log10(systemData.locale.toDouble(text)) * 100.0;
+			const int sliderPosition = int(log10(systemData.locale.toDouble(text)) * 100.0);
 			slider->setValue(sliderPosition);
 		}
 		else
@@ -408,21 +408,21 @@ void cAutomatedWidgets::slotLogLineEditChanged(const QString &text)
 	}
 }
 
-void cAutomatedWidgets::slotLogVect3Changed(const QString &text)
+void cAutomatedWidgets::slotLogVect3Changed(const QString &text) const
 {
 	using namespace std;
-	QString lineEditName = this->sender()->objectName();
+	const QString lineEditName = this->sender()->objectName();
 	QString type, parameterName;
 	GetNameAndType(lineEditName, &parameterName, &type);
-	QString sliderName = QString("logslidervect3_") + parameterName;
+	const QString sliderName = QString("logslidervect3_") + parameterName;
 
 	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
 	if (slider)
 	{
-		double value = systemData.locale.toDouble(text);
+		const double value = systemData.locale.toDouble(text);
 		if (value > 0.0)
 		{
-			int sliderPosition = log10(systemData.locale.toDouble(text)) * 100.0;
+			const int sliderPosition = int(log10(systemData.locale.toDouble(text)) * 100.0);
 			slider->setValue(sliderPosition);
 		}
 		else
@@ -437,15 +437,14 @@ void cAutomatedWidgets::slotLogVect3Changed(const QString &text)
 	}
 }
 
-//NOTE: there are used sliderMoved() signals instead of valueChangd(), because valueChanged()
-//caused problems with editing values in edit fields and spinboxes.
+// NOTE: there are used sliderMoved() signals instead of valueChanged(), because valueChanged()
+// caused problems with editing values in edit fields and SpinBoxes.
 
 // automatic setting of event slots for all sliders
-void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
+void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 {
 	QList<QSlider *> widgetList = window->findChildren<QSlider *>();
-	QList<QSlider *>::iterator it;
-	for (it = widgetList.begin(); it != widgetList.end(); ++it)
+	for (QList<QSlider *>::iterator it = widgetList.begin(); it != widgetList.end(); ++it)
 	{
 		QString name = (*it)->objectName();
 		if (name.length() > 1 && (*it)->metaObject()->className() == QString("QSlider"))
@@ -459,7 +458,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 			{
 				QApplication::connect(slider, SIGNAL(sliderMoved(int)), this, SLOT(slotSliderMoved(int)));
 
-				QString spinBoxName = QString("spinbox_") + parameterName;
+				const QString spinBoxName = QString("spinbox_") + parameterName;
 				QDoubleSpinBox *spinBox = slider->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 				if (spinBox)
 				{
@@ -477,7 +476,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 				QApplication::connect(
 					slider, SIGNAL(sliderMoved(int)), this, SLOT(slotLogSliderMoved(int)));
 
-				QString editFieldName = QString("logedit_") + parameterName;
+				const QString editFieldName = QString("logedit_") + parameterName;
 				QLineEdit *lineEdit = slider->parent()->findChild<QLineEdit *>(editFieldName);
 				if (lineEdit)
 				{
@@ -495,7 +494,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 				QApplication::connect(
 					slider, SIGNAL(sliderMoved(int)), this, SLOT(slotLogSliderVect3Moved(int)));
 
-				QString editFieldName = QString("logvect3_") + parameterName;
+				const QString editFieldName = QString("logvect3_") + parameterName;
 				QLineEdit *lineEdit = slider->parent()->findChild<QLineEdit *>(editFieldName);
 				if (lineEdit)
 				{
@@ -513,7 +512,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 				QApplication::connect(
 					slider, SIGNAL(sliderMoved(int)), this, SLOT(slotIntSliderMoved(int)));
 
-				QString spinboxName = QString("spinboxInt_") + parameterName;
+				const QString spinboxName = QString("spinboxInt_") + parameterName;
 				QSpinBox *spinbox = slider->parent()->findChild<QSpinBox *>(spinboxName);
 				if (spinbox)
 				{
@@ -529,7 +528,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 			if (type == QString("slider3"))
 			{
 				QApplication::connect(slider, SIGNAL(sliderMoved(int)), this, SLOT(slotSlider3Moved(int)));
-				QString spinboxName = QString("spinbox3_") + parameterName;
+				const QString spinboxName = QString("spinbox3_") + parameterName;
 				QDoubleSpinBox *spinbox = slider->parent()->findChild<QDoubleSpinBox *>(spinboxName);
 				if (spinbox)
 				{
@@ -545,7 +544,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 			if (type == QString("slider4"))
 			{
 				QApplication::connect(slider, SIGNAL(valueChanged(int)), this, SLOT(slotSlider4Moved(int)));
-				QString spinboxName = QString("spinbox4_") + parameterName;
+				const QString spinboxName = QString("spinbox4_") + parameterName;
 				QDoubleSpinBox *spinbox = slider->parent()->findChild<QDoubleSpinBox *>(spinboxName);
 				if (spinbox)
 				{
@@ -562,8 +561,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 	}
 
 	QList<QDial *> widgetList2 = window->findChildren<QDial *>(QString());
-	QList<QDial *>::iterator it2;
-	for (it2 = widgetList2.begin(); it2 != widgetList2.end(); ++it2)
+	for (QList<QDial *>::iterator it2 = widgetList2.begin(); it2 != widgetList2.end(); ++it2)
 	{
 		QString name = (*it2)->objectName();
 		if (name.length() > 1 && (*it2)->metaObject()->className() == QString("QDial"))
@@ -574,9 +572,9 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 
 			if (type == QString("dial3"))
 			{
-				QApplication::connect(dial, SIGNAL(sliderMoved(int)), this, SLOT(slotDial3Moved(int)));
+				QApplication::connect(dial, SIGNAL(valueChanged(int)), this, SLOT(slotDial3Moved(int)));
 
-				QString spinBoxName = QString("spinboxd3_") + parameterName;
+				const QString spinBoxName = QString("spinboxd3_") + parameterName;
 				QDoubleSpinBox *spinBox = dial->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 				if (spinBox)
 				{
@@ -593,7 +591,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 			{
 				QApplication::connect(dial, SIGNAL(sliderMoved(int)), this, SLOT(slotDial4Moved(int)));
 
-				QString spinBoxName = QString("spinboxd4_") + parameterName;
+				const QString spinBoxName = QString("spinboxd4_") + parameterName;
 				QDoubleSpinBox *spinBox = dial->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 				if (spinBox)
 				{
@@ -610,7 +608,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window)
 			{
 				QApplication::connect(dial, SIGNAL(sliderMoved(int)), this, SLOT(slotDialMoved(int)));
 
-				QString spinBoxName = QString("spinboxd_") + parameterName;
+				const QString spinBoxName = QString("spinboxd_") + parameterName;
 				QDoubleSpinBox *spinBox = dial->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 				if (spinBox)
 				{
