@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2016-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,33 +14,36 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
  * without even the implied warranty
  * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- * See the GNU General Public License for more details. You should have received
- * a copy of the GNU
- * General Public License along with Mandelbulber. If not, see
- * <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details.
+ * You should have received a copy of the GNU General Public License
+ * along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
  *
- * Authors: Sebastian Jennen (sebastian.jennen@gmx.de)
+ * ###########################################################################
+ *
+ * Authors: Sebastian Jennen (jenzebas@gmail.com)
  *
  * file image class to store different image file formats
  *
- * Each imagefiletype derives ImageFileSave and implements the SaveImage
+ * Each image file type derives ImageFileSave and implements the SaveImage
  * method to store the image data with the corresponding file format
  *
  */
 
-#ifndef FILE_IMAGE_H_
-#define FILE_IMAGE_H_
+#ifndef MANDELBULBER2_SRC_FILE_IMAGE_HPP_
+#define MANDELBULBER2_SRC_FILE_IMAGE_HPP_
 
-#include "cimage.hpp"
 #include <QtCore>
 
+#include "color_structures.hpp"
+
+// custom includes
 extern "C" {
 #include <png.h>
 }
@@ -50,6 +53,9 @@ extern "C" {
 // libpng: http://www.libpng.org/pub/png/libpng.html
 /**********************************************************/
 
+// forward declarations
+class cImage;
+
 class ImageFileSave : public QObject
 {
 	Q_OBJECT
@@ -91,14 +97,13 @@ public:
 	struct structSaveImageChannel
 	{
 		structSaveImageChannel()
-				: contentType((enumImageContentType)0),
-					channelQuality((enumImageChannelQualityType)0),
+				: contentType(enumImageContentType(0)),
+					channelQuality(enumImageChannelQualityType(0)),
 					postfix("")
 		{
 		}
 		structSaveImageChannel(enumImageContentType _contentType,
-			enumImageChannelQualityType _channelQuality,
-			QString _postfix)
+			enumImageChannelQualityType _channelQuality, QString _postfix)
 				: contentType(_contentType), channelQuality(_channelQuality), postfix(_postfix)
 		{
 		}
@@ -111,11 +116,17 @@ public:
 	typedef QMap<enumImageContentType, structSaveImageChannel> ImageConfig;
 	static QString ImageFileExtension(enumImageFileType imageFileType);
 	static QString ImageChannelName(enumImageContentType imageContentType);
+	static QStringList ImageChannelNames();
+	static QString ImageNameWithoutExtension(QString path);
 	static enumImageFileType ImageFileType(QString imageFileExtension);
 	static ImageFileSave *create(
-		QString filename, enumImageFileType filetype, cImage *image, ImageConfig imageConfig);
+		QString filename, enumImageFileType fileType, cImage *image, ImageConfig imageConfig);
 	virtual void SaveImage() = 0;
 	virtual QString getJobName() = 0;
+	void updateProgressAndStatusChannel(double progress);
+	void updateProgressAndStatusStarted();
+	void updateProgressAndStatusFinished();
+	static const uint64_t SAVE_CHUNK_SIZE = 64;
 
 protected:
 	QString filename;
@@ -138,14 +149,25 @@ public:
 	ImageFileSavePNG(QString filename, cImage *image, ImageConfig imageConfig)
 			: ImageFileSave(filename, image, imageConfig)
 	{
+		hasAppendAlphaCustom = false;
+		appendAlphaCustom = false;
 	}
-	void SaveImage();
-	QString getJobName() { return tr("Saving %1").arg("PNG"); }
-	static void SavePNG(
+	void SetAppendAlphaCustom(bool _appendAlphaCustom)
+	{
+		appendAlphaCustom = _appendAlphaCustom;
+		hasAppendAlphaCustom = true;
+	}
+	void SaveImage() override;
+	QString getJobName() override { return tr("Saving %1").arg("PNG"); }
+	void SavePNG(
 		QString filename, cImage *image, structSaveImageChannel imageChannel, bool appendAlpha = false);
 	static void SavePNG16(QString filename, int width, int height, sRGB16 *image16);
 	static void SaveFromTilesPNG16(const char *filename, int width, int height, int tiles);
 	static bool SavePNGQtBlackAndWhite(QString filename, unsigned char *image, int width, int height);
+
+private:
+	bool hasAppendAlphaCustom;
+	bool appendAlphaCustom;
 };
 
 class ImageFileSaveJPG : public ImageFileSave
@@ -156,8 +178,8 @@ public:
 			: ImageFileSave(filename, image, imageConfig)
 	{
 	}
-	void SaveImage();
-	QString getJobName() { return tr("Saving %1").arg("JPG"); }
+	void SaveImage() override;
+	QString getJobName() override { return tr("Saving %1").arg("JPG"); }
 	static bool SaveJPEGQt(
 		QString filename, unsigned char *image, int width, int height, int quality);
 	static bool SaveJPEGQtGreyscale(
@@ -173,9 +195,9 @@ public:
 			: ImageFileSave(filename, image, imageConfig)
 	{
 	}
-	void SaveImage();
-	QString getJobName() { return tr("Saving %1").arg("TIFF"); }
-	static bool SaveTIFF(
+	void SaveImage() override;
+	QString getJobName() override { return tr("Saving %1").arg("TIFF"); }
+	bool SaveTIFF(
 		QString filename, cImage *image, structSaveImageChannel imageChannel, bool appendAlpha = false);
 };
 #endif /* USE_TIFF */
@@ -189,12 +211,11 @@ public:
 			: ImageFileSave(filename, image, imageConfig)
 	{
 	}
-	void SaveImage();
-	QString getJobName() { return tr("Saving %1").arg("EXR"); }
-	static void SaveEXR(QString filename,
-		cImage *image,
+	void SaveImage() override;
+	QString getJobName() override { return tr("Saving %1").arg("EXR"); }
+	void SaveEXR(QString filename, cImage *image,
 		QMap<enumImageContentType, structSaveImageChannel> imageConfig);
 };
 #endif /* USE_EXR */
 
-#endif /* FILE_IMAGE_H_ */
+#endif /* MANDELBULBER2_SRC_FILE_IMAGE_HPP_ */
