@@ -1,67 +1,98 @@
 /**
- * Mandelbulber v2, a 3D fractal generator
+ * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
+ *                                             ,B" ]L,,p%%%,,,§;, "K
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
+ * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
+ *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
+ * Mandelbulber is free software:     §R.ß~-Q/M=,=5"v"]=Qf,'§"M= =,M.§ Rz]M"Kw
+ * you can redistribute it and/or     §w "xDY.J ' -"m=====WeC=\ ""%""y=%"]"" §
+ * modify it under the terms of the    "§M=M =D=4"N #"%==A%p M§ M6  R' #"=~.4M
+ * GNU General Public License as        §W =, ][T"]C  §  § '§ e===~ U  !§[Z ]N
+ * published by the                    4M",,Jm=,"=e~  §  §  j]]""N  BmM"py=ßM
+ * Free Software Foundation,          ]§ T,M=& 'YmMMpM9MMM%=w=,,=MT]M m§;'§,
+ * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
+ * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
+ * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
+ * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
+ * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
+ * but WITHOUT ANY WARRANTY;                            .'''
+ * without even the implied warranty
+ * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- * cParamRender class - container for scene parameters
+ * See the GNU General Public License for more details.
+ * You should have received a copy of the GNU General Public License
+ * along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2014 Krzysztof Marczak
- *
- * This file is part of Mandelbulber.
- *
- * Mandelbulber is free software: you can redistribute it and/or modify it under the terms of the
- * GNU General Public License as published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * Mandelbulber is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * See the GNU General Public License for more details. You should have received a copy of the GNU
- * General Public License along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
+ * ###########################################################################
  *
  * Authors: Krzysztof Marczak (buddhi1980@gmail.com)
+ *
+ * sParamRender struct - container for scene parameters
  */
 
 #include "fractparams.hpp"
 
-cParamRender::cParamRender(const cParameterContainer *container, QVector<cObjectData> *objectData) :
-		primitives(container, objectData)
+#include "object_data.hpp"
+#include "parameters.hpp"
+
+sParamRender::sParamRender(const cParameterContainer *container, QVector<cObjectData> *objectData)
+		: primitives(container, objectData)
 {
+	antialiasingEnabled = container->Get<bool>("antialiasing_enabled");
+	antialiasingSize = container->Get<int>("antialiasing_size");
 	ambientOcclusion = container->Get<double>("ambient_occlusion");
 	ambientOcclusionEnabled = container->Get<bool>("ambient_occlusion_enabled");
 	ambientOcclusionFastTune = container->Get<double>("ambient_occlusion_fast_tune");
-	ambientOcclusionMode = (params::enumAOMode) container->Get<int>("ambient_occlusion_mode");
+	ambientOcclusionMode = params::enumAOMode(container->Get<int>("ambient_occlusion_mode"));
 	ambientOcclusionQuality = container->Get<int>("ambient_occlusion_quality");
 	auxLightNumber = 4;
 	auxLightRandomNumber = container->Get<int>("random_lights_number");
 	auxLightRandomSeed = container->Get<int>("random_lights_random_seed");
 	auxLightRandomCenter = container->Get<CVector3>("random_lights_distribution_center");
 	auxLightRandomRadius = container->Get<double>("random_lights_distribution_radius");
-	auxLightRandomMaxDistanceFromFractal = container->Get<double>("random_lights_max_distance_from_fractal");
+	auxLightRandomMaxDistanceFromFractal =
+		container->Get<double>("random_lights_max_distance_from_fractal");
 	auxLightRandomIntensity = container->Get<double>("random_lights_intensity");
 	auxLightRandomEnabled = container->Get<bool>("random_lights_group");
 	auxLightVisibility = container->Get<double>("aux_light_visibility");
 	auxLightVisibilitySize = container->Get<double>("aux_light_visibility_size");
+	background3ColorsEnable = container->Get<bool>("background_3_colors_enable");
 	background_color1 = container->Get<sRGB>("background_color", 1);
 	background_color2 = container->Get<sRGB>("background_color", 2);
 	background_color3 = container->Get<sRGB>("background_color", 3);
 	background_brightness = container->Get<double>("background_brightness");
+	backgroundHScale = container->Get<double>("background_h_scale");
+	backgroundVScale = container->Get<double>("background_v_scale");
+	backgroundTextureOffsetX = container->Get<double>("background_texture_offset_x");
+	backgroundTextureOffsetY = container->Get<double>("background_texture_offset_y");
+	backgroundVScale = container->Get<double>("background_v_scale");
+	backgroundRotation = container->Get<CVector3>("background_rotation");
 	booleanOperatorsEnabled = container->Get<bool>("boolean_operators");
 	camera = container->Get<CVector3>("camera");
 	cameraDistanceToTarget = container->Get<double>("camera_distance_to_target");
 	constantDEThreshold = container->Get<bool>("constant_DE_threshold");
 	constantFactor = container->Get<double>("fractal_constant_factor");
 	DEFactor = container->Get<double>("DE_factor");
-	delta_DE_function = (fractal::enumDEFunctionType)container->Get<int>("delta_DE_function");
-	delta_DE_method = (fractal::enumDEMethod)container->Get<int>("delta_DE_method");
+	delta_DE_function = fractal::enumDEFunctionType(container->Get<int>("delta_DE_function"));
+	delta_DE_method = fractal::enumDEMethod(container->Get<int>("delta_DE_method"));
 	detailLevel = container->Get<double>("detail_level");
 	DEThresh = container->Get<double>("DE_thresh");
 	DOFEnabled = container->Get<bool>("DOF_enabled");
 	DOFFocus = container->Get<double>("DOF_focus");
 	DOFRadius = container->Get<double>("DOF_radius");
-	DOFHDRmode = container->Get<bool>("DOF_HDR");
+	DOFMaxRadius = container->Get<double>("DOF_max_radius");
+	DOFHDRMode = container->Get<bool>("DOF_HDR");
+	DOFMonteCarlo = container->Get<bool>("DOF_monte_carlo");
 	DOFNumberOfPasses = container->Get<int>("DOF_number_of_passes");
-  DOFBlurOpacity = container->Get<double>("DOF_blur_opacity");
+	DOFSamples = container->Get<int>("DOF_samples");
+	DOFMinSamples = container->Get<int>("DOF_min_samples");
+	DOFBlurOpacity = container->Get<double>("DOF_blur_opacity");
+	DOFMaxNoise = container->Get<double>("DOF_max_noise");
 	envMappingEnable = container->Get<bool>("env_mapping_enable");
-	fakeLightsEnabled = container->Get<double>("fake_lights_enabled");
+	fakeLightsColor = container->Get<sRGB>("fake_lights_color");
+	fakeLightsEnabled = container->Get<bool>("fake_lights_enabled");
 	fakeLightsIntensity = container->Get<double>("fake_lights_intensity");
 	fakeLightsVisibility = container->Get<double>("fake_lights_visibility");
 	fakeLightsVisibilitySize = container->Get<double>("fake_lights_visibility_size");
@@ -74,14 +105,19 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 	glowColor2 = container->Get<sRGB>("glow_color", 2);
 	glowEnabled = container->Get<bool>("glow_enabled");
 	glowIntensity = container->Get<double>("glow_intensity");
+	hdrBlurEnabled = container->Get<bool>("hdr_blur_enabled");
+	hdrBlurRadius = container->Get<double>("hdr_blur_radius");
+	hdrBlurIntensity = container->Get<double>("hdr_blur_intensity");
 	hybridFractalEnable = container->Get<bool>("hybrid_fractal_enable");
 	imageAdjustments.brightness = container->Get<double>("brightness");
 	imageAdjustments.contrast = container->Get<double>("contrast");
 	imageAdjustments.hdrEnabled = container->Get<bool>("hdr");
 	imageAdjustments.imageGamma = container->Get<double>("gamma");
+	imageAdjustments.saturation = container->Get<double>("saturation");
 	imageHeight = container->Get<int>("image_height");
 	imageWidth = container->Get<int>("image_width");
 	interiorMode = container->Get<bool>("interior_mode");
+	iterFogBrightnessBoost = container->Get<double>("iteration_fog_brightness_boost");
 	iterFogColor1Maxiter = container->Get<double>("iteration_fog_color_1_maxiter");
 	iterFogColor2Maxiter = container->Get<double>("iteration_fog_color_2_maxiter");
 	iterFogColour1 = container->Get<sRGB>("iteration_fog_color", 1);
@@ -90,7 +126,6 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 	iterFogEnabled = container->Get<bool>("iteration_fog_enable");
 	iterFogOpacity = container->Get<double>("iteration_fog_opacity");
 	iterFogOpacityTrim = container->Get<double>("iteration_fog_opacity_trim");
-	iterThreshMode = container->Get<bool>("iteration_threshold_mode");
 	legacyCoordinateSystem = container->Get<bool>("legacy_coordinate_system");
 	limitMax = container->Get<CVector3>("limit_max");
 	limitMin = container->Get<CVector3>("limit_min");
@@ -106,7 +141,7 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 	minN = container->Get<int>("minN");
 	N = container->Get<int>("N");
 	penetratingLights = container->Get<bool>("penetrating_lights");
-	perspectiveType = (params::enumPerspectiveType) container->Get<int>("perspective_type");
+	perspectiveType = params::enumPerspectiveType(container->Get<int>("perspective_type"));
 	raytracedReflections = container->Get<bool>("raytraced_reflections");
 	reflectionsMax = container->Get<int>("reflections_max");
 	repeatFrom = container->Get<int>("repeat_from");
@@ -116,13 +151,15 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 	slowShading = container->Get<bool>("slow_shading");
 	smoothness = container->Get<double>("smoothness");
 	SSAO_random_mode = container->Get<bool>("SSAO_random_mode");
+	stereoEyeDistance = container->Get<double>("stereo_eye_distance");
+	stereoInfiniteCorrection = container->Get<double>("stereo_infinite_correction");
 	sweetSpotHAngle = container->Get<double>("sweet_spot_horizontal_angle") / 180.0 * M_PI;
 	sweetSpotVAngle = container->Get<double>("sweet_spot_vertical_angle") / 180.0 * M_PI;
 	target = container->Get<CVector3>("target");
 	target = container->Get<CVector3>("target");
 	texturedBackground = container->Get<bool>("textured_background");
 	texturedBackgroundMapType =
-			(params::enumTextureMapType) container->Get<int>("textured_background_map_type");
+		params::enumTextureMapType(container->Get<int>("textured_background_map_type"));
 	topVector = container->Get<CVector3>("camera_top");
 	useDefaultBailout = container->Get<bool>("use_default_bailout");
 	viewAngle = container->Get<CVector3>("camera_rotation");
@@ -136,8 +173,10 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 	volFogDensity = container->Get<double>("volumetric_fog_density");
 	volFogDistanceFactor = container->Get<double>("volumetric_fog_distance_factor");
 	volFogEnabled = container->Get<bool>("volumetric_fog_enabled");
-	volumetricLightEnabled[0] = container->Get<double>("main_light_volumetric_enabled");
+	volumetricLightEnabled[0] = container->Get<bool>("main_light_volumetric_enabled");
 	volumetricLightIntensity[0] = container->Get<double>("main_light_volumetric_intensity");
+	volumetricLightDEFactor = container->Get<double>("volumetric_light_DE_Factor");
+	mRotBackgroundRotation.SetRotation(backgroundRotation * M_PI / 180.0);
 
 	for (int i = 0; i < 4; ++i)
 	{
@@ -150,7 +189,7 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 	for (int i = 1; i <= 4; i++)
 	{
 		volumetricLightIntensity[i] = container->Get<double>("aux_light_volumetric_intensity", i);
-		volumetricLightEnabled[i] = container->Get<double>("aux_light_volumetric_enabled", i);
+		volumetricLightEnabled[i] = container->Get<bool>("aux_light_volumetric_enabled", i);
 	}
 
 	volumetricLightAnyEnabled = false;
@@ -161,8 +200,8 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 
 	for (int i = 0; i < NUMBER_OF_FRACTALS - 1; i++)
 	{
-		booleanOperator[i] = (params::enumBooleanOperator) container->Get<int>("boolean_operator",
-																																					 i + 1);
+		booleanOperator[i] =
+			params::enumBooleanOperator(container->Get<int>("boolean_operator", i + 1));
 	}
 
 	for (int i = 0; i < NUMBER_OF_FRACTALS; i++)
@@ -174,7 +213,7 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 		mRotFormulaRotation[i].SetRotation2(formulaRotation[i] * (M_PI / 180.0));
 		formulaMaterialId[i] = container->Get<int>("formula_material_id", i + 1);
 
-		if(objectData)
+		if (objectData)
 		{
 			cObjectData oneObjectData;
 			oneObjectData.position = formulaPosition[i];
@@ -186,14 +225,14 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 		}
 	}
 
-	if(!booleanOperatorsEnabled && objectData)
+	if (!booleanOperatorsEnabled && objectData)
 	{
 		formulaMaterialId[0] = container->Get<int>("formula_material_id");
 		(*objectData)[0].materialId = formulaMaterialId[0];
 	}
 
-	common.fakeLightsMaxIter = container->Get<double>("fake_lights_max_iter");
-	common.fakeLightsMinIter = container->Get<double>("fake_lights_min_iter");
+	common.fakeLightsMaxIter = container->Get<int>("fake_lights_max_iter");
+	common.fakeLightsMinIter = container->Get<int>("fake_lights_min_iter");
 	common.fakeLightsOrbitTrap = container->Get<CVector3>("fake_lights_orbit_trap");
 	common.foldings.boxEnable = container->Get<bool>("box_folding");
 	common.foldings.boxLimit = container->Get<double>("box_folding_limit");
@@ -205,7 +244,8 @@ cParamRender::cParamRender(const cParameterContainer *container, QVector<cObject
 	common.fractalRotation = container->Get<CVector3>("fractal_rotation");
 	common.mRotFractalRotation.SetRotation2(common.fractalRotation / 180.0 * M_PI);
 	common.repeat = container->Get<CVector3>("repeat");
+	common.iterThreshMode = iterThreshMode = container->Get<bool>("iteration_threshold_mode");
+	common.linearDEOffset = container->Get<double>("linear_DE_offset");
 
-	//formula = Get<int>("tile_number");
+	// formula = Get<int>("tile_number");
 }
-
