@@ -1,49 +1,64 @@
 /**
- * Mandelbulber v2, a 3D fractal generator
+ * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
+ *                                             ,B" ]L,,p%%%,,,§;, "K
+ * Copyright (C) 2015-18 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
+ * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
+ *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
+ * Mandelbulber is free software:     §R.ß~-Q/M=,=5"v"]=Qf,'§"M= =,M.§ Rz]M"Kw
+ * you can redistribute it and/or     §w "xDY.J ' -"m=====WeC=\ ""%""y=%"]"" §
+ * modify it under the terms of the    "§M=M =D=4"N #"%==A%p M§ M6  R' #"=~.4M
+ * GNU General Public License as        §W =, ][T"]C  §  § '§ e===~ U  !§[Z ]N
+ * published by the                    4M",,Jm=,"=e~  §  §  j]]""N  BmM"py=ßM
+ * Free Software Foundation,          ]§ T,M=& 'YmMMpM9MMM%=w=,,=MT]M m§;'§,
+ * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
+ * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
+ * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
+ * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
+ * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
+ * but WITHOUT ANY WARRANTY;                            .'''
+ * without even the implied warranty
+ * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *
+ * See the GNU General Public License for more details.
+ * You should have received a copy of the GNU General Public License
+ * along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ###########################################################################
+ *
+ * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Sebastian Jennen (jenzebas@gmail.com)
  *
  * cHeadless - class to handle CLI instructions without GUI manipulation
- *
- * Copyright (C) 2014 Krzysztof Marczak
- *
- * This file is part of Mandelbulber.
- *
- * Mandelbulber is free software: you can redistribute it and/or modify it under the terms of the
- * GNU General Public License as published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * Mandelbulber is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * See the GNU General Public License for more details. You should have received a copy of the GNU
- * General Public License along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
- *
- * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Sebastian Jennen
  */
 
 #include "headless.h"
-#include "initparameters.hpp"
-#include "fractal_container.hpp"
-#include "queue.hpp"
-#include "cimage.hpp"
-#include "files.h"
-#include "file_image.hpp"
-#include "render_job.hpp"
-#include "global_data.hpp"
-#include "interface.hpp"
-#include "error_message.hpp"
+
 #include "animation_flight.hpp"
 #include "animation_keyframes.hpp"
+#include "cimage.hpp"
+#include "error_message.hpp"
+#include "file_image.hpp"
+#include "files.h"
+#include "fractal_container.hpp"
+#include "global_data.hpp"
+#include "initparameters.hpp"
+#include "interface.hpp"
+#include "mesh_export.hpp"
+#include "opencl_engine_render_fractal.h"
+#include "opencl_engine_render_ssao.h"
+#include "opencl_global.h"
+#include "queue.hpp"
+#include "render_job.hpp"
+#include "rendering_configuration.hpp"
 #include "voxel_export.hpp"
 
-cHeadless::cHeadless() :
-		QObject()
+cHeadless::cHeadless() : QObject()
 {
-
 }
 
 cHeadless::~cHeadless()
 {
-
 }
 
 void cHeadless::RenderStillImage(QString filename, QString imageFileFormat)
@@ -52,13 +67,20 @@ void cHeadless::RenderStillImage(QString filename, QString imageFileFormat)
 	cRenderJob *renderJob = new cRenderJob(gPar, gParFractal, image, &gMainInterface->stopRequest);
 
 	QObject::connect(renderJob,
-									 SIGNAL(updateProgressAndStatus(const QString&, const QString&, double)),
-									 this,
-									 SLOT(slotUpdateProgressAndStatus(const QString&, const QString&, double)));
-	QObject::connect(renderJob,
-									 SIGNAL(updateStatistics(cStatistics)),
-									 this,
-									 SLOT(slotUpdateStatistics(cStatistics)));
+		SIGNAL(updateProgressAndStatus(const QString &, const QString &, double)), this,
+		SLOT(slotUpdateProgressAndStatus(const QString &, const QString &, double)));
+	QObject::connect(renderJob, SIGNAL(updateStatistics(cStatistics)), this,
+		SLOT(slotUpdateStatistics(cStatistics)));
+
+#ifdef USE_OPENCL
+	// connect signal for progress bar update
+	connect(gOpenCl->openClEngineRenderFractal,
+		SIGNAL(updateProgressAndStatus(const QString &, const QString &, double)), this,
+		SLOT(slotUpdateProgressAndStatus(const QString &, const QString &, double)));
+	connect(gOpenCl->openClEngineRenderSSAO,
+		SIGNAL(updateProgressAndStatus(const QString &, const QString &, double)), this,
+		SLOT(slotUpdateProgressAndStatus(const QString &, const QString &, double)));
+#endif
 
 	cRenderingConfiguration config;
 	config.DisableRefresh();
@@ -68,8 +90,7 @@ void cHeadless::RenderStillImage(QString filename, QString imageFileFormat)
 	renderJob->Init(cRenderJob::still, config);
 	renderJob->Execute();
 
-	QFileInfo fi(filename);
-	filename = fi.path() + QDir::separator() + fi.baseName();
+	QString filenameWithoutExtension = ImageFileSave::ImageNameWithoutExtension(filename);
 
 	QString ext;
 	if (imageFileFormat == "png16" || imageFileFormat == "png16alpha")
@@ -77,18 +98,22 @@ void cHeadless::RenderStillImage(QString filename, QString imageFileFormat)
 		ext = ".png";
 		ImageFileSave::structSaveImageChannel saveImageChannel(
 			ImageFileSave::IMAGE_CONTENT_COLOR, ImageFileSave::IMAGE_CHANNEL_QUALITY_16, "");
+		ImageFileSave::ImageConfig imageConfig;
+		imageConfig.insert(ImageFileSave::IMAGE_CONTENT_COLOR, saveImageChannel);
 		bool appendAlpha = (imageFileFormat == "png16alpha");
-		ImageFileSavePNG::SavePNG(filename + ext, image, saveImageChannel, appendAlpha);
+		ImageFileSavePNG imageSaver(filenameWithoutExtension, image, imageConfig);
+		imageSaver.SetAppendAlphaCustom(appendAlpha);
+		imageSaver.SaveImage();
 	}
 	else
 	{
 		ext = "." + imageFileFormat;
 		ImageFileSave::enumImageFileType imageFileType = ImageFileSave::ImageFileType(imageFileFormat);
-		SaveImage(filename + ext, imageFileType, image, this);
+		SaveImage(filenameWithoutExtension + ext, imageFileType, image, this);
 	}
 
 	QTextStream out(stdout);
-	out << "Image saved to: " << filename << ext << "\n";
+	out << tr("Image saved to: %1\n").arg(filenameWithoutExtension + ext);
 
 	delete renderJob;
 	delete image;
@@ -104,11 +129,11 @@ void cHeadless::RenderQueue()
 		do
 		{
 			gApplication->processEvents();
-			Wait(100); // TODO substitute with proper handling
+			Wait(100);
 		} while (gQueue->GetQueueSize() > 0);
 
-		cErrorMessage::showMessage("Queue is empty. Waiting for something to do.",
-															 cErrorMessage::infoMessage);
+		cErrorMessage::showMessage(
+			"Queue is empty. Waiting for something to do.", cErrorMessage::infoMessage);
 		do
 		{
 			gApplication->processEvents();
@@ -119,11 +144,11 @@ void cHeadless::RenderQueue()
 	}
 }
 
-void cHeadless::RenderVoxel()
+void cHeadless::RenderVoxel(QString voxelFormat)
 {
 	CVector3 limitMin;
 	CVector3 limitMax;
-	if(gPar->Get<bool>("voxel_custom_limit_enabled"))
+	if (gPar->Get<bool>("voxel_custom_limit_enabled"))
 	{
 		limitMin = gPar->Get<CVector3>("voxel_limit_min");
 		limitMax = gPar->Get<CVector3>("voxel_limit_max");
@@ -134,74 +159,80 @@ void cHeadless::RenderVoxel()
 		limitMax = gPar->Get<CVector3>("limit_max");
 	}
 	int maxIter = gPar->Get<int>("voxel_max_iter");
-	QString folderString = gPar->Get<QString>("voxel_image_path");
 	int samplesX = gPar->Get<int>("voxel_samples_x");
 	int samplesY = gPar->Get<int>("voxel_samples_y");
 	int samplesZ = gPar->Get<int>("voxel_samples_z");
-	QDir folder(folderString);
 
-	cVoxelExport *voxelExport =
-		new cVoxelExport(samplesX, samplesY, samplesZ, limitMin, limitMax, folder, maxIter);
-	QObject::connect(voxelExport,
-		SIGNAL(updateProgressAndStatus(const QString&, const QString&, double)),
-		this,
-		SLOT(slotUpdateProgressAndStatus(const QString&, const QString&, double)));
-	voxelExport->ProcessVolume();
-	delete voxelExport;
+	if (voxelFormat == "slice")
+	{
+		QString folderString = gPar->Get<QString>("voxel_image_path");
+		QDir folder(folderString);
+		cVoxelExport *voxelExport =
+			new cVoxelExport(samplesX, samplesY, samplesZ, limitMin, limitMax, folder, maxIter);
+		QObject::connect(voxelExport,
+			SIGNAL(updateProgressAndStatus(const QString &, const QString &, double)), this,
+			SLOT(slotUpdateProgressAndStatus(const QString &, const QString &, double)));
+		voxelExport->ProcessVolume();
+		delete voxelExport;
+	}
+	else if (voxelFormat == "ply")
+	{
+		QString fileString = gPar->Get<QString>("mesh_output_filename");
+		QList<MeshFileSave::enumMeshContentType> meshContent({MeshFileSave::MESH_CONTENT_GEOMETRY});
+		if (gPar->Get<bool>("mesh_color")) meshContent << MeshFileSave::MESH_CONTENT_COLOR;
+		MeshFileSave::structSaveMeshConfig meshConfig(MeshFileSave::MESH_FILE_TYPE_PLY, meshContent,
+			(MeshFileSave::enumMeshFileModeType)gPar->Get<int>("mesh_file_mode"));
+
+		cMeshExport *meshExport = new cMeshExport(
+			samplesX, samplesY, samplesZ, limitMin, limitMax, fileString, maxIter, meshConfig);
+		QObject::connect(meshExport,
+			SIGNAL(updateProgressAndStatus(const QString &, const QString &, double)), this,
+			SLOT(slotUpdateProgressAndStatus(const QString &, const QString &, double)));
+		meshExport->ProcessVolume();
+		delete meshExport;
+	}
 	emit finished();
 }
 
-void cHeadless::RenderFlightAnimation()
+void cHeadless::RenderFlightAnimation() const
 {
 	cImage *image = new cImage(gPar->Get<int>("image_width"), gPar->Get<int>("image_height"));
-	gFlightAnimation = new cFlightAnimation(gMainInterface,
-																					gAnimFrames,
-																					image,
-																					NULL,
-																					gPar,
-																					gParFractal,
-																					NULL);
-	QObject::connect(gFlightAnimation,
-									 SIGNAL(updateProgressAndStatus(const QString&, const QString&, double, cProgressText::enumProgressType)),
-									 this,
-									 SLOT(slotUpdateProgressAndStatus(const QString&, const QString&, double, cProgressText::enumProgressType)));
-	// QObject::connect(gFlightAnimation, SIGNAL(updateProgressHide(cProgressText::enumProgressType)), unused
+	gFlightAnimation =
+		new cFlightAnimation(gMainInterface, gAnimFrames, image, nullptr, gPar, gParFractal, nullptr);
+	QObject::connect(gFlightAnimation, SIGNAL(updateProgressAndStatus(const QString &,
+																			 const QString &, double, cProgressText::enumProgressType)),
+		this, SLOT(slotUpdateProgressAndStatus(
+						const QString &, const QString &, double, cProgressText::enumProgressType)));
+	// QObject::connect(gFlightAnimation, SIGNAL(updateProgressHide(cProgressText::enumProgressType)),
+	// unused
 	//								 this, SLOT(slotUpdateProgressHide(cProgressText::enumProgressType)));
-	QObject::connect(gFlightAnimation,
-									 SIGNAL(updateStatistics(cStatistics)),
-									 this,
-									 SLOT(slotUpdateStatistics(cStatistics)));
+	QObject::connect(gFlightAnimation, SIGNAL(updateStatistics(cStatistics)), this,
+		SLOT(slotUpdateStatistics(cStatistics)));
 	gFlightAnimation->slotRenderFlight();
 	delete image;
 	delete gFlightAnimation;
-	gFlightAnimation = NULL;
+	gFlightAnimation = nullptr;
 	return;
 }
 
-void cHeadless::RenderKeyframeAnimation()
+void cHeadless::RenderKeyframeAnimation() const
 {
 	cImage *image = new cImage(gPar->Get<int>("image_width"), gPar->Get<int>("image_height"));
-	gKeyframeAnimation = new cKeyframeAnimation(gMainInterface,
-																							gKeyframes,
-																							image,
-																							NULL,
-																							gPar,
-																							gParFractal,
-																							NULL);
-	QObject::connect(gKeyframeAnimation,
-									 SIGNAL(updateProgressAndStatus(const QString&, const QString&, double, cProgressText::enumProgressType)),
-									 this,
-									 SLOT(slotUpdateProgressAndStatus(const QString&, const QString&, double, cProgressText::enumProgressType)));
-	// QObject::connect(gKeyframeAnimation, SIGNAL(updateProgressHide(cProgressText::enumProgressType)), unused
+	gKeyframeAnimation =
+		new cKeyframeAnimation(gMainInterface, gKeyframes, image, nullptr, gPar, gParFractal, nullptr);
+	QObject::connect(gKeyframeAnimation, SIGNAL(updateProgressAndStatus(const QString &,
+																				 const QString &, double, cProgressText::enumProgressType)),
+		this, SLOT(slotUpdateProgressAndStatus(
+						const QString &, const QString &, double, cProgressText::enumProgressType)));
+	// QObject::connect(gKeyframeAnimation,
+	// SIGNAL(updateProgressHide(cProgressText::enumProgressType)), unused
 	// 								 this, SLOT(slotUpdateProgressHide(cProgressText::enumProgressType)));
-	QObject::connect(gKeyframeAnimation,
-									 SIGNAL(updateStatistics(cStatistics)),
-									 this,
-									 SLOT(slotUpdateStatistics(cStatistics)));
+	QObject::connect(gKeyframeAnimation, SIGNAL(updateStatistics(cStatistics)), this,
+		SLOT(slotUpdateStatistics(cStatistics)));
 	gKeyframeAnimation->slotRenderKeyframes();
 	delete image;
 	delete gKeyframeAnimation;
-	gKeyframeAnimation = NULL;
+	gKeyframeAnimation = nullptr;
 	return;
 }
 
@@ -211,13 +242,10 @@ void cHeadless::slotNetRender()
 	cImage *image = new cImage(gPar->Get<int>("image_width"), gPar->Get<int>("image_height"));
 	cRenderJob *renderJob = new cRenderJob(gPar, gParFractal, image, &gMainInterface->stopRequest);
 	QObject::connect(renderJob,
-									 SIGNAL(updateProgressAndStatus(const QString&, const QString&, double)),
-									 this,
-									 SLOT(slotUpdateProgressAndStatus(const QString&, const QString&, double)));
-	QObject::connect(renderJob,
-									 SIGNAL(updateStatistics(cStatistics)),
-									 this,
-									 SLOT(slotUpdateStatistics(cStatistics)));
+		SIGNAL(updateProgressAndStatus(const QString &, const QString &, double)), this,
+		SLOT(slotUpdateProgressAndStatus(const QString &, const QString &, double)));
+	QObject::connect(renderJob, SIGNAL(updateStatistics(cStatistics)), this,
+		SLOT(slotUpdateStatistics(cStatistics)));
 
 	cRenderingConfiguration config;
 	config.DisableRefresh();
@@ -233,7 +261,7 @@ void cHeadless::slotNetRender()
 }
 
 void cHeadless::slotUpdateProgressAndStatus(const QString &text, const QString &progressText,
-		double progress, cProgressText::enumProgressType progressType)
+	double progress, cProgressText::enumProgressType progressType)
 {
 	static bool firstCallProgressUpdate = true;
 	if (firstCallProgressUpdate)
@@ -249,18 +277,12 @@ void cHeadless::slotUpdateProgressAndStatus(const QString &text, const QString &
 	{
 		switch (progressType)
 		{
-			case cProgressText::progress_IMAGE:
-				MoveCursor(0, -1);
-				break;
-			case cProgressText::progress_ANIMATION:
-				MoveCursor(0, -2);
-				break;
-			case cProgressText::progress_QUEUE:
-				MoveCursor(0, -3);
-				break;
+			case cProgressText::progress_IMAGE: MoveCursor(0, -1); break;
+			case cProgressText::progress_ANIMATION: MoveCursor(0, -2); break;
+			case cProgressText::progress_QUEUE: MoveCursor(0, -3); break;
 		}
 
-		// not enough space to display info in animation bar
+		// not enough space to display information in animation bar
 		QString displayText = (progressType == cProgressText::progress_ANIMATION ? "" : text);
 
 		RenderingProgressOutput(displayText, progressText, progress);
@@ -270,20 +292,27 @@ void cHeadless::slotUpdateProgressAndStatus(const QString &text, const QString &
 			case cProgressText::progress_QUEUE:
 				MoveCursor(0, 1);
 				EraseLine();
+				MoveCursor(0, 1);
+				EraseLine();
+				MoveCursor(0, 1);
+				EraseLine();
+				break;
 			case cProgressText::progress_ANIMATION:
 				MoveCursor(0, 1);
 				EraseLine();
-			case cProgressText::progress_IMAGE:
 				MoveCursor(0, 1);
+				EraseLine();
+				break;
+			case cProgressText::progress_IMAGE: MoveCursor(0, 1); break;
 		}
 		if (systemData.statsOnCLI) MoveCursor(0, 2);
 	}
 }
 
-void cHeadless::slotUpdateStatistics(cStatistics stat)
+void cHeadless::slotUpdateStatistics(const cStatistics &stat) const
 {
 	if (!systemData.statsOnCLI) return;
-	/*ui->label_histogram_de->SetBarcolor(QColor(0, 255, 0));
+	/*ui->label_histogram_de->SetBarColor(QColor(0, 255, 0));
 	 ui->label_histogram_de->UpdateHistogram(stat.histogramStepCount);
 	 ui->label_histogram_iter->UpdateHistogram(stat.histogramIterations);
 	 */
@@ -292,35 +321,31 @@ void cHeadless::slotUpdateStatistics(cStatistics stat)
 	QString statsText = "";
 	statsText += tr("Total number of iters").leftJustified(25, ' ') + ": ";
 	statsText += colorize(QString::number(stat.GetTotalNumberOfIterations()).rightJustified(12, ' '),
-												ansiBlue,
-												noExplicitColor,
-												true) + ", ";
+								 ansiBlue, noExplicitColor, true)
+							 + ", ";
 	statsText += tr("Number of iters / pixel").leftJustified(25, ' ') + ": ";
-	statsText += colorize(QString::number(stat.GetNumberOfIterationsPerPixel()).rightJustified(12,
-																																														 ' '),
-												ansiBlue,
-												noExplicitColor,
-												true) + "\n";
+	statsText +=
+		colorize(QString::number(stat.GetNumberOfIterationsPerPixel()).rightJustified(12, ' '),
+			ansiBlue, noExplicitColor, true)
+		+ "\n";
 	statsText += tr("Number of iters / second").leftJustified(25, ' ') + ": ";
-	statsText += colorize(QString::number(stat.GetNumberOfIterationsPerSecond()).rightJustified(12,
-																																															' '),
-												ansiBlue,
-												noExplicitColor,
-												true) + ", ";
+	statsText +=
+		colorize(QString::number(stat.GetNumberOfIterationsPerSecond()).rightJustified(12, ' '),
+			ansiBlue, noExplicitColor, true)
+		+ ", ";
 	statsText += tr("Percentage of wrong DE").leftJustified(25, ' ') + ": ";
 	statsText += colorize(QString::number(stat.GetMissedDEPercentage()).rightJustified(12, ' '),
-												ansiBlue,
-												noExplicitColor,
-												true) + "\n";
+								 ansiBlue, noExplicitColor, true)
+							 + "\n";
 	out << statsText;
 	out.flush();
 }
 
-void cHeadless::RenderingProgressOutput(const QString &header, const QString &progressTxt,
-		double percentDone)
+void cHeadless::RenderingProgressOutput(
+	const QString &header, const QString &progressTxt, double percentDone)
 {
 	QTextStream out(stdout);
-	QString formatedText = formatLine(progressTxt) + " ";
+	QString formattedText = formatLine(progressTxt) + " ";
 	QString useHeader = header;
 	QString text;
 	if (systemData.terminalWidth > 0)
@@ -330,7 +355,7 @@ void cHeadless::RenderingProgressOutput(const QString &header, const QString &pr
 		int intProgress = freeWidth * percentDone;
 		text = "\r";
 		text += colorize(useHeader, ansiYellow, noExplicitColor, true);
-		text += formatedText;
+		text += formattedText;
 		text += colorize("[", ansiBlue, noExplicitColor, true);
 		text += colorize(QString(intProgress, '#'), ansiMagenta, noExplicitColor, true);
 		text += QString(freeWidth - intProgress, ' ');
@@ -345,17 +370,17 @@ void cHeadless::RenderingProgressOutput(const QString &header, const QString &pr
 	out.flush();
 }
 
-QString cHeadless::colorize(QString text, ansiColor foregroundcolor, ansiColor backgroundColor,
-		bool bold)
+QString cHeadless::colorize(
+	QString text, ansiColor foregroundColor, ansiColor backgroundColor, bool bold)
 {
-	// more info on ANSI escape codes here: https://en.wikipedia.org/wiki/ANSI_escape_code
-#ifdef WIN32 /* WINDOWS */
+// more information on ANSI escape codes here: https://en.wikipedia.org/wiki/ANSI_escape_code
+#ifdef _WIN32 /* WINDOWS */
 	return text;
 #else
 	if (!systemData.useColor) return text;
 
 	QStringList ansiSequence;
-	if (foregroundcolor != noExplicitColor) ansiSequence << QString::number(foregroundcolor + 30);
+	if (foregroundColor != noExplicitColor) ansiSequence << QString::number(foregroundColor + 30);
 	if (backgroundColor != noExplicitColor) ansiSequence << QString::number(backgroundColor + 40);
 	if (bold) ansiSequence << "1";
 
@@ -370,25 +395,31 @@ QString cHeadless::colorize(QString text, ansiColor foregroundcolor, ansiColor b
 #endif
 }
 
-QString cHeadless::formatLine(const QString& text)
+QString cHeadless::formatLine(const QString &text)
 {
-#ifdef WIN32 /* WINDOWS */
+#ifdef _WIN32 /* WINDOWS */
 	return text;
 #else
 	if (!systemData.useColor) return text;
 	QList<QRegularExpression> reType;
-	reType.append(QRegularExpression("^(Done )(.*?)(, )(elapsed: )(.*?)(, )(estimated to end: )(.*)"));
+	reType.append(
+		QRegularExpression("^(Done )(.*?)(, )(elapsed: )(.*?)(, )(estimated to end: )(.*)"));
 	reType.append(QRegularExpression("^(Gotowe )(.*?)(, )(upłynęło: )(.*?)(, )(do końca: )(.*)"));
-	reType.append(QRegularExpression("^(Fortschritt )(.*?)(, )(vergangen: )(.*?)(, )(voraussichtlich noch: )(.*)"));
+	reType.append(QRegularExpression(
+		"^(Fortschritt )(.*?)(, )(vergangen: )(.*?)(, )(voraussichtlich noch: )(.*)"));
 
 	reType.append(QRegularExpression("^(.*?)( Done)(, )(total time: )(.*)"));
 	reType.append(QRegularExpression("^(.*?)( gotowe)(, )(całkowity czas: )(.*)"));
 	reType.append(QRegularExpression("^(.*?)( Fertig)(, )(Gesamtzeit: )(.*)"));
 
 	// animation
-	reType.append(QRegularExpression("^(Frame .*? of .*? Done )(.*?)(, )(elapsed: )(.*?)(, )(estimated to end: )(.*)"));
-	reType.append(QRegularExpression("^(Klatka .*? z .*? Gotowe )(.*?)(, )(upłynęło: )(.*?)(, )(do końca: )(.*)"));
-	reType.append(QRegularExpression("^(Frame .*? von .*? Fortschritt )(.*?)(, )(vergangen: )(.*?)(, )(voraussichtlich noch: )(.*)"));
+	reType.append(QRegularExpression(
+		"^(Frame .*? of .*? Done )(.*?)(, )(elapsed: )(.*?)(, )(estimated to end: )(.*)"));
+	reType.append(QRegularExpression(
+		"^(Klatka .*? z .*? Gotowe )(.*?)(, )(upłynęło: )(.*?)(, )(do końca: )(.*)"));
+	reType.append(
+		QRegularExpression("^(Frame .*? von .*? Fortschritt )(.*?)(, )(vergangen: )(.*?)(, "
+											 ")(voraussichtlich noch: )(.*)"));
 
 	QRegularExpressionMatch matchType;
 	for (int i = 0; i < reType.size(); i++)
@@ -433,7 +464,7 @@ bool cHeadless::ConfirmMessage(QString message)
 
 void cHeadless::EraseLine()
 {
-#ifdef WIN32 /* WINDOWS */
+#ifdef _WIN32 /* WINDOWS */
 	return;
 #else
 	QTextStream out(stdout);
@@ -444,7 +475,7 @@ void cHeadless::EraseLine()
 
 void cHeadless::MoveCursor(int leftRight, int downUp)
 {
-#ifdef WIN32 /* WINDOWS */
+#ifdef _WIN32 /* WINDOWS */
 	return;
 #else
 	QTextStream out(stdout);
