@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2015-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -34,12 +34,12 @@
  * Handles the 2D matrix of the list of parameters / list of frames
  * and exposes functions to modify this matrix. This functionality is
  * derived from the cAnimationFrames class. Additionally this class
- * interpolates subframes with the help of the cMorph class.
+ * interpolates sub-frames with the help of the cMorph class.
  */
 
 #include "keyframes.hpp"
 
-cKeyframes *gKeyframes = NULL;
+cKeyframes *gKeyframes = nullptr;
 
 cKeyframes::cKeyframes() : cAnimationFrames()
 {
@@ -71,12 +71,14 @@ cKeyframes &cKeyframes::operator=(const cKeyframes &source)
 	this->frames = source.frames;
 	this->listOfParameters = source.listOfParameters;
 	this->framesPerKeyframe = source.framesPerKeyframe;
-
+	this->audioTracks = source.audioTracks;
 	return *this;
 }
 
-cAnimationFrames::sAnimationFrame cKeyframes::GetInterpolatedFrame(int index)
+cAnimationFrames::sAnimationFrame cKeyframes::GetInterpolatedFrame(
+	int index, cParameterContainer *params, cFractalContainer *fractal)
 {
+	Q_UNUSED(fractal);
 	int keyframe = index / framesPerKeyframe;
 	int subIndex = index % framesPerKeyframe;
 
@@ -87,9 +89,12 @@ cAnimationFrames::sAnimationFrame cKeyframes::GetInterpolatedFrame(int index)
 	//}
 
 	sAnimationFrame interpolated;
-	QList<QString> parameterList = frames.at(keyframe).parameters.GetListOfParameters();
-	for (int i = 0; i < parameterList.size(); i++)
+
+	for (int i = 0; i < listOfParameters.size(); i++)
 	{
+		QString fullParameterName =
+			listOfParameters[i].containerName + "_" + listOfParameters[i].parameterName;
+
 		// prepare interpolator
 		if (morph.size() <= i)
 		{
@@ -99,12 +104,18 @@ cAnimationFrames::sAnimationFrame cKeyframes::GetInterpolatedFrame(int index)
 		{
 			if (morph[i]->findInMorph(k) == -1)
 			{
-				morph[i]->AddData(k, frames.at(k).parameters.GetAsOneParameter(parameterList.at(i)));
+				morph[i]->AddData(k, frames.at(k).parameters.GetAsOneParameter(fullParameterName));
 			}
 		}
-		// interpolate each parameter and write back
-		interpolated.parameters.AddParamFromOneParameter(
-			parameterList.at(i), morph[i]->Interpolate(keyframe, 1.0 * subIndex / framesPerKeyframe));
+		// interpolate each parameter
+		cOneParameter oneParameter =
+			morph[i]->Interpolate(keyframe, 1.0 * subIndex / framesPerKeyframe);
+
+		// apply audio animation
+
+		oneParameter =
+			ApplyAudioAnimation(index, oneParameter, listOfParameters[i].parameterName, params);
+		interpolated.parameters.AddParamFromOneParameter(fullParameterName, oneParameter);
 	}
 	return interpolated;
 }
@@ -114,7 +125,7 @@ void cKeyframes::GetInterpolatedFrameAndConsolidate(
 {
 	if (index >= 0 && index < frames.count() * framesPerKeyframe)
 	{
-		sAnimationFrame frame = GetInterpolatedFrame(index);
+		sAnimationFrame frame = GetInterpolatedFrame(index, params, fractal);
 
 		for (int i = 0; i < listOfParameters.size(); ++i)
 		{
@@ -123,6 +134,7 @@ void cKeyframes::GetInterpolatedFrameAndConsolidate(
 			QString parameterName = listOfParameters[i].parameterName;
 			cOneParameter oneParameter =
 				frame.parameters.GetAsOneParameter(listOfParameters[i].containerName + "_" + parameterName);
+
 			container->SetFromOneParameter(parameterName, oneParameter);
 		}
 	}
@@ -168,8 +180,8 @@ void cKeyframes::ChangeMorphType(int parameterIndex, parameterContainer::enumMor
 		if (parameterIndex < morph.size()) morph[parameterIndex]->Clear();
 
 		listOfParameters[parameterIndex].morphType = morphType;
-		QString fullParameterName = listOfParameters[parameterIndex].containerName + "_" +
-																listOfParameters[parameterIndex].parameterName;
+		QString fullParameterName = listOfParameters[parameterIndex].containerName + "_"
+																+ listOfParameters[parameterIndex].parameterName;
 
 		for (int i = 0; i < frames.size(); i++)
 		{
@@ -180,14 +192,14 @@ void cKeyframes::ChangeMorphType(int parameterIndex, parameterContainer::enumMor
 	}
 }
 void cKeyframes::AddAnimatedParameter(
-	const QString &parameterName, const cOneParameter &defaultValue)
+	const QString &parameterName, const cOneParameter &defaultValue, cParameterContainer *params)
 {
 	morph.clear();
-	cAnimationFrames::AddAnimatedParameter(parameterName, defaultValue);
+	cAnimationFrames::AddAnimatedParameter(parameterName, defaultValue, params);
 }
 
-bool cKeyframes::AddAnimatedParameter(const QString &fullParameterName,
-	const cParameterContainer *param, const cFractalContainer *fractal)
+bool cKeyframes::AddAnimatedParameter(
+	const QString &fullParameterName, cParameterContainer *param, const cFractalContainer *fractal)
 {
 	morph.clear();
 	return cAnimationFrames::AddAnimatedParameter(fullParameterName, param, fractal);
@@ -198,3 +210,8 @@ void cKeyframes::RemoveAnimatedParameter(const QString &fullParameterName)
 	morph.clear();
 	cAnimationFrames::RemoveAnimatedParameter(fullParameterName);
 }
+
+void cKeyframes::setAudioParameterPrefix()
+{
+	audioTracks.SetPrefix("animsound");
+}
