@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2015-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -34,7 +34,7 @@
  * Handles the 2D matrix of the list of parameters / list of frames
  * and exposes functions to modify this matrix. This functionality is
  * derived from the cAnimationFrames class. Additionally this class
- * interpolates subframes with the help of the cMorph class.
+ * interpolates sub-frames with the help of the cMorph class.
  */
 
 #ifndef MANDELBULBER2_SRC_KEYFRAMES_HPP_
@@ -54,19 +54,22 @@ public:
 	~cKeyframes();
 	cKeyframes(const cKeyframes &source);
 	cKeyframes &operator=(const cKeyframes &source);
-	sAnimationFrame GetInterpolatedFrame(int index);
+	sAnimationFrame GetInterpolatedFrame(
+		int index, cParameterContainer *params, cFractalContainer *fractal);
 	void GetInterpolatedFrameAndConsolidate(
 		int index, cParameterContainer *params, cFractalContainer *fractal);
 	void SetFramesPerKeyframe(int frPerKey) { framesPerKeyframe = frPerKey; }
-	int GetFramesPerKeyframe() { return framesPerKeyframe; }
+	int GetFramesPerKeyframe() const { return framesPerKeyframe; }
 	void ChangeMorphType(int parameterIndex, parameterContainer::enumMorphType morphType);
 	void ClearMorphCache() { morph.clear(); }
-	int GetUnrenderedTotal();
-	int GetUnrenderedTillIndex(int frameIndex);
-	void AddAnimatedParameter(const QString &parameterName, const cOneParameter &defaultValue);
-	bool AddAnimatedParameter(const QString &fullParameterName, const cParameterContainer *param,
-		const cFractalContainer *fractal);
-	void RemoveAnimatedParameter(const QString &fullParameterName);
+	int GetUnrenderedTotal() override;
+	int GetUnrenderedTillIndex(int frameIndex) override;
+	void AddAnimatedParameter(const QString &parameterName, const cOneParameter &defaultValue,
+		cParameterContainer *params = nullptr) override;
+	bool AddAnimatedParameter(const QString &fullParameterName, cParameterContainer *param,
+		const cFractalContainer *fractal) override;
+	void RemoveAnimatedParameter(const QString &fullParameterName) override;
+	void setAudioParameterPrefix() override;
 
 private:
 	int framesPerKeyframe;
