@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2015-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -36,27 +36,28 @@
  */
 
 #include "morph.hpp"
+
 #include "common_math.h"
 
 cMorph::cMorph()
 {
 	listSize = 6;
 	interpolationAccelerator = gsl_interp_accel_alloc();
-	splineAkimaPeriodic = gsl_spline_alloc(gsl_interp_akima_periodic, (size_t)listSize);
+	splineAkimaPeriodic = gsl_spline_alloc(gsl_interp_akima_periodic, size_t(listSize));
 }
 
 cMorph::~cMorph()
 {
 	gsl_spline_free(splineAkimaPeriodic);
 	gsl_interp_accel_free(interpolationAccelerator);
-	splineAkimaPeriodic = NULL;
-	interpolationAccelerator = NULL;
+	splineAkimaPeriodic = nullptr;
+	interpolationAccelerator = nullptr;
 }
 
 cMorph::cMorph(const cMorph &source)
 {
-	splineAkimaPeriodic = NULL;
-	interpolationAccelerator = NULL;
+	splineAkimaPeriodic = nullptr;
+	interpolationAccelerator = nullptr;
 	*this = source;
 }
 
@@ -66,7 +67,7 @@ cMorph &cMorph::operator=(const cMorph &source)
 	{
 		listSize = source.listSize;
 		interpolationAccelerator = gsl_interp_accel_alloc();
-		splineAkimaPeriodic = gsl_spline_alloc(gsl_interp_akima_periodic, (size_t)listSize);
+		splineAkimaPeriodic = gsl_spline_alloc(gsl_interp_akima_periodic, size_t(listSize));
 		dataSets = source.dataSets;
 	}
 	return *this;
@@ -101,8 +102,8 @@ cOneParameter cMorph::Interpolate(const int keyframe, double factor)
 {
 	int key = findInMorph(keyframe);
 	if (key == -1) return dataSets[0].parameter;
-	if (dataSets[key].parameter.GetValueType() == typeString ||
-			dataSets[key].parameter.GetValueType() == typeBool)
+	if (dataSets[key].parameter.GetValueType() == typeString
+			|| dataSets[key].parameter.GetValueType() == typeBool)
 	{
 		return None(key);
 	}
@@ -120,7 +121,7 @@ cOneParameter cMorph::Interpolate(const int keyframe, double factor)
 	}
 }
 
-cOneParameter cMorph::None(const int key)
+cOneParameter cMorph::None(const int key) const
 {
 	return dataSets[key].parameter;
 }
@@ -146,26 +147,26 @@ cOneParameter cMorph::Linear(const int key, const double factor, const bool angu
 		case typeInt:
 		{
 			double v1, v2;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
 			val.Store(LinearInterpolate(factor, v1, v2, angular));
 			break;
 		}
 		case typeRgb:
 		{
 			sRGB v1, v2;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			val.Store(sRGB((int)LinearInterpolate(factor, v1.R, v2.R, angular),
-				(int)LinearInterpolate(factor, v1.G, v2.G, angular),
-				(int)LinearInterpolate(factor, v1.B, v2.B, angular)));
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			val.Store(sRGB(int(LinearInterpolate(factor, v1.R, v2.R, angular)),
+				int(LinearInterpolate(factor, v1.G, v2.G, angular)),
+				int(LinearInterpolate(factor, v1.B, v2.B, angular))));
 			break;
 		}
 		case typeVector3:
 		{
 			CVector3 v1, v2;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
 			val.Store(CVector3(LinearInterpolate(factor, v1.x, v2.x, angular),
 				LinearInterpolate(factor, v1.y, v2.y, angular),
 				LinearInterpolate(factor, v1.z, v2.z, angular)));
@@ -174,8 +175,8 @@ cOneParameter cMorph::Linear(const int key, const double factor, const bool angu
 		case typeVector4:
 		{
 			CVector4 v1, v2;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
 			val.Store(CVector4(LinearInterpolate(factor, v1.x, v2.x, angular),
 				LinearInterpolate(factor, v1.y, v2.y, angular),
 				LinearInterpolate(factor, v1.z, v2.z, angular),
@@ -186,21 +187,21 @@ cOneParameter cMorph::Linear(const int key, const double factor, const bool angu
 		{
 			cColorPalette v1, v2;
 			cColorPalette out;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
 			for (int i = 0; i < v1.GetSize(); i++)
 			{
 				out.AppendColor(
-					sRGB((int)LinearInterpolate(factor, v1.GetColor(i).R, v2.GetColor(i).R, angular),
-						(int)LinearInterpolate(factor, v1.GetColor(i).G, v2.GetColor(i).G, angular),
-						(int)LinearInterpolate(factor, v1.GetColor(i).B, v2.GetColor(i).B, angular)));
+					sRGB(int(LinearInterpolate(factor, v1.GetColor(i).R, v2.GetColor(i).R, angular)),
+						int(LinearInterpolate(factor, v1.GetColor(i).G, v2.GetColor(i).G, angular)),
+						int(LinearInterpolate(factor, v1.GetColor(i).B, v2.GetColor(i).B, angular))));
 			}
 			val.Store(out);
 			break;
 		}
 	}
 
-	interpolated.SetMultival(val, valueActual);
+	interpolated.SetMultiVal(val, valueActual);
 	return interpolated;
 }
 
@@ -241,32 +242,32 @@ cOneParameter cMorph::CatmullRom(int const key, double const factor, bool const
 		case typeInt:
 		{
 			double v1, v2, v3, v4;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
 			val.Store(CatmullRomInterpolate(factor, v1, v2, v3, v4, angular));
 			break;
 		}
 		case typeRgb:
 		{
 			sRGB v1, v2, v3, v4;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
-			val.Store(sRGB((int)CatmullRomInterpolate(factor, v1.R, v2.R, v3.R, v4.R, angular),
-				(int)CatmullRomInterpolate(factor, v1.G, v2.G, v3.G, v4.G, angular),
-				(int)CatmullRomInterpolate(factor, v1.B, v2.B, v3.B, v4.B, angular)));
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
+			val.Store(sRGB(int(CatmullRomInterpolate(factor, v1.R, v2.R, v3.R, v4.R, angular)),
+				int(CatmullRomInterpolate(factor, v1.G, v2.G, v3.G, v4.G, angular)),
+				int(CatmullRomInterpolate(factor, v1.B, v2.B, v3.B, v4.B, angular))));
 			break;
 		}
 		case typeVector3:
 		{
 			CVector3 v1, v2, v3, v4;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
 			val.Store(CVector3(CatmullRomInterpolate(factor, v1.x, v2.x, v3.x, v4.x, angular),
 				CatmullRomInterpolate(factor, v1.y, v2.y, v3.y, v4.y, angular),
 				CatmullRomInterpolate(factor, v1.z, v2.z, v3.z, v4.z, angular)));
@@ -275,10 +276,10 @@ cOneParameter cMorph::CatmullRom(int const key, double const factor, bool const
 		case typeVector4:
 		{
 			CVector4 v1, v2, v3, v4;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
 			val.Store(CVector4(CatmullRomInterpolate(factor, v1.x, v2.x, v3.x, v4.x, angular),
 				CatmullRomInterpolate(factor, v1.y, v2.y, v3.y, v4.y, angular),
 				CatmullRomInterpolate(factor, v1.z, v2.z, v3.z, v4.z, angular),
@@ -289,25 +290,25 @@ cOneParameter cMorph::CatmullRom(int const key, double const factor, bool const
 		{
 			cColorPalette v1, v2, v3, v4;
 			cColorPalette out;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
 			for (int i = 0; i < v1.GetSize(); i++)
 			{
-				out.AppendColor(sRGB((int)CatmullRomInterpolate(factor, v1.GetColor(i).R, v2.GetColor(i).R,
-															 v3.GetColor(i).R, v4.GetColor(i).R, angular),
-					(int)CatmullRomInterpolate(factor, v1.GetColor(i).G, v2.GetColor(i).G, v3.GetColor(i).G,
-															 v4.GetColor(i).G, angular),
-					(int)CatmullRomInterpolate(factor, v1.GetColor(i).B, v2.GetColor(i).B, v3.GetColor(i).B,
-															 v4.GetColor(i).B, angular)));
+				out.AppendColor(sRGB(int(CatmullRomInterpolate(factor, v1.GetColor(i).R, v2.GetColor(i).R,
+															 v3.GetColor(i).R, v4.GetColor(i).R, angular)),
+					int(CatmullRomInterpolate(factor, v1.GetColor(i).G, v2.GetColor(i).G, v3.GetColor(i).G,
+						v4.GetColor(i).G, angular)),
+					int(CatmullRomInterpolate(factor, v1.GetColor(i).B, v2.GetColor(i).B, v3.GetColor(i).B,
+						v4.GetColor(i).B, angular))));
 			}
 			val.Store(out);
 			break;
 		}
 	}
 
-	interpolated.SetMultival(val, valueActual);
+	interpolated.SetMultiVal(val, valueActual);
 	return interpolated;
 }
 
@@ -355,38 +356,38 @@ cOneParameter cMorph::Akima(int const key, double const factor, bool const angul
 		case typeInt:
 		{
 			double v1, v2, v3, v4, v5, v6;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
-			dataSets[k5].parameter.GetMultival(valueActual).Get(v5);
-			dataSets[k6].parameter.GetMultival(valueActual).Get(v6);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
+			dataSets[k5].parameter.GetMultiVal(valueActual).Get(v5);
+			dataSets[k6].parameter.GetMultiVal(valueActual).Get(v6);
 			val.Store(AkimaInterpolate(factor, v1, v2, v3, v4, v5, v6, angular));
 			break;
 		}
 		case typeRgb:
 		{
 			sRGB v1, v2, v3, v4, v5, v6;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
-			dataSets[k5].parameter.GetMultival(valueActual).Get(v5);
-			dataSets[k6].parameter.GetMultival(valueActual).Get(v6);
-			val.Store(sRGB((int)AkimaInterpolate(factor, v1.R, v2.R, v3.R, v4.R, v5.R, v6.R, angular),
-				(int)AkimaInterpolate(factor, v1.G, v2.G, v3.G, v4.G, v5.G, v6.G, angular),
-				(int)AkimaInterpolate(factor, v1.B, v2.B, v3.B, v4.B, v5.B, v6.B, angular)));
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
+			dataSets[k5].parameter.GetMultiVal(valueActual).Get(v5);
+			dataSets[k6].parameter.GetMultiVal(valueActual).Get(v6);
+			val.Store(sRGB(int(AkimaInterpolate(factor, v1.R, v2.R, v3.R, v4.R, v5.R, v6.R, angular)),
+				int(AkimaInterpolate(factor, v1.G, v2.G, v3.G, v4.G, v5.G, v6.G, angular)),
+				int(AkimaInterpolate(factor, v1.B, v2.B, v3.B, v4.B, v5.B, v6.B, angular))));
 			break;
 		}
 		case typeVector3:
 		{
 			CVector3 v1, v2, v3, v4, v5, v6;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
-			dataSets[k5].parameter.GetMultival(valueActual).Get(v5);
-			dataSets[k6].parameter.GetMultival(valueActual).Get(v6);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
+			dataSets[k5].parameter.GetMultiVal(valueActual).Get(v5);
+			dataSets[k6].parameter.GetMultiVal(valueActual).Get(v6);
 			val.Store(CVector3(AkimaInterpolate(factor, v1.x, v2.x, v3.x, v4.x, v5.x, v6.x, angular),
 				AkimaInterpolate(factor, v1.y, v2.y, v3.y, v4.y, v5.y, v6.y, angular),
 				AkimaInterpolate(factor, v1.z, v2.z, v3.z, v4.z, v5.z, v6.z, angular)));
@@ -395,12 +396,12 @@ cOneParameter cMorph::Akima(int const key, double const factor, bool const angul
 		case typeVector4:
 		{
 			CVector4 v1, v2, v3, v4, v5, v6;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
-			dataSets[k5].parameter.GetMultival(valueActual).Get(v5);
-			dataSets[k6].parameter.GetMultival(valueActual).Get(v6);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
+			dataSets[k5].parameter.GetMultiVal(valueActual).Get(v5);
+			dataSets[k6].parameter.GetMultiVal(valueActual).Get(v6);
 			val.Store(CVector4(AkimaInterpolate(factor, v1.x, v2.x, v3.x, v4.x, v5.x, v6.x, angular),
 				AkimaInterpolate(factor, v1.y, v2.y, v3.y, v4.y, v5.y, v6.y, angular),
 				AkimaInterpolate(factor, v1.z, v2.z, v3.z, v4.z, v5.z, v6.z, angular),
@@ -411,28 +412,28 @@ cOneParameter cMorph::Akima(int const key, double const factor, bool const angul
 		{
 			cColorPalette v1, v2, v3, v4, v5, v6;
 			cColorPalette out;
-			dataSets[k1].parameter.GetMultival(valueActual).Get(v1);
-			dataSets[k2].parameter.GetMultival(valueActual).Get(v2);
-			dataSets[k3].parameter.GetMultival(valueActual).Get(v3);
-			dataSets[k4].parameter.GetMultival(valueActual).Get(v4);
-			dataSets[k5].parameter.GetMultival(valueActual).Get(v5);
-			dataSets[k6].parameter.GetMultival(valueActual).Get(v6);
+			dataSets[k1].parameter.GetMultiVal(valueActual).Get(v1);
+			dataSets[k2].parameter.GetMultiVal(valueActual).Get(v2);
+			dataSets[k3].parameter.GetMultiVal(valueActual).Get(v3);
+			dataSets[k4].parameter.GetMultiVal(valueActual).Get(v4);
+			dataSets[k5].parameter.GetMultiVal(valueActual).Get(v5);
+			dataSets[k6].parameter.GetMultiVal(valueActual).Get(v6);
 			for (int i = 0; i < v1.GetSize(); i++)
 			{
 				out.AppendColor(
-					sRGB((int)AkimaInterpolate(factor, v1.GetColor(i).R, v2.GetColor(i).R, v3.GetColor(i).R,
-								 v4.GetColor(i).R, v5.GetColor(i).R, v6.GetColor(i).R, angular),
-						(int)AkimaInterpolate(factor, v1.GetColor(i).G, v2.GetColor(i).G, v3.GetColor(i).G,
-								 v4.GetColor(i).G, v5.GetColor(i).G, v6.GetColor(i).G, angular),
-						(int)AkimaInterpolate(factor, v1.GetColor(i).B, v2.GetColor(i).B, v3.GetColor(i).B,
-								 v4.GetColor(i).B, v5.GetColor(i).B, v6.GetColor(i).B, angular)));
+					sRGB(int(AkimaInterpolate(factor, v1.GetColor(i).R, v2.GetColor(i).R, v3.GetColor(i).R,
+								 v4.GetColor(i).R, v5.GetColor(i).R, v6.GetColor(i).R, angular)),
+						int(AkimaInterpolate(factor, v1.GetColor(i).G, v2.GetColor(i).G, v3.GetColor(i).G,
+							v4.GetColor(i).G, v5.GetColor(i).G, v6.GetColor(i).G, angular)),
+						int(AkimaInterpolate(factor, v1.GetColor(i).B, v2.GetColor(i).B, v3.GetColor(i).B,
+							v4.GetColor(i).B, v5.GetColor(i).B, v6.GetColor(i).B, angular))));
 			}
 			val.Store(out);
 			break;
 		}
 	}
 
-	interpolated.SetMultival(val, valueActual);
+	interpolated.SetMultiVal(val, valueActual);
 	return interpolated;
 }
 
@@ -487,8 +488,8 @@ double cMorph::CatmullRomInterpolate(
 			}
 		}
 	}
-	double value = 0.5 * ((2 * v2) + (-v1 + v3) * factor + (2 * v1 - 5 * v2 + 4 * v3 - v4) * factor2 +
-												 (-v1 + 3 * v2 - 3 * v3 + v4) * factor3);
+	double value = 0.5 * ((2 * v2) + (-v1 + v3) * factor + (2 * v1 - 5 * v2 + 4 * v3 - v4) * factor2
+												 + (-v1 + 3 * v2 - 3 * v3 + v4) * factor3);
 	if (logarithmic)
 	{
 		if (negative)
@@ -511,7 +512,7 @@ double cMorph::CatmullRomInterpolate(
 }
 
 double cMorph::AkimaInterpolate(const double factor, double v1, double v2, double v3, double v4,
-	double v5, double v6, const bool angular)
+	double v5, double v6, const bool angular) const
 {
 	if (angular)
 	{
@@ -522,8 +523,8 @@ double cMorph::AkimaInterpolate(const double factor, double v1, double v2, doubl
 
 	double x[] = {-2, -1, 0, 1, 2, 3};
 	double y[] = {v1, v2, v3, v4, v5, v6};
-	// more info: http://www.alglib.net/interpolation/spline3.php
-	gsl_spline_init(splineAkimaPeriodic, x, y, (size_t)listSize);
+	// more information: http://www.alglib.net/interpolation/spline3.php
+	gsl_spline_init(splineAkimaPeriodic, x, y, size_t(listSize));
 	double value = gsl_spline_eval(splineAkimaPeriodic, factor, interpolationAccelerator);
 
 	if (angular)
