@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2015-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -35,14 +35,16 @@
  * interpolate them with different interpolation techniques.
  */
 
-#ifndef MORPH_HPP_
-#define MORPH_HPP_
+#ifndef MANDELBULBER2_SRC_MORPH_HPP_
+#define MANDELBULBER2_SRC_MORPH_HPP_
 
-#include "one_parameter.hpp"
-#include <QList>
 #include <gsl/gsl_interp.h>
 #include <gsl/gsl_spline.h>
 
+#include <QList>
+
+#include "one_parameter.hpp"
+
 class cMorph
 {
 public:
@@ -64,17 +66,17 @@ public:
 	int findInMorph(const int keyframe);
 	void Clear() { dataSets.clear(); }
 	cOneParameter Interpolate(const int keyframe, double factor);
-	cOneParameter None(const int key);
+	cOneParameter None(const int key) const;
 	cOneParameter Linear(const int key, const double factor, bool const angular);
 	cOneParameter CatmullRom(const int key, const double factor, bool const angular);
 	cOneParameter Akima(const int key, const double factor, bool const angular);
 
-	double LinearInterpolate(const double factor, double v1, double v2, bool const angular);
-	double CatmullRomInterpolate(
+	static double LinearInterpolate(const double factor, double v1, double v2, bool const angular);
+	static double CatmullRomInterpolate(
 		const double factor, double v1, double v2, double v3, double v4, bool const angular);
 	double AkimaInterpolate(const double factor, double v1, double v2, double v3, double v4,
-		double v5, double v6, const bool angular);
-	void NearestNeighbourAngle(QList<double *> vals);
+		double v5, double v6, const bool angular) const;
+	static void NearestNeighbourAngle(QList<double *> vals);
 
 private:
 	int listSize;
@@ -83,4 +85,4 @@ private:
 	QList<sMorphParameter> dataSets;
 };
 
-#endif /* MORPH_HPP_ */
+#endif /* MANDELBULBER2_SRC_MORPH_HPP_ */
