@@ -1,51 +1,66 @@
 /**
- * Mandelbulber v2, a 3D fractal generator
+ * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
+ *                                             ,B" ]L,,p%%%,,,§;, "K
+ * Copyright (C) 2015-18 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
+ * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
+ *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
+ * Mandelbulber is free software:     §R.ß~-Q/M=,=5"v"]=Qf,'§"M= =,M.§ Rz]M"Kw
+ * you can redistribute it and/or     §w "xDY.J ' -"m=====WeC=\ ""%""y=%"]"" §
+ * modify it under the terms of the    "§M=M =D=4"N #"%==A%p M§ M6  R' #"=~.4M
+ * GNU General Public License as        §W =, ][T"]C  §  § '§ e===~ U  !§[Z ]N
+ * published by the                    4M",,Jm=,"=e~  §  §  j]]""N  BmM"py=ßM
+ * Free Software Foundation,          ]§ T,M=& 'YmMMpM9MMM%=w=,,=MT]M m§;'§,
+ * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
+ * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
+ * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
+ * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
+ * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
+ * but WITHOUT ANY WARRANTY;                            .'''
+ * without even the implied warranty
+ * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *
+ * See the GNU General Public License for more details.
+ * You should have received a copy of the GNU General Public License
+ * along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ###########################################################################
+ *
+ * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Sebastian Jennen (jenzebas@gmail.com)
  *
  * Netrender class - Networking class for command and payload communication
- * between one server and multiple clients
- *
- * Copyright (C) 2014 Krzysztof Marczak
- *
- * This file is part of Mandelbulber.
- *
- * Mandelbulber is free software: you can redistribute it and/or modify it under the terms of the
- * GNU General Public License as published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * Mandelbulber is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * See the GNU General Public License for more details. You should have received a copy of the GNU
- * General Public License along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
- *
- * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Sebastian Jennen
  */
 
 #include "netrender.hpp"
-#include "system.hpp"
+
 #include <QAbstractSocket>
 #include <QHostInfo>
-#include "settings.hpp"
-#include "global_data.hpp"
-#include "interface.hpp"
+
 #include "error_message.hpp"
+#include "fractal_container.hpp"
+#include "global_data.hpp"
 #include "headless.h"
 #include "initparameters.hpp"
-#include "fractal_container.hpp"
+#include "interface.hpp"
+#include "lzo_compression.h"
+#include "render_window.hpp"
+#include "settings.hpp"
+#include "system.hpp"
+#include "texture.hpp"
 
-CNetRender *gNetRender = NULL;
+CNetRender *gNetRender = nullptr;
 
-CNetRender::CNetRender(qint32 workerCount) :
-		QObject(NULL)
+CNetRender::CNetRender(qint32 workerCount) : QObject(nullptr)
 {
 	this->workerCount = workerCount;
 	deviceType = netRender_UNKNOWN;
 	version = 1000 * MANDELBULBER_VERSION;
-	clientSocket = NULL;
-	server = NULL;
+	clientSocket = nullptr;
+	server = nullptr;
 	portNo = 0;
 	status = netRender_NEW;
-	reconnectTimer = NULL;
+	reconnectTimer = nullptr;
 	actualId = 0;
 	totalReceivedUncompressed = 0;
 	totalReceived = 0;
@@ -71,16 +86,16 @@ void CNetRender::SetServer(qint32 portNo)
 	{
 		if (server->serverError() == QAbstractSocket::AddressInUseError)
 		{
-			cErrorMessage::showMessage(QObject::tr("NetRender - address already in use.\n\nIs there already a mandelbulber server instance running on this port?"),
-																 cErrorMessage::errorMessage,
-																 gMainInterface->mainWindow);
+			cErrorMessage::showMessage(
+				QObject::tr("NetRender - address already in use.\n\nIs there already a mandelbulber server "
+										"instance running on this port?"),
+				cErrorMessage::errorMessage, gMainInterface->mainWindow);
 		}
 		else
 		{
-			cErrorMessage::showMessage(QObject::tr("NetRender - SetServer Error:\n\n")
-																		 + server->errorString(),
-																 cErrorMessage::errorMessage,
-																 gMainInterface->mainWindow);
+			cErrorMessage::showMessage(
+				QObject::tr("NetRender - SetServer Error:\n\n") + server->errorString(),
+				cErrorMessage::errorMessage, gMainInterface->mainWindow);
 		}
 		deviceType = netRender_UNKNOWN;
 	}
@@ -111,7 +126,7 @@ void CNetRender::DeleteServer()
 	{
 		server->close();
 		delete server;
-		server = NULL;
+		server = nullptr;
 	}
 	clients.clear();
 	emit ClientsChanged();
@@ -124,12 +139,17 @@ void CNetRender::DeleteClient()
 	if (deviceType != netRender_CLIENT) return;
 	deviceType = netRender_UNKNOWN;
 	WriteLog("NetRender - Delete Client", 2);
-	if (reconnectTimer->isActive()) reconnectTimer->stop();
+	if (reconnectTimer)
+	{
+		if (reconnectTimer->isActive()) reconnectTimer->stop();
+		delete reconnectTimer;
+		reconnectTimer = nullptr;
+	}
 	if (clientSocket)
 	{
 		clientSocket->close();
 		delete clientSocket;
-		clientSocket = NULL;
+		clientSocket = nullptr;
 	}
 	status = netRender_DISABLED;
 	emit NotifyStatus();
@@ -161,7 +181,12 @@ void CNetRender::HandleNewConnection()
 		// tell mandelbulber version to client
 		sMessage msg;
 		msg.command = netRender_VERSION;
-		msg.payload.append((char*) &version, sizeof(qint32));
+
+		QDataStream stream(&msg.payload, QIODevice::WriteOnly);
+		stream << qint32(version);
+		QString machineName = QHostInfo::localHostName();
+		stream << qint32(machineName.toUtf8().size());
+		stream.writeRawData(machineName.toUtf8().data(), machineName.toUtf8().size());
 		SendData(client.socket, msg);
 		emit ClientsChanged();
 	}
@@ -175,7 +200,8 @@ void CNetRender::ClientDisconnected()
 	{
 		int index = GetClientIndexFromSocket(socket);
 		WriteLog("NetRender - Client disconnected #" + QString::number(index) + " ("
-				+ socket->peerAddress().toString() + ")", 2);
+							 + socket->peerAddress().toString() + ")",
+			2);
 		if (index > -1)
 		{
 			clients.removeAt(index);
@@ -186,7 +212,7 @@ void CNetRender::ClientDisconnected()
 	}
 }
 
-int CNetRender::GetClientIndexFromSocket(const QTcpSocket *socket)
+int CNetRender::GetClientIndexFromSocket(const QTcpSocket *socket) const
 {
 	for (int i = 0; i < clients.size(); i++)
 	{
@@ -205,6 +231,7 @@ void CNetRender::ReceiveFromClient()
 	int index = GetClientIndexFromSocket(socket);
 	if (index != -1)
 	{
+		WriteLog("NetRender - ReceiveFromClient()", 3);
 		ReceiveData(socket, &clients[index].msg);
 	}
 }
@@ -227,16 +254,16 @@ void CNetRender::SetClient(QString address, int portNo)
 
 	reconnectTimer->start();
 	QTimer::singleShot(50, this, SLOT(TryServerConnect()));
-	WriteLog("NetRender - Client Setup, link to server: " + address + ", port: "
-			+ QString::number(portNo), 2);
+	WriteLog(
+		"NetRender - Client Setup, link to server: " + address + ", port: " + QString::number(portNo),
+		2);
 	emit NotifyStatus();
 
 	if (systemData.noGui)
 	{
 		QTextStream out(stdout);
-		out
-				<< "NetRender - Client Setup, link to server: " + address + ", port: "
-						+ QString::number(portNo) + "\n";
+		out << "NetRender - Client Setup, link to server: " + address + ", port: "
+						 + QString::number(portNo) + "\n";
 	}
 }
 
@@ -250,6 +277,8 @@ void CNetRender::ServerDisconnected()
 
 	reconnectTimer->start();
 
+	WriteLog("NetRender - server disconnected", 2);
+
 	if (systemData.noGui)
 	{
 		QTextStream out(stdout);
@@ -258,7 +287,7 @@ void CNetRender::ServerDisconnected()
 	}
 	else
 	{
-		cErrorMessage::showMessage(QString("Connection lost"), cErrorMessage::errorMessage);
+		qCritical() << "Connection lost";
 	}
 }
 
@@ -266,11 +295,13 @@ void CNetRender::TryServerConnect()
 {
 	if (clientSocket)
 	{
+		WriteLog("NetRender - TryServerConnect", 3);
 		if (clientSocket->state() == QAbstractSocket::ConnectedState)
 		{
 			reconnectTimer->stop();
 		}
-		else if (clientSocket->state() == QAbstractSocket::ConnectingState)
+		else if (clientSocket->state() == QAbstractSocket::ConnectingState
+						 || clientSocket->state() == QAbstractSocket::HostLookupState)
 		{
 			return; // wait for result
 		}
@@ -286,10 +317,11 @@ void CNetRender::TryServerConnect()
 
 void CNetRender::ReceiveFromServer()
 {
+	WriteLog("NetRender - ReceiveFromServer()", 3);
 	ReceiveData(clientSocket, &msgFromServer);
 }
 
-bool CNetRender::SendData(QTcpSocket *socket, sMessage msg)
+bool CNetRender::SendData(QTcpSocket *socket, sMessage msg) const
 {
 	//			NetRender Message format         (optional)
 	// | qint32		| qint32	| qint32	|( 0 - ???	| quint16  |)
@@ -301,9 +333,16 @@ bool CNetRender::SendData(QTcpSocket *socket, sMessage msg)
 	QByteArray byteArray;
 	QDataStream socketWriteStream(&byteArray, QIODevice::ReadWrite);
 
+	msg.payload = lzoCompress(msg.payload);
 	msg.size = msg.payload.size();
 	msg.id = actualId;
 
+	WriteLog(QString("NetRender - send data, command %1, bytes %2, id %3")
+						 .arg(msg.command)
+						 .arg(msg.size)
+						 .arg(msg.id),
+		3);
+
 	// append header
 	socketWriteStream << msg.command << msg.id << msg.size;
 
@@ -317,25 +356,30 @@ bool CNetRender::SendData(QTcpSocket *socket, sMessage msg)
 	}
 
 	// write to socket
-	socket->write(byteArray);
-	//socket->waitForBytesWritten();
+	if (socket->isOpen() && socket->state() == QAbstractSocket::ConnectedState)
+	{
+		socket->write(byteArray);
+	}
+	else
+	{
+		qCritical() << "CNetRender::SendData(QTcpSocket *socket, sMessage msg): socket closed!";
+	}
 
 	return true;
 }
 
 void CNetRender::ResetMessage(sMessage *msg)
 {
-	if (msg == NULL)
+	if (msg == nullptr)
 	{
-		qCritical() << "CNetRender::ResetMessage(sMessage *msg): message has NULL pointer!";
+		qCritical() << "CNetRender::ResetMessage(sMessage *msg): message is nullptr!";
 	}
 	else
 	{
 		msg->command = netRender_NONE;
 		msg->id = 0;
 		msg->size = 0;
-		if(!msg->payload.isEmpty())
-			msg->payload.clear();
+		if (!msg->payload.isEmpty()) msg->payload.clear();
 	}
 }
 
@@ -349,7 +393,7 @@ void CNetRender::ReceiveData(QTcpSocket *socket, sMessage *msg)
 		if (msg->command == netRender_NONE)
 		{
 			if (socket->bytesAvailable()
-					< (qint64) ((sizeof(msg->command) + sizeof(msg->id) + sizeof(msg->size))))
+					< qint64((sizeof(msg->command) + sizeof(msg->id) + sizeof(msg->size))))
 			{
 				return;
 			}
@@ -357,18 +401,23 @@ void CNetRender::ReceiveData(QTcpSocket *socket, sMessage *msg)
 			socketReadStream >> msg->command;
 			socketReadStream >> msg->id;
 			socketReadStream >> msg->size;
+			WriteLog(QString("NetRender - ReceiveData(), command %1, bytes %2, id %3")
+								 .arg(msg->command)
+								 .arg(msg->size)
+								 .arg(msg->id),
+				3);
 		}
 
 		bytesAvailable = socket->bytesAvailable();
 
 		if (msg->size > 0)
 		{
-			if (bytesAvailable < (qint64) (sizeof(quint16) + msg->size))
+			if (bytesAvailable < qint64(sizeof(quint16) + msg->size))
 			{
 				return;
 			}
 			// full payload available
-			char* buffer = new char[msg->size];
+			char *buffer = new char[msg->size];
 			socketReadStream.readRawData(buffer, msg->size);
 			msg->payload.append(buffer, msg->size);
 
@@ -379,11 +428,14 @@ void CNetRender::ReceiveData(QTcpSocket *socket, sMessage *msg)
 			delete[] buffer;
 			if (crcCalculated != crcReceived)
 			{
-				//ResetMessage(msg);
-				//socketReadStream.atEnd();
-				//socketReadStream.skipRawData(socket->bytesAvailable());
+				// ResetMessage(msg);
+				// socketReadStream.atEnd();
+				// socketReadStream.skipRawData(socket->bytesAvailable());
+				WriteLog("NetRender - ReceiveData() : crc error", 2);
+
 				return;
 			}
+			msg->payload = lzoUncompress(msg->payload);
 			msg->size = msg->payload.size();
 		}
 		ProcessData(socket, msg);
@@ -400,43 +452,53 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 	//------------------------- CLIENT ------------------------
 	if (IsClient())
 	{
-		switch ((netCommand) inMsg->command)
+		switch (netCommand(inMsg->command))
 		{
 			case netRender_VERSION:
 			{
 				sMessage outMsg;
-				qint32 serverVersion = *(qint32*) inMsg->payload.data();
-
+				QDataStream stream(&inMsg->payload, QIODevice::ReadOnly);
+				qint32 serverVersion;
+				stream >> serverVersion;
+				QByteArray buffer;
+				qint32 size;
+				stream >> size;
+				buffer.resize(size);
+				stream.readRawData(buffer.data(), size);
+				serverName = QString::fromUtf8(buffer.data(), buffer.size());
 				if (CompareMajorVersion(serverVersion, version))
 				{
-					WriteLog("NetRender - version matches (" + QString::number(version)
-							+ "), connection established", 2);
-
+					QString connectionMsg = "NetRender - version matches (" + QString::number(version) + ")";
+					QString serverInfo = QString("NetRender - Connection established, Server is %1:%2 [%3]")
+																 .arg(address, QString::number(portNo), serverName);
+					WriteLog(connectionMsg, 2);
+					WriteLog(serverInfo, 2);
 					if (systemData.noGui)
 					{
 						QTextStream out(stdout);
-						out
-								<< "NetRender - version matches (" + QString::number(version)
-										+ "), connection established\n";
+						out << connectionMsg << "\n";
+						out << serverInfo << "\n";
 					}
 
 					// server version matches, send worker count
 					outMsg.command = netRender_WORKER;
-					QDataStream stream(&outMsg.payload, QIODevice::WriteOnly);
-					stream << workerCount;
+					QDataStream outStream(&outMsg.payload, QIODevice::WriteOnly);
+					outStream << workerCount;
 					QString machineName = QHostInfo::localHostName();
-					stream << (qint32) machineName.toUtf8().size();
-					stream.writeRawData(machineName.toUtf8().data(), machineName.toUtf8().size());
+					outStream << qint32(machineName.toUtf8().size());
+					outStream.writeRawData(machineName.toUtf8().data(), machineName.toUtf8().size());
 					status = netRender_READY;
 					emit NewStatusClient();
+					WriteLog(
+						QString("NetRender - ProcessData(), command VERSION, version %1").arg(serverVersion),
+						2);
 				}
 				else
 				{
-          cErrorMessage::showMessage(tr("NetRender - version mismatch!\n")
-                                         + tr("Client version: %1\n").arg(version)
-                                         + tr("Server version: %1").arg(serverVersion),
-                                     cErrorMessage::errorMessage,
-                                     gMainInterface->mainWindow);
+					cErrorMessage::showMessage(tr("NetRender - version mismatch!\n")
+																			 + tr("Client version: %1\n").arg(version)
+																			 + tr("Server version: %1").arg(serverVersion),
+						cErrorMessage::errorMessage, gMainInterface->mainWindow);
 
 					outMsg.command = netRender_BAD;
 				}
@@ -445,14 +507,15 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 			}
 			case netRender_STOP:
 			{
-				status = netRender_READY;
+				// status = netRender_READY;
 				gMainInterface->stopRequest = true;
-				emit NotifyStatus();
-				WriteLog("CNetRender - STOP", 2);
+				// emit NotifyStatus();
+				WriteLog("NetRender - ProcessData(), command STOP", 2);
 				break;
 			}
 			case netRender_STATUS:
 			{
+				WriteLog("NetRender - ProcessData(), command STATUS", 3);
 				emit NotifyStatus();
 				break;
 			}
@@ -460,7 +523,7 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 			{
 				if (inMsg->id == actualId)
 				{
-					WriteLog("NetRender - received new job", 2);
+					WriteLog("NetRender - ProcessData(), command JOB", 2);
 					QDataStream stream(&inMsg->payload, QIODevice::ReadOnly);
 					QByteArray buffer;
 					qint32 size;
@@ -472,10 +535,18 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 					buffer.resize(size);
 					stream.readRawData(buffer.data(), size);
 					settingsText = QString::fromUtf8(buffer.data(), buffer.size());
+					WriteLog(
+						QString("NetRender - ProcessData(), command JOB, settings size: %1").arg(size), 2);
+					WriteLog(
+						QString("NetRender - ProcessData(), command JOB, settings: %1").arg(settingsText), 3);
 
 					qint32 numberOfTextures;
 					stream >> numberOfTextures;
 
+					WriteLog(QString("NetRender - ProcessData(), command JOB, number of textures: %1")
+										 .arg(numberOfTextures),
+						2);
+
 					// read textures
 					for (int i = 0; i < numberOfTextures; i++)
 					{
@@ -483,16 +554,21 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 						stream >> sizeOfName;
 
 						QString textureName;
-						if(sizeOfName > 0)
+						if (sizeOfName > 0)
 						{
 							QByteArray bufferForName;
 							bufferForName.resize(sizeOfName);
 							stream.readRawData(bufferForName.data(), sizeOfName);
 							textureName = QString::fromUtf8(bufferForName);
+							WriteLog(QString("NetRender - ProcessData(), command JOB, texture name: %1")
+												 .arg(textureName),
+								2);
 						}
 
 						cTexture texture;
 						stream >> size;
+						WriteLog(
+							QString("NetRender - ProcessData(), command JOB, texture size: %1").arg(size), 2);
 						if (size > 0)
 						{
 							buffer.resize(size);
@@ -503,9 +579,14 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 
 					cSettings parSettings(cSettings::formatCondensedText);
 					parSettings.BeQuiet(true);
+
+					gInterfaceReadyForSynchronization = false;
 					parSettings.LoadFromString(settingsText);
 					parSettings.Decode(gPar, gParFractal);
 
+					WriteLog("NetRender - ProcessData(), command JOB, starting rendering", 2);
+
+					gInterfaceReadyForSynchronization = true;
 					if (!systemData.noGui)
 					{
 						gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
@@ -513,22 +594,18 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 					}
 					else
 					{
-						//in noGui mode it must be started as separate thread to be able to process event loop
+						// in noGui mode it must be started as separate thread to be able to process event loop
 						gMainInterface->headless = new cHeadless;
 
-						QThread *thread = new QThread; //deleted by deleteLater()
+						QThread *thread = new QThread; // deleted by deleteLater()
 						gMainInterface->headless->moveToThread(thread);
-						QObject::connect(thread,
-														 SIGNAL(started()),
-														 gMainInterface->headless,
-														 SLOT(slotNetRender()));
+						QObject::connect(
+							thread, SIGNAL(started()), gMainInterface->headless, SLOT(slotNetRender()));
 						thread->setObjectName("RenderJob");
 						thread->start();
 
-						QObject::connect(gMainInterface->headless,
-														 SIGNAL(finished()),
-														 gMainInterface->headless,
-														 SLOT(deleteLater()));
+						QObject::connect(gMainInterface->headless, SIGNAL(finished()), gMainInterface->headless,
+							SLOT(deleteLater()));
 						QObject::connect(gMainInterface->headless, SIGNAL(finished()), thread, SLOT(quit()));
 						QObject::connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
 					}
@@ -541,6 +618,7 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 			}
 			case netRender_RENDER:
 			{
+				WriteLog("NetRender - ProcessData(), command RENDER", 3);
 				if (inMsg->id == actualId)
 				{
 					QDataStream stream(&inMsg->payload, QIODevice::ReadOnly);
@@ -553,7 +631,9 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 						stream >> line;
 						done.append(line);
 					}
-
+					WriteLog(QString("NetRender - ProcessData(), command RENDER, done list size: %1")
+										 .arg(done.size()),
+						3);
 					emit ToDoListArrived(done);
 				}
 				else
@@ -565,6 +645,7 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 
 			case netRender_SETUP:
 			{
+				WriteLog("NetRender - ProcessData(), command SETUP", 2);
 				QDataStream stream(&inMsg->payload, QIODevice::ReadOnly);
 
 				stream >> actualId;
@@ -577,12 +658,17 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 					qint32 line;
 					stream >> line;
 					startingPositions.append(line);
+					WriteLog(QString("NetRender - ProcessData(), command SETUP, start line %1 = %2")
+										 .arg(i)
+										 .arg(line),
+						3);
 				}
 				break;
 			}
 
 			case netRender_ACK:
 			{
+				WriteLog("NetRender - ProcessData(), command ACK", 3);
 				if (inMsg->id == actualId)
 				{
 					emit AckReceived();
@@ -590,8 +676,14 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 				break;
 			}
 
-			default:
+			case netRender_KICK_AND_KILL:
+			{
+				WriteLog("NetRender - ProcessData(), command KICK AND KILL", 2);
+				gApplication->quit();
 				break;
+			}
+
+			default: break;
 		}
 	}
 
@@ -601,20 +693,19 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 		int index = GetClientIndexFromSocket(socket);
 		if (index > -1)
 		{
-			switch ((netCommand) inMsg->command)
+			switch (netCommand(inMsg->command))
 			{
 				case netRender_BAD:
 				{
 
-		      cErrorMessage::showMessage(QObject::tr("NetRender - Client version mismatch!\n Client address:")
-		                                     + socket->peerAddress().toString(),
-		                                 cErrorMessage::errorMessage,
-		                                 gMainInterface->mainWindow);
+					cErrorMessage::showMessage(
+						QObject::tr("NetRender - Client version mismatch!\n Client address:")
+							+ socket->peerAddress().toString(),
+						cErrorMessage::errorMessage, gMainInterface->mainWindow);
 
-				  clients.removeAt(index);
+					clients.removeAt(index);
 					emit ClientsChanged();
-					return; //to avoid reseting already deleted message buffer
-					break;
+					return; // to avoid resetting already deleted message buffer
 				}
 				case netRender_WORKER:
 				{
@@ -629,7 +720,8 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 
 					if (clients[index].status == netRender_NEW) clients[index].status = netRender_READY;
 					WriteLog("NetRender - new Client #" + QString::number(index) + "(" + clients[index].name
-							+ " - " + clients[index].socket->peerAddress().toString() + ")", 1);
+										 + " - " + clients[index].socket->peerAddress().toString() + ")",
+						1);
 					emit ClientsChanged(index);
 
 					if (systemData.noGui)
@@ -640,16 +732,19 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 						out << " CPUs: " + QString::number(clients[index].clientWorkerCount) + "\n";
 					}
 
-					// when the client connects while a render is in progress, send the current job to the client
-					if(msgCurrentJob.command != netRender_NONE)
+					// when the client connects while a render is in progress, send the current job to the
+					// client
+					if (msgCurrentJob.command != netRender_NONE)
 					{
 						SendData(clients[index].socket, msgCurrentJob);
 						clients[index].linesRendered = 0;
+						WriteLog("CNetRender::ProcessData(): Send data at reconnect", 3);
 					}
 					break;
 				}
 				case netRender_DATA:
 				{
+					WriteLog("NetRender - ProcessData(), command DATA", 3);
 					if (inMsg->id == actualId)
 					{
 						QDataStream stream(&inMsg->payload, QIODevice::ReadOnly);
@@ -668,11 +763,16 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 							stream.readRawData(lineData.data(), lineData.size());
 							receivedLineNumbers.append(line);
 							receivedRenderedLines.append(lineData);
+							WriteLog(
+								QString("NetRender - ProcessData(), command DATA, line %1, lineDataLength %2")
+									.arg(line)
+									.arg(lineLength),
+								3);
 						}
 						clients[index].linesRendered += receivedLineNumbers.size();
 						emit NewLinesArrived(receivedLineNumbers, receivedRenderedLines);
 
-						//send acknowledge
+						// send acknowledge
 						sMessage outMsg;
 						outMsg.id = actualId;
 						outMsg.command = netRender_ACK;
@@ -686,13 +786,14 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 				}
 				case netRender_STATUS:
 				{
-					clients[index].status = (netRenderStatus) *(qint32*) inMsg->payload.data();
+					WriteLog("NetRender - ProcessData(), command STATUS", 3);
+					clients[index].status =
+						netRenderStatus(*reinterpret_cast<qint32 *>(inMsg->payload.data()));
 					emit ClientsChanged(index);
 					break;
 				}
 
-				default:
-					break;
+				default: break;
 			}
 		}
 		else
@@ -703,16 +804,16 @@ void CNetRender::ProcessData(QTcpSocket *socket, sMessage *inMsg)
 	ResetMessage(inMsg);
 }
 
-//send rendered lines
-void CNetRender::SendRenderedLines(QList<int> lineNumbers, QList<QByteArray> lines)
+// send rendered lines
+void CNetRender::SendRenderedLines(QList<int> lineNumbers, QList<QByteArray> lines) const
 {
 	sMessage msg;
 	msg.command = netRender_DATA;
 	QDataStream stream(&msg.payload, QIODevice::WriteOnly);
 	for (int i = 0; i < lineNumbers.size(); i++)
 	{
-		stream << (qint32) lineNumbers.at(i);
-		stream << (qint32) lines.at(i).size();
+		stream << qint32(lineNumbers.at(i));
+		stream << qint32(lines.at(i).size());
 		stream.writeRawData(lines.at(i).data(), lines.at(i).size());
 	}
 	SendData(clientSocket, msg);
@@ -729,8 +830,8 @@ void CNetRender::Stop()
 	}
 }
 
-void CNetRender::SetCurrentJob(cParameterContainer settings, cFractalContainer fractal,
-		QStringList listOfTextures)
+void CNetRender::SetCurrentJob(
+	cParameterContainer settings, cFractalContainer fractal, QStringList listOfTextures)
 {
 	WriteLog("NetRender - Sending job", 2);
 	cSettings settingsData(cSettings::formatNetRender);
@@ -742,33 +843,33 @@ void CNetRender::SetCurrentJob(cParameterContainer settings, cFractalContainer f
 		QDataStream stream(&msgCurrentJob.payload, QIODevice::WriteOnly);
 
 		// write settings
-		stream << (qint32) settingsText.toUtf8().size();
+		stream << qint32(settingsText.toUtf8().size());
 		stream.writeRawData(settingsText.toUtf8().data(), settingsText.toUtf8().size());
 
-		//send number of textures
-		stream << (qint32)listOfTextures.size();
+		// send number of textures
+		stream << qint32(listOfTextures.size());
 
 		// write textures (from files)
 		for (int i = 0; i < listOfTextures.size(); i++)
 		{
-			//send length of texture name
-			stream << (qint32)listOfTextures[i].toUtf8().size();
+			// send length of texture name
+			stream << qint32(listOfTextures[i].toUtf8().size());
 
-			//send texture name
+			// send texture name
 			stream.writeRawData(listOfTextures[i].toUtf8(), listOfTextures[i].length());
 
 			QFile file(listOfTextures[i]);
 			if (file.open(QIODevice::ReadOnly))
 			{
 				QByteArray buffer = file.readAll();
-				//send size of file
-				stream << (qint32) buffer.size();
-				//send file content
+				// send size of file
+				stream << qint32(buffer.size());
+				// send file content
 				stream.writeRawData(buffer.data(), buffer.size());
 				continue;
 			}
 
-			stream << (qint32) 0; // empty entry
+			stream << qint32(0); // empty entry
 		}
 
 		for (int i = 0; i < clients.size(); i++)
@@ -781,11 +882,11 @@ void CNetRender::SetCurrentJob(cParameterContainer settings, cFractalContainer f
 
 void CNetRender::NotifyStatus()
 {
-	if (clientSocket != NULL)
+	if (clientSocket != nullptr)
 	{
 		sMessage outMsg;
 		outMsg.command = netRender_STATUS;
-		outMsg.payload.append((char*) &status, sizeof(qint32));
+		outMsg.payload.append(reinterpret_cast<char *>(&status), sizeof(qint32));
 		SendData(clientSocket, outMsg);
 	}
 	emit NewStatusClient();
@@ -798,18 +899,18 @@ void CNetRender::SendToDoList(int clientIndex, QList<int> done)
 		sMessage msg;
 		msg.command = netRender_RENDER;
 		QDataStream stream(&msg.payload, QIODevice::WriteOnly);
-		stream << (qint32) done.size();
+		stream << qint32(done.size());
 		for (int i = 0; i < done.size(); i++)
 		{
-			stream << (qint32) done.at(i);
+			stream << qint32(done.at(i));
 		}
 		SendData(clients[clientIndex].socket, msg);
 	}
 	else
 	{
-		qCritical()
-				<< "CNetRender::SendToDoList(int clientIndex, QList<int> done, QList<int> startPositions): Client index out of range:"
-				<< clientIndex;
+		qCritical() << "CNetRender::SendToDoList(int clientIndex, QList<int> done, QList<int> "
+									 "startPositions): Client index out of range:"
+								<< clientIndex;
 	}
 }
 
@@ -826,20 +927,37 @@ void CNetRender::SendSetup(int clientIndex, int id, QList<int> startingPositions
 		sMessage msg;
 		msg.command = netRender_SETUP;
 		QDataStream stream(&msg.payload, QIODevice::WriteOnly);
-		stream << (qint32) id;
-		stream << (qint32) startingPositions.size();
+		stream << qint32(id);
+		stream << qint32(startingPositions.size());
 		for (int i = 0; i < startingPositions.size(); i++)
 		{
-			stream << (qint32) startingPositions.at(i);
+			stream << qint32(startingPositions.at(i));
 		}
 		SendData(clients[clientIndex].socket, msg);
 		actualId = id;
 	}
 	else
 	{
-		qCritical()
-				<< "CNetRender::SendSetup(int clientIndex, int id, QList<int> startingPositions): Client index out of range:"
-				<< clientIndex;
+		qCritical() << "CNetRender::SendSetup(int clientIndex, int id, QList<int> startingPositions): "
+									 "Client index out of range:"
+								<< clientIndex;
+	}
+}
+
+void CNetRender::KickAndKillClient(int clientIndex)
+{
+	WriteLog("NetRender - kick and kill client", 2);
+	if (clientIndex < clients.size())
+	{
+		sMessage msg;
+		msg.command = netRender_KICK_AND_KILL;
+		SendData(clients[clientIndex].socket, msg);
+	}
+	else
+	{
+		qCritical() << "CNetRender::KickAndKillClient(int clientIndex): "
+									 "Client index out of range:"
+								<< clientIndex;
 	}
 }
 
@@ -847,18 +965,12 @@ QString CNetRender::GetStatusText(netRenderStatus displayStatus)
 {
 	switch (displayStatus)
 	{
-		case CNetRender::netRender_DISABLED:
-			return tr("DISABLED");
-		case CNetRender::netRender_READY:
-			return tr("READY");
-		case CNetRender::netRender_WORKING:
-			return tr("WORKING");
-		case CNetRender::netRender_NEW:
-			return tr("NEW");
-		case CNetRender::netRender_CONNECTING:
-			return tr("(RE-)CONNECTING");
-		case CNetRender::netRender_ERROR:
-			return tr("ERROR");
+		case CNetRender::netRender_DISABLED: return tr("DISABLED");
+		case CNetRender::netRender_READY: return tr("READY");
+		case CNetRender::netRender_WORKING: return tr("WORKING");
+		case CNetRender::netRender_NEW: return tr("NEW");
+		case CNetRender::netRender_CONNECTING: return tr("(RE-)CONNECTING");
+		case CNetRender::netRender_ERROR: return tr("ERROR");
 	}
 	return tr("UNKNOWN");
 }
@@ -867,23 +979,17 @@ QString CNetRender::GetStatusColor(netRenderStatus displayStatus)
 {
 	switch (displayStatus)
 	{
-		case CNetRender::netRender_DISABLED:
-			return "darkgrey";
-		case CNetRender::netRender_READY:
-			return "green";
-		case CNetRender::netRender_WORKING:
-			return "darkblue";
-		case CNetRender::netRender_NEW:
-			return "purple";
-		case CNetRender::netRender_CONNECTING:
-			return "orange";
-		case CNetRender::netRender_ERROR:
-			return "red";
+		case CNetRender::netRender_DISABLED: return "darkgrey";
+		case CNetRender::netRender_READY: return "darkgreen";
+		case CNetRender::netRender_WORKING: return "darkblue";
+		case CNetRender::netRender_NEW: return "darkpurple";
+		case CNetRender::netRender_CONNECTING: return "darkorange";
+		case CNetRender::netRender_ERROR: return "darkred";
 	}
 	return "red";
 }
 
-const CNetRender::sClient& CNetRender::GetClient(int index)
+const CNetRender::sClient &CNetRender::GetClient(int index)
 {
 	if (index >= 0 && index < GetClientCount())
 	{
@@ -892,7 +998,7 @@ const CNetRender::sClient& CNetRender::GetClient(int index)
 	else
 	{
 		qCritical() << "CNetRender::sClient& CNetRender::getClient(int index): client " << index
-				<< " doesn't exist";
+								<< " doesn't exist";
 		return nullClient;
 	}
 }
@@ -921,7 +1027,7 @@ bool CNetRender::WaitForAllClientsReady(double timeout)
 			if (GetClientStatus(i) != netRender_READY)
 			{
 				allReady = false;
-				WriteLog(QString("Client # %1 is not ready yet").arg(i), 2);
+				WriteLog(QString("Client # %1 is not ready yet").arg(i), 1);
 				break;
 			}
 		}
@@ -955,9 +1061,9 @@ bool CNetRender::Block()
 
 bool CNetRender::CompareMajorVersion(qint32 version1, qint32 version2)
 {
-  qint32 majorVersion1 = version1 / 10;
-  qint32 majorVersion2 = version2 / 10;
-  return majorVersion1 == majorVersion2;
+	qint32 majorVersion1 = version1 / 10;
+	qint32 majorVersion2 = version2 / 10;
+	return majorVersion1 == majorVersion2;
 }
 
 QByteArray *CNetRender::GetTexture(QString textureName)
