@@ -1,29 +1,45 @@
 /**
- * Mandelbulber v2, a 3D fractal generator
+ * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
+ *                                             ,B" ]L,,p%%%,,,§;, "K
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
+ * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
+ *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
+ * Mandelbulber is free software:     §R.ß~-Q/M=,=5"v"]=Qf,'§"M= =,M.§ Rz]M"Kw
+ * you can redistribute it and/or     §w "xDY.J ' -"m=====WeC=\ ""%""y=%"]"" §
+ * modify it under the terms of the    "§M=M =D=4"N #"%==A%p M§ M6  R' #"=~.4M
+ * GNU General Public License as        §W =, ][T"]C  §  § '§ e===~ U  !§[Z ]N
+ * published by the                    4M",,Jm=,"=e~  §  §  j]]""N  BmM"py=ßM
+ * Free Software Foundation,          ]§ T,M=& 'YmMMpM9MMM%=w=,,=MT]M m§;'§,
+ * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
+ * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
+ * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
+ * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
+ * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
+ * but WITHOUT ANY WARRANTY;                            .'''
+ * without even the implied warranty
+ * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- * cParameterContainer class - multipurpose parameter container
+ * See the GNU General Public License for more details.
+ * You should have received a copy of the GNU General Public License
+ * along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2014 Krzysztof Marczak
- *
- * This file is part of Mandelbulber.
- *
- * Mandelbulber is free software: you can redistribute it and/or modify it under the terms of the
- * GNU General Public License as published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * Mandelbulber is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * See the GNU General Public License for more details. You should have received a copy of the GNU
- * General Public License along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
+ * ###########################################################################
  *
  * Authors: Krzysztof Marczak (buddhi1980@gmail.com)
+ *
+ * cParameterContainer class - multipurpose parameter container
  */
 
 #include "parameters.hpp"
-#include <QtAlgorithms>
+
 #include <algorithm>
 
+#include <QtAlgorithms>
+
+#include "nine_fractals.hpp"
+
 //#define _PARAM_DEBUG
 
 using namespace parameterContainer;
@@ -38,17 +54,29 @@ cParameterContainer::~cParameterContainer()
 	myMap.clear();
 }
 
-//defining of params without limits
-template<class T>
-void cParameterContainer::addParam(QString name, T defaultVal, enumMorphType morphType,
-		enumParameterType parType)
+cParameterContainer &cParameterContainer::operator=(const cParameterContainer &par)
 {
+	QMutexLocker lock(&m_lock);
+
+	myMap = par.myMap;
+	containerName = par.containerName;
+	return *this;
+}
+
+// defining of params without limits
+template <class T>
+void cParameterContainer::addParam(QString name, T defaultVal, enumMorphType morphType,
+	enumParameterType parType, QStringList enumLookup)
+{
+	QMutexLocker lock(&m_lock);
+
 	cOneParameter newRecord;
 	newRecord.Set(defaultVal, valueDefault);
 	newRecord.Set(defaultVal, valueActual);
 	newRecord.SetMorphType(morphType);
 	newRecord.SetParameterType(parType);
 	newRecord.SetOriginalContainerName(containerName);
+	newRecord.SetEnumLookup(enumLookup);
 
 	if (myMap.find(name) != myMap.end())
 	{
@@ -60,27 +88,29 @@ void cParameterContainer::addParam(QString name, T defaultVal, enumMorphType mor
 	}
 }
 template void cParameterContainer::addParam<double>(QString name, double defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<int>(QString name, int defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<QString>(QString name, QString defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<CVector3>(QString name, CVector3 defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<CVector4>(QString name, CVector4 defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<sRGB>(QString name, sRGB defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<bool>(QString name, bool defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<cColorPalette>(QString name, cColorPalette defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 
-//defining of params with limits
-template<class T>
+// defining of params with limits
+template <class T>
 void cParameterContainer::addParam(QString name, T defaultVal, T minVal, T maxVal,
-		enumMorphType morphType, enumParameterType parType)
+	enumMorphType morphType, enumParameterType parType)
 {
+	QMutexLocker lock(&m_lock);
+
 	cOneParameter newRecord;
 	newRecord.Set(defaultVal, valueDefault);
 	newRecord.Set(defaultVal, valueActual);
@@ -100,21 +130,23 @@ void cParameterContainer::addParam(QString name, T defaultVal, T minVal, T maxVa
 	}
 }
 template void cParameterContainer::addParam<double>(QString name, double defaultVal, double minVal,
-		double maxVal, enumMorphType morphType, enumParameterType parType);
+	double maxVal, enumMorphType morphType, enumParameterType parType);
 template void cParameterContainer::addParam<int>(QString name, int defaultVal, int minVal,
-		int maxVal, enumMorphType morphType, enumParameterType parType);
+	int maxVal, enumMorphType morphType, enumParameterType parType);
 template void cParameterContainer::addParam<CVector3>(QString name, CVector3 defaultVal,
-		CVector3 minVal, CVector3 maxVal, enumMorphType morphType, enumParameterType parType);
+	CVector3 minVal, CVector3 maxVal, enumMorphType morphType, enumParameterType parType);
 template void cParameterContainer::addParam<CVector4>(QString name, CVector4 defaultVal,
-		CVector4 minVal, CVector4 maxVal, enumMorphType morphType, enumParameterType parType);
+	CVector4 minVal, CVector4 maxVal, enumMorphType morphType, enumParameterType parType);
 template void cParameterContainer::addParam<sRGB>(QString name, sRGB defaultVal, sRGB minVal,
-		sRGB maxVal, enumMorphType morphType, enumParameterType parType);
+	sRGB maxVal, enumMorphType morphType, enumParameterType parType);
 
-//defining of params without limits and with index
-template<class T>
+// defining of params without limits and with index
+template <class T>
 void cParameterContainer::addParam(QString name, int index, T defaultVal, enumMorphType morphType,
-		enumParameterType parType)
+	enumParameterType parType, QStringList enumLookup)
 {
+	QMutexLocker lock(&m_lock);
+
 	if (index >= 0)
 	{
 		cOneParameter newRecord;
@@ -123,6 +155,7 @@ void cParameterContainer::addParam(QString name, int index, T defaultVal, enumMo
 		newRecord.SetMorphType(morphType);
 		newRecord.SetParameterType(parType);
 		newRecord.SetOriginalContainerName(containerName);
+		newRecord.SetEnumLookup(enumLookup);
 
 		QString indexName = nameWithIndex(&name, index);
 		if (myMap.find(indexName) != myMap.end())
@@ -137,29 +170,31 @@ void cParameterContainer::addParam(QString name, int index, T defaultVal, enumMo
 	else
 	{
 		qWarning() << "addParam(): element '" << name << "' has negative index (" << index << ")"
-				<< endl;
+							 << endl;
 	}
 }
 template void cParameterContainer::addParam<double>(QString name, int index, double defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<int>(QString name, int index, int defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<QString>(QString name, int index, QString defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<CVector3>(QString name, int index, CVector3 defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<CVector4>(QString name, int index, CVector4 defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<sRGB>(QString name, int index, sRGB defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 template void cParameterContainer::addParam<bool>(QString name, int index, bool defaultVal,
-		enumMorphType morphType, enumParameterType parType);
+	enumMorphType morphType, enumParameterType parType, QStringList enumLookup);
 
-//defining of params with limits and index
-template<class T>
+// defining of params with limits and index
+template <class T>
 void cParameterContainer::addParam(QString name, int index, T defaultVal, T minVal, T maxVal,
-		enumMorphType morphType, enumParameterType parType)
+	enumMorphType morphType, enumParameterType parType)
 {
+	QMutexLocker lock(&m_lock);
+
 	if (index >= 0)
 	{
 		cOneParameter newRecord;
@@ -184,24 +219,26 @@ void cParameterContainer::addParam(QString name, int index, T defaultVal, T minV
 	else
 	{
 		qWarning() << "addParam(): element '" << name << "' has negative index (" << index << ")"
-				<< endl;
+							 << endl;
 	}
 }
 template void cParameterContainer::addParam<double>(QString name, int index, double defaultVal,
-		double minVal, double maxVal, enumMorphType morphType, enumParameterType parType);
+	double minVal, double maxVal, enumMorphType morphType, enumParameterType parType);
 template void cParameterContainer::addParam<int>(QString name, int index, int defaultVal,
-		int minVal, int maxVal, enumMorphType morphType, enumParameterType parType);
+	int minVal, int maxVal, enumMorphType morphType, enumParameterType parType);
 template void cParameterContainer::addParam<CVector3>(QString name, int index, CVector3 defaultVal,
-		CVector3 minVal, CVector3 maxVal, enumMorphType morphType, enumParameterType parType);
+	CVector3 minVal, CVector3 maxVal, enumMorphType morphType, enumParameterType parType);
 template void cParameterContainer::addParam<CVector4>(QString name, int index, CVector4 defaultVal,
-		CVector4 minVal, CVector4 maxVal, enumMorphType morphType, enumParameterType parType);
+	CVector4 minVal, CVector4 maxVal, enumMorphType morphType, enumParameterType parType);
 template void cParameterContainer::addParam<sRGB>(QString name, int index, sRGB defaultVal,
-		sRGB minVal, sRGB maxVal, enumMorphType morphType, enumParameterType parType);
+	sRGB minVal, sRGB maxVal, enumMorphType morphType, enumParameterType parType);
 
-//set parameter value by name
-template<class T>
+// set parameter value by name
+template <class T>
 void cParameterContainer::Set(QString name, T val)
 {
+	QMutexLocker lock(&m_lock);
+
 	QMap<QString, cOneParameter>::iterator it;
 	it = myMap.find(name);
 	if (it != myMap.end())
@@ -222,10 +259,12 @@ template void cParameterContainer::Set<sRGB>(QString name, sRGB val);
 template void cParameterContainer::Set<bool>(QString name, bool val);
 template void cParameterContainer::Set<cColorPalette>(QString name, cColorPalette val);
 
-//set parameter value by name and index
-template<class T>
+// set parameter value by name and index
+template <class T>
 void cParameterContainer::Set(QString name, int index, T val)
 {
+	QMutexLocker lock(&m_lock);
+
 	if (index >= 0)
 	{
 		QString indexName = nameWithIndex(&name, index);
@@ -253,10 +292,12 @@ template void cParameterContainer::Set<CVector4>(QString name, int index, CVecto
 template void cParameterContainer::Set<sRGB>(QString name, int index, sRGB val);
 template void cParameterContainer::Set<bool>(QString name, int index, bool val);
 
-//get parameter value by name
-template<class T>
+// get parameter value by name
+template <class T>
 T cParameterContainer::Get(QString name) const
 {
+	QMutexLocker lock(&m_lock);
+
 	QMap<QString, cOneParameter>::const_iterator it;
 	it = myMap.find(name);
 	T val = T();
@@ -279,10 +320,12 @@ template sRGB cParameterContainer::Get<sRGB>(QString name) const;
 template bool cParameterContainer::Get<bool>(QString name) const;
 template cColorPalette cParameterContainer::Get<cColorPalette>(QString name) const;
 
-//get parameter value by name and index
-template<class T>
+// get parameter value by name and index
+template <class T>
 T cParameterContainer::Get(QString name, int index) const
 {
+	QMutexLocker lock(&m_lock);
+
 	T val = T();
 	if (index >= 0)
 	{
@@ -312,10 +355,12 @@ template CVector4 cParameterContainer::Get<CVector4>(QString name, int index) co
 template sRGB cParameterContainer::Get<sRGB>(QString name, int index) const;
 template bool cParameterContainer::Get<bool>(QString name, int index) const;
 
-//get parameter default value by name
-template<class T>
+// get parameter default value by name
+template <class T>
 T cParameterContainer::GetDefault(QString name) const
 {
+	QMutexLocker lock(&m_lock);
+
 	QMap<QString, cOneParameter>::const_iterator it;
 	it = myMap.find(name);
 	T val = T();
@@ -338,10 +383,12 @@ template sRGB cParameterContainer::GetDefault<sRGB>(QString name) const;
 template bool cParameterContainer::GetDefault<bool>(QString name) const;
 template cColorPalette cParameterContainer::GetDefault<cColorPalette>(QString name) const;
 
-//get parameter value by name and index
-template<class T>
+// get parameter value by name and index
+template <class T>
 T cParameterContainer::GetDefault(QString name, int index) const
 {
+	QMutexLocker lock(&m_lock);
+
 	T val = T();
 	if (index >= 0)
 	{
@@ -360,7 +407,7 @@ T cParameterContainer::GetDefault(QString name, int index) const
 	else
 	{
 		qWarning() << "GetDefault(): element '" << name << "' has negative index (" << index << ")"
-				<< endl;
+							 << endl;
 	}
 	return val;
 }
@@ -372,20 +419,22 @@ template CVector4 cParameterContainer::GetDefault<CVector4>(QString name, int in
 template sRGB cParameterContainer::GetDefault<sRGB>(QString name, int index) const;
 template bool cParameterContainer::GetDefault<bool>(QString name, int index) const;
 
-QString cParameterContainer::nameWithIndex(QString *str, int index) const
+QString cParameterContainer::nameWithIndex(QString *str, int index)
 {
 	QString name = *str + "_" + QString::number(index);
 	return name;
 }
 
-void cParameterContainer::Copy(QString name, const cParameterContainer &sourceContainer)
+void cParameterContainer::Copy(QString name, const cParameterContainer *sourceContainer)
 {
+	QMutexLocker lock(&m_lock);
+
 	QMap<QString, cOneParameter>::const_iterator itSource;
 	QMap<QString, cOneParameter>::iterator itDest;
 	itDest = myMap.find(name);
 	if (itDest != myMap.end())
 	{
-		itSource = sourceContainer.myMap.find(name);
+		itSource = sourceContainer->myMap.find(name);
 		if (itSource != myMap.end())
 		{
 			itDest.value() = itSource.value();
@@ -401,15 +450,34 @@ void cParameterContainer::Copy(QString name, const cParameterContainer &sourceCo
 	}
 }
 
-QList<QString> cParameterContainer::GetListOfParameters(void) const
+QList<QString> cParameterContainer::GetListOfParameters() const
 {
+	QMutexLocker lock(&m_lock);
+
 	QList<QString> list = myMap.keys();
 	std::sort(list.begin(), list.end(), compareStrings);
 	return list;
 }
 
+void cParameterContainer::PrintListOfParameters() const
+{
+	QString parametersOutput = "Non-Default Parameters for Rendered Example Settings\n";
+	QList<QString> listOfParameters = this->GetListOfParameters();
+	for (int i = 0; i < listOfParameters.size(); i++)
+	{
+		const QString parameterName = listOfParameters.at(i);
+		const QString parameterValue = this->Get<QString>(parameterName);
+		const QString defaultValue = this->GetDefault<QString>(parameterName);
+		if (parameterValue != defaultValue)
+			parametersOutput += parameterName + "=" + parameterValue + "\n";
+	}
+	WriteLog(parametersOutput, 1);
+}
+
 enumVarType cParameterContainer::GetVarType(QString name) const
 {
+	QMutexLocker lock(&m_lock);
+
 	enumVarType type = typeNull;
 
 	QMap<QString, cOneParameter>::const_iterator it;
@@ -427,6 +495,8 @@ enumVarType cParameterContainer::GetVarType(QString name) const
 
 enumParameterType cParameterContainer::GetParameterType(QString name) const
 {
+	QMutexLocker lock(&m_lock);
+
 	enumParameterType type = paramStandard;
 
 	QMap<QString, cOneParameter>::const_iterator it;
@@ -444,6 +514,8 @@ enumParameterType cParameterContainer::GetParameterType(QString name) const
 
 bool cParameterContainer::isDefaultValue(QString name) const
 {
+	QMutexLocker lock(&m_lock);
+
 	bool isDefault = true;
 
 	QMap<QString, cOneParameter>::const_iterator it;
@@ -455,19 +527,21 @@ bool cParameterContainer::isDefaultValue(QString name) const
 	else
 	{
 		qWarning() << "cParameterContainer::isDefaultValue(QString name): element '" << name
-				<< "' doesn't exists" << endl;
+							 << "' doesn't exists" << endl;
 	}
 	return isDefault;
 }
 
-void cParameterContainer::ResetAllToDefault(void)
+void cParameterContainer::ResetAllToDefault()
 {
+	QMutexLocker lock(&m_lock);
+
 	QMap<QString, cOneParameter>::iterator it = myMap.begin();
 	while (it != myMap.end())
 	{
 		cOneParameter record = it.value();
-		if (record.GetParameterType() != paramApp) it.value().SetMultival(record.GetMultival(valueDefault),
-																																			valueActual);
+		if (record.GetParameterType() != paramApp)
+			it.value().SetMultiVal(record.GetMultiVal(valueDefault), valueActual);
 		++it;
 	}
 }
@@ -488,6 +562,8 @@ bool cParameterContainer::IfExists(const QString &name) const
 
 void cParameterContainer::DeleteParameter(const QString &name)
 {
+	QMutexLocker lock(&m_lock);
+
 	QMap<QString, cOneParameter>::iterator it;
 	it = myMap.find(name);
 	if (it != myMap.end())
@@ -502,6 +578,8 @@ void cParameterContainer::DeleteParameter(const QString &name)
 
 cOneParameter cParameterContainer::GetAsOneParameter(QString name) const
 {
+	QMutexLocker lock(&m_lock);
+
 	QMap<QString, cOneParameter>::const_iterator it;
 	it = myMap.find(name);
 	cOneParameter val;
@@ -512,13 +590,15 @@ cOneParameter cParameterContainer::GetAsOneParameter(QString name) const
 	else
 	{
 		qWarning() << "cParameterContainer::GetAsOneParameter(QString name): element '" << name
-				<< "' doesn't exists" << endl;
+							 << "' doesn't exists" << endl;
 	}
 	return val;
 }
 
 void cParameterContainer::SetFromOneParameter(QString name, const cOneParameter &parameter)
 {
+	QMutexLocker lock(&m_lock);
+
 	QMap<QString, cOneParameter>::iterator it;
 	it = myMap.find(name);
 	if (it != myMap.end())
@@ -527,19 +607,21 @@ void cParameterContainer::SetFromOneParameter(QString name, const cOneParameter
 	}
 	else
 	{
-		qWarning()
-				<< "cParameterContainer::SetFromOneParameter(QString name, const cOneParameter &parameter): element '"
-				<< name << "' doesn't exists" << endl;
+		qWarning() << "cParameterContainer::SetFromOneParameter(QString name, const cOneParameter "
+									"&parameter): element '"
+							 << name << "' doesn't exists" << endl;
 	}
 }
 
 void cParameterContainer::AddParamFromOneParameter(QString name, const cOneParameter &parameter)
 {
+	QMutexLocker lock(&m_lock);
+
 	if (myMap.find(name) != myMap.end())
 	{
-		qWarning()
-				<< "cParameterContainer::AddParamFromOneParameter(QString name, const cOneParameter &parameter): element '"
-				<< name << "' already existed" << endl;
+		qWarning() << "cParameterContainer::AddParamFromOneParameter(QString name, const cOneParameter "
+									"&parameter): element '"
+							 << name << "' already existed" << endl;
 	}
 	else
 	{
