@@ -1,31 +1,44 @@
 /**
- * Mandelbulber v2, a 3D fractal generator
+ * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
+ *                                             ,B" ]L,,p%%%,,,§;, "K
+ * Copyright (C) 2014-18 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
+ * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
+ *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
+ * Mandelbulber is free software:     §R.ß~-Q/M=,=5"v"]=Qf,'§"M= =,M.§ Rz]M"Kw
+ * you can redistribute it and/or     §w "xDY.J ' -"m=====WeC=\ ""%""y=%"]"" §
+ * modify it under the terms of the    "§M=M =D=4"N #"%==A%p M§ M6  R' #"=~.4M
+ * GNU General Public License as        §W =, ][T"]C  §  § '§ e===~ U  !§[Z ]N
+ * published by the                    4M",,Jm=,"=e~  §  §  j]]""N  BmM"py=ßM
+ * Free Software Foundation,          ]§ T,M=& 'YmMMpM9MMM%=w=,,=MT]M m§;'§,
+ * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
+ * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
+ * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
+ * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
+ * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
+ * but WITHOUT ANY WARRANTY;                            .'''
+ * without even the implied warranty
+ * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- * definitions of primitive objects
+ * See the GNU General Public License for more details.
+ * You should have received a copy of the GNU General Public License
+ * along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2014 Krzysztof Marczak
- *
- * This file is part of Mandelbulber.
- *
- * Mandelbulber is free software: you can redistribute it and/or modify it under the terms of the
- * GNU General Public License as published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * Mandelbulber is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * See the GNU General Public License for more details. You should have received a copy of the GNU
- * General Public License along with Mandelbulber. If not, see <http://www.gnu.org/licenses/>.
+ * ###########################################################################
  *
  * Authors: Krzysztof Marczak (buddhi1980@gmail.com)
+ *
+ * definitions of primitive objects
  */
 
 #include "primitives.h"
+
+#include <QtAlgorithms>
+
 #include "displacement_map.hpp"
+#include "parameters.hpp"
 #include "system.hpp"
-//#include "common_math.h"
-//#include "render_worker.hpp"
-//#include "calculate_distance.hpp"
 
 using namespace fractal;
 
@@ -33,43 +46,42 @@ QString PrimitiveNames(enumObjectType primitiveType)
 {
 	switch (primitiveType)
 	{
-		case objPlane:
-			return "plane";
-		case objWater:
-			return "water";
-		case objSphere:
-			return "sphere";
-		case objBox:
-			return "box";
-		case objRectangle:
-			return "rectangle";
-		case objCircle:
-			return "circle";
-		case objCone:
-			return "cone";
-		case objCylinder:
-			return "cylinder";
-		case objTorus:
-			return "torus";
-		default:
-			return "";
+		case objPlane: return "plane";
+		case objWater: return "water";
+		case objSphere: return "sphere";
+		case objBox: return "box";
+		case objRectangle: return "rectangle";
+		case objCircle: return "circle";
+		case objCone: return "cone";
+		case objCylinder: return "cylinder";
+		case objTorus: return "torus";
+		default: return "";
 	}
 }
 
 enumObjectType PrimitiveNameToEnum(const QString &primitiveType)
 {
 	enumObjectType type = objNone;
-	if (primitiveType == QString("plane")) type = objPlane;
-	else if (primitiveType == QString("water")) type = objWater;
-	else if (primitiveType == QString("sphere")) type = objSphere;
-	else if (primitiveType == QString("box")) type = objBox;
-	else if (primitiveType == QString("rectangle")) type = objRectangle;
-	else if (primitiveType == QString("circle")) type = objCircle;
-	else if (primitiveType == QString("cone")) type = objCone;
-	else if (primitiveType == QString("cylinder")) type = objCylinder;
-	else if (primitiveType == QString("torus")) type = objTorus;
+	if (primitiveType == QString("plane"))
+		type = objPlane;
+	else if (primitiveType == QString("water"))
+		type = objWater;
+	else if (primitiveType == QString("sphere"))
+		type = objSphere;
+	else if (primitiveType == QString("box"))
+		type = objBox;
+	else if (primitiveType == QString("rectangle"))
+		type = objRectangle;
+	else if (primitiveType == QString("circle"))
+		type = objCircle;
+	else if (primitiveType == QString("cone"))
+		type = objCone;
+	else if (primitiveType == QString("cylinder"))
+		type = objCylinder;
+	else if (primitiveType == QString("torus"))
+		type = objTorus;
 	else
-	qCritical() << "Wrong primitive name: " << primitiveType;
+		qCritical() << "Wrong primitive name: " << primitiveType;
 
 	return type;
 }
@@ -82,7 +94,7 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 	QList<QString> listOfParameters = par->GetListOfParameters();
 	QList<sPrimitiveItem> listOfPrimitives;
 
-	//generating fresh list of primitives based of parameter list
+	// generating fresh list of primitives based of parameter list
 	for (int i = 0; i < listOfParameters.size(); i++)
 	{
 		QString parameterName = listOfParameters.at(i);
@@ -90,13 +102,13 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 		{
 			isAnyPrimitive = true;
 
-			QString primitiveName = parameterName.left(parameterName.lastIndexOf('_'));
 			QStringList split = parameterName.split('_');
+			QString primitiveName = split.at(0) + "_" + split.at(1) + "_" + split.at(2);
 			QString typeName = split.at(1);
 			fractal::enumObjectType type = PrimitiveNameToEnum(typeName);
 			int index = split.at(2).toInt();
 
-			//check if item is already on the list
+			// check if item is already on the list
 			bool found = false;
 			for (int l = 0; l < listOfPrimitives.size(); l++)
 			{
@@ -118,14 +130,14 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 		sPrimitiveItem item = listOfPrimitives.at(i);
 
 		using namespace fractal;
-		sPrimitiveBasic* primitive;
+		sPrimitiveBasic *primitive;
 
 		switch (item.type)
 		{
 			case objPlane:
 			{
 				primitive = new sPrimitivePlane;
-				sPrimitivePlane *obj = (sPrimitivePlane *) primitive;
+				sPrimitivePlane *obj = static_cast<sPrimitivePlane *>(primitive);
 				obj->empty = par->Get<bool>(item.name + "_empty");
 				obj->size = CVector3(1.0, 1.0, 1.0);
 				break;
@@ -133,7 +145,7 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 			case objBox:
 			{
 				primitive = new sPrimitiveBox;
-				sPrimitiveBox *obj = (sPrimitiveBox *) primitive;
+				sPrimitiveBox *obj = static_cast<sPrimitiveBox *>(primitive);
 				obj->empty = par->Get<bool>(item.name + "_empty");
 				obj->rounding = par->Get<double>(item.name + "_rounding");
 				obj->repeat = par->Get<CVector3>(item.name + "_repeat");
@@ -143,7 +155,7 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 			case objSphere:
 			{
 				primitive = new sPrimitiveSphere;
-				sPrimitiveSphere *obj = (sPrimitiveSphere *) primitive;
+				sPrimitiveSphere *obj = static_cast<sPrimitiveSphere *>(primitive);
 				obj->empty = par->Get<bool>(item.name + "_empty");
 				obj->radius = par->Get<double>(item.name + "_radius");
 				obj->repeat = par->Get<CVector3>(item.name + "_repeat");
@@ -153,12 +165,15 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 			case objWater:
 			{
 				primitive = new sPrimitiveWater;
-				sPrimitiveWater *obj = (sPrimitiveWater *) primitive;
+				sPrimitiveWater *obj = static_cast<sPrimitiveWater *>(primitive);
 				obj->empty = par->Get<bool>(item.name + "_empty");
-				obj->amplitude = par->Get<double>(item.name + "_amplitude");
+				obj->relativeAmplitude = par->Get<double>(item.name + "_relative_amplitude");
 				obj->length = par->Get<double>(item.name + "_length");
 				obj->animSpeed = par->Get<double>(item.name + "_anim_speed");
 				obj->iterations = par->Get<int>(item.name + "_iterations");
+				obj->waveFromObjectsEnable = par->Get<bool>(item.name + "_wave_from_objects_enable");
+				obj->waveFromObjectsRelativeAmplitude =
+					par->Get<double>(item.name + "_wave_from_objects_relative_amplitude");
 				obj->animFrame = par->Get<int>("frame_no");
 				obj->size = CVector3(1.0, 1.0, 1.0);
 				break;
@@ -166,7 +181,7 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 			case objCone:
 			{
 				primitive = new sPrimitiveCone;
-				sPrimitiveCone *obj = (sPrimitiveCone *) primitive;
+				sPrimitiveCone *obj = static_cast<sPrimitiveCone *>(primitive);
 				obj->caps = par->Get<bool>(item.name + "_caps");
 				obj->empty = par->Get<bool>(item.name + "_empty");
 				obj->radius = par->Get<double>(item.name + "_radius");
@@ -180,7 +195,7 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 			case objCylinder:
 			{
 				primitive = new sPrimitiveCylinder;
-				sPrimitiveCylinder *obj = (sPrimitiveCylinder *) primitive;
+				sPrimitiveCylinder *obj = static_cast<sPrimitiveCylinder *>(primitive);
 				obj->caps = par->Get<bool>(item.name + "_caps");
 				obj->empty = par->Get<bool>(item.name + "_empty");
 				obj->radius = par->Get<double>(item.name + "_radius");
@@ -192,20 +207,21 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 			case objTorus:
 			{
 				primitive = new sPrimitiveTorus;
-				sPrimitiveTorus *obj = (sPrimitiveTorus *) primitive;
+				sPrimitiveTorus *obj = static_cast<sPrimitiveTorus *>(primitive);
 				obj->empty = par->Get<bool>(item.name + "_empty");
 				obj->radius = par->Get<double>(item.name + "_radius");
+				obj->radius_lpow = par->Get<double>(item.name + "_radius_lpow");
 				obj->tube_radius = par->Get<double>(item.name + "_tube_radius");
+				obj->tube_radius_lpow = par->Get<double>(item.name + "_tube_radius_lpow");
 				obj->repeat = par->Get<CVector3>(item.name + "_repeat");
-				obj->size = CVector3(	(obj->radius + obj->tube_radius) * 2.0,
-																(obj->radius + obj->tube_radius) * 2.0,
-																obj->tube_radius);
+				obj->size = CVector3((obj->radius + obj->tube_radius) * 2.0,
+					(obj->radius + obj->tube_radius) * 2.0, obj->tube_radius);
 				break;
 			}
 			case objCircle:
 			{
 				primitive = new sPrimitiveCircle;
-				sPrimitiveCircle *obj = (sPrimitiveCircle *) primitive;
+				sPrimitiveCircle *obj = static_cast<sPrimitiveCircle *>(primitive);
 				obj->radius = par->Get<double>(item.name + "_radius");
 				obj->size = CVector3(obj->radius * 2.0, obj->radius * 2.0, 1.0);
 				break;
@@ -213,7 +229,7 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 			case objRectangle:
 			{
 				primitive = new sPrimitiveRectangle;
-				sPrimitiveRectangle *obj = (sPrimitiveRectangle *) primitive;
+				sPrimitiveRectangle *obj = static_cast<sPrimitiveRectangle *>(primitive);
 				obj->height = par->Get<double>(item.name + "_height");
 				obj->width = par->Get<double>(item.name + "_width");
 				obj->size = CVector3(obj->width, obj->height, 1.0);
@@ -221,7 +237,8 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 			}
 			default:
 			{
-				qCritical() << "cannot handle " << PrimitiveNames(item.type) << " in cPrimitives::cPrimitives()";
+				qCritical() << "cannot handle " << PrimitiveNames(item.type)
+										<< " in cPrimitives::cPrimitives()";
 				return;
 			}
 		}
@@ -232,10 +249,10 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 		primitive->objectType = item.type;
 		primitive->SetRotation(par->Get<CVector3>(item.name + "_rotation"));
 		primitive->enable = par->Get<bool>(item.name + "_enabled");
-		//primitive->reflect = par->Get<double>(item.name + "_reflection");
-		//primitive->color = par->Get<sRGB>(item.name + "_color");
+		// primitive->reflect = par->Get<double>(item.name + "_reflection");
+		// primitive->color = par->Get<sRGB>(item.name + "_color");
 
-		if(objectData)
+		if (objectData)
 		{
 			objectData->append(*primitive);
 			primitive->objectId = objectData->size() - 1;
@@ -250,6 +267,11 @@ cPrimitives::cPrimitives(const cParameterContainer *par, QVector<cObjectData> *o
 	WriteLog("cPrimitives::cPrimitives(const cParameterContainer *par) finished", 2);
 }
 
+cPrimitives::~cPrimitives()
+{
+	qDeleteAll(allPrimitives);
+}
+
 double sPrimitivePlane::PrimitiveDistance(CVector3 _point) const
 {
 	CVector3 point = _point - position;
@@ -263,6 +285,7 @@ double sPrimitiveBox::PrimitiveDistance(CVector3 _point) const
 	CVector3 point = _point - position;
 	point = rotationMatrix.RotateVector(point);
 	point = point.mod(repeat);
+
 	if (empty)
 	{
 		double boxDist = -1e10;
@@ -318,8 +341,8 @@ double sPrimitiveCircle::PrimitiveDistance(CVector3 _point) const
 {
 	CVector3 point = _point - position;
 	point = rotationMatrix.RotateVector(point);
-	CVector2<double> circTemp(point.x, point.y);
-	double distTemp = circTemp.Length() - radius;
+	CVector2<double> circleTemp(point.x, point.y);
+	double distTemp = circleTemp.Length() - radius;
 	distTemp = max(fabs(point.z), distTemp);
 	return distTemp;
 }
@@ -341,32 +364,56 @@ double sPrimitiveCone::PrimitiveDistance(CVector3 _point) const
 
 double sPrimitiveWater::PrimitiveDistance(CVector3 _point) const
 {
-	//TODO to use rendering technique from here: //https://www.shadertoy.com/view/Ms2SD1
+	Q_UNUSED(_point);
+	return 0.0;
+}
+
+double sPrimitiveWater::PrimitiveDistanceWater(CVector3 _point, double distanceFromAnother) const
+{
+	// TODO to use rendering technique from here: //https://www.shadertoy.com/view/Ms2SD1
 
 	CVector3 point = _point - position;
 	point = rotationMatrix.RotateVector(point);
 
+	if (waveFromObjectsEnable)
+	{
+		point.x +=
+			length * 20.0 * exp(-distanceFromAnother / length / 5.0) * waveFromObjectsRelativeAmplitude;
+	}
+
+	double amplitude = length * relativeAmplitude;
+	double objectWave = 0.0;
+
 	double planeDistance = point.z;
 	if (planeDistance < amplitude * 10.0)
 	{
 		double phase = animSpeed * animFrame * 0.1;
+
+		if (waveFromObjectsEnable)
+		{
+			objectWave = sin(distanceFromAnother / length * 5.0 - phase * 2.0)
+									 * exp(-distanceFromAnother / length / 5.0) * waveFromObjectsRelativeAmplitude;
+		}
+
 		double k = 0.23;
-		double waveXtemp = point.x;
-		double waveYtemp = point.y;
-		double waveX = 0;
-		double waveY = 0;
+		double waveXTemp;
+		double waveYTemp;
+		double waveX = objectWave;
+		double waveY = -objectWave;
 		double p = 1.0;
 		double p2 = 0.05;
 		for (int i = 1; i <= iterations; i++)
 		{
 			float p3 = p * p2;
 			double shift = phase / (i / 3.0 + 1.0);
-			waveXtemp = sin(i + 0.4 * (waveX) * p3 + sin(k * point.y / length * p3)
-					+ point.x / length * p3 + shift) / p;
-			waveYtemp = cos(i + 0.4 * (waveY) * p3 + sin(point.x / length * p3)
-					+ k * point.y / length * p3 + shift * 0.23) / p;
-			waveX += waveXtemp;
-			waveY += waveYtemp;
+			waveXTemp =
+				sin(i + 0.4 * (waveX)*p3 + sin(k * point.y / length * p3) + point.x / length * p3 + shift)
+				/ p;
+			waveYTemp = cos(i + 0.4 * (waveY)*p3 + sin(point.x / length * p3) + k * point.y / length * p3
+											+ shift * 0.23)
+									/ p;
+			waveX += waveXTemp;
+			waveY += waveYTemp;
 			p2 = p2 + (1.0 - p2) * 0.7;
 			p *= 1.872;
 		}
@@ -382,13 +429,13 @@ double sPrimitiveTorus::PrimitiveDistance(CVector3 _point) const
 	point = rotationMatrix.RotateVector(point);
 	point = point.mod(repeat);
 
-	double d1 = sqrt(point.x * point.x + point.y * point.y) - radius;
-	double dist = sqrt(d1 * d1 + point.z * point.z) - tube_radius;
+	double d1 = CVector2<double>(point.x, point.y).LengthPow(pow(2, radius_lpow)) - radius;
+	double dist = CVector2<double>(d1, point.z).LengthPow(pow(2, tube_radius_lpow)) - tube_radius;
 	return empty ? fabs(dist) : dist;
 }
 
-double cPrimitives::TotalDistance(CVector3 point, double fractalDistance,
-		int *closestObjectId, sRenderData *data) const
+double cPrimitives::TotalDistance(
+	CVector3 point, double fractalDistance, int *closestObjectId, sRenderData *data) const
 {
 	using namespace fractal;
 	int closestObject = *closestObjectId;
@@ -399,57 +446,33 @@ double cPrimitives::TotalDistance(CVector3 point, double fractalDistance,
 		CVector3 point2 = point - allPrimitivesPosition;
 		point2 = mRotAllPrimitivesRotation.RotateVector(point2);
 
-		for(int i = 0; i < allPrimitives.size(); i++)
+		for (int i = 0; i < allPrimitives.size(); i++)
 		{
-			const sPrimitiveBasic* primitive = allPrimitives.at(i);
+			sPrimitiveBasic *primitive = allPrimitives[i];
 			if (primitive->enable)
 			{
-				double distTemp = 0;
-				switch (primitive->objectType)
+				sPrimitiveWater *water = dynamic_cast<sPrimitiveWater *>(primitive);
+				double distTemp;
+				if (water)
 				{
-					case objPlane:
-						distTemp = ((sPrimitivePlane*) primitive)->PrimitiveDistance(point);
-						break;
-					case objBox:
-						distTemp = ((sPrimitiveBox*) primitive)->PrimitiveDistance(point);
-						break;
-					case objSphere:
-						distTemp = ((sPrimitiveSphere*) primitive)->PrimitiveDistance(point);
-						break;
-					case objWater:
-						distTemp = ((sPrimitiveWater*) primitive)->PrimitiveDistance(point);
-						break;
-					case objCone:
-						distTemp = ((sPrimitiveCone*) primitive)->PrimitiveDistance(point);
-						break;
-					case objCylinder:
-						distTemp = ((sPrimitiveCylinder*) primitive)->PrimitiveDistance(point);
-						break;
-					case objTorus:
-						distTemp = ((sPrimitiveTorus*) primitive)->PrimitiveDistance(point);
-						break;
-					case objCircle:
-						distTemp = ((sPrimitiveCircle*) primitive)->PrimitiveDistance(point);
-						break;
-					case objRectangle:
-						distTemp = ((sPrimitiveRectangle*) primitive)->PrimitiveDistance(point);
-						break;
-					default:
-						qCritical() << "cannot handle " << PrimitiveNames(primitive->objectType) << " in cPrimitives::TotalDistance()";
-						break;
+					distTemp = water->PrimitiveDistanceWater(point, distance);
+				}
+				else
+				{
+					distTemp = primitive->PrimitiveDistance(point);
 				}
 				distTemp = DisplacementMap(distTemp, point, primitive->objectId, data);
 				if (distTemp < distance)
 				{
 					closestObject = primitive->objectId;
-					//color = plane.color;
-					//reflect = plane.reflect;
+					// color = plane.color;
+					// reflect = plane.reflect;
 				}
 				distance = min(distance, distTemp);
 			}
 		}
 
-	} //if is any primitive
+	} // if is any primitive
 
 	*closestObjectId = closestObject;
 
