@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -37,17 +37,20 @@
  */
 
 #include "scheduler.hpp"
+
 #include <QtCore>
 
 #include "system.hpp"
 #define LINE_DONE_BY_SERVER 9999
 
-cScheduler::cScheduler(int _numberOfLines, int progressive)
+cScheduler::cScheduler(cRegion<int> screenRegion, int progressive)
 {
-	numberOfLines = _numberOfLines;
-	linePendingThreadId = new int[numberOfLines];
-	lineDone = new bool[numberOfLines];
-	lastLinesDone = new bool[numberOfLines];
+	startLine = screenRegion.y1;
+	endLine = screenRegion.y2;
+	numberOfLines = screenRegion.height;
+	linePendingThreadId = new int[endLine];
+	lineDone = new bool[endLine];
+	lastLinesDone = new bool[endLine];
 	stopRequest = false;
 	progressiveStep = progressive;
 	progressivePass = 1;
@@ -62,17 +65,17 @@ cScheduler::~cScheduler()
 	delete[] lastLinesDone;
 }
 
-void cScheduler::Reset()
+void cScheduler::Reset() const
 {
-	memset(linePendingThreadId, 0, sizeof(int) * numberOfLines);
-	memset(lineDone, 0, sizeof(bool) * numberOfLines);
-	memset(lastLinesDone, 0, sizeof(bool) * numberOfLines);
+	memset(linePendingThreadId, 0, sizeof(int) * endLine);
+	memset(lineDone, 0, sizeof(bool) * endLine);
+	memset(lastLinesDone, 0, sizeof(bool) * endLine);
 }
 
-bool cScheduler::ThereIsStillSomethingToDo(int threadId)
+bool cScheduler::ThereIsStillSomethingToDo(int threadId) const
 {
 	bool result = false;
-	for (int i = 0; i < numberOfLines; i++)
+	for (int i = startLine; i < endLine; i++)
 	{
 		// qDebug() << "ThereIsStillSomethingToDo:" << i << lineDone[i];
 		if (!lineDone[i] && (linePendingThreadId[i] == threadId || linePendingThreadId[i] == 0))
@@ -88,10 +91,10 @@ bool cScheduler::ThereIsStillSomethingToDo(int threadId)
 	return result;
 }
 
-bool cScheduler::AllLinesDone(void)
+bool cScheduler::AllLinesDone() const
 {
 	bool result = true;
-	for (int i = 0; i < numberOfLines; i++)
+	for (int i = startLine; i < endLine; i++)
 	{
 		// qDebug() << "AllLinesDone:" << i << lineDone[i];
 		if (!lineDone[i])
@@ -106,7 +109,7 @@ bool cScheduler::AllLinesDone(void)
 	return result;
 }
 
-bool cScheduler::ShouldIBreak(int threadId, int actualLine)
+bool cScheduler::ShouldIBreak(int threadId, int actualLine) const
 {
 	if (actualLine >= 0)
 	{
@@ -125,13 +128,13 @@ int cScheduler::NextLine(int threadId, int actualLine, bool lastLineWasBroken)
 	mutex.lock();
 	// qDebug() << "threadID:" << threadId << " Actual line:" << actualLine;
 
-	int nextLine = -1;
+	int nextLine;
 
 	if (!lastLineWasBroken)
 	{
 		for (int i = 0; i < progressiveStep; i++)
 		{
-			if (actualLine + i < numberOfLines)
+			if (actualLine + i < endLine)
 			{
 				lineDone[actualLine + i] = true;
 				lastLinesDone[actualLine + i] = true;
@@ -144,7 +147,7 @@ int cScheduler::NextLine(int threadId, int actualLine, bool lastLineWasBroken)
 	}
 
 	// next line is not occupied by any thread
-	if (actualLine < numberOfLines - progressiveStep
+	if (actualLine < endLine - progressiveStep
 			&& linePendingThreadId[actualLine + progressiveStep] == 0 && !lastLineWasBroken)
 	{
 		nextLine = actualLine + progressiveStep;
@@ -163,7 +166,7 @@ int cScheduler::NextLine(int threadId, int actualLine, bool lastLineWasBroken)
 		{
 			for (int i = 0; i < progressiveStep; i++)
 			{
-				if (nextLine + i < numberOfLines)
+				if (nextLine + i < endLine)
 				{
 					linePendingThreadId[nextLine + i] = threadId;
 				}
@@ -183,21 +186,21 @@ int cScheduler::NextLine(int threadId, int actualLine, bool lastLineWasBroken)
 	return nextLine;
 }
 
-int cScheduler::FindBiggestGap()
+int cScheduler::FindBiggestGap() const
 {
 	bool firstFreeFound = false;
 	int firstFree = -1;
-	int lastFree = -1;
+	int lastFree;
 	int maxHole = 0;
 	int theBest = -1;
 
-	for (int i = 0; i < numberOfLines; i++)
+	for (int i = startLine; i < endLine; i++)
 	{
 		if (!firstFreeFound && linePendingThreadId[i] == 0)
 		{
 			firstFreeFound = true;
 			firstFree = i;
-			if (i == numberOfLines - 1)
+			if (i == endLine - 1)
 			{
 				theBest = i;
 				theBest /= progressiveStep;
@@ -207,7 +210,7 @@ int cScheduler::FindBiggestGap()
 			continue;
 		}
 
-		if (firstFreeFound && (linePendingThreadId[i] > 0 || i == numberOfLines - 1))
+		if (firstFreeFound && (linePendingThreadId[i] > 0 || i == endLine - 1))
 		{
 			lastFree = i;
 			int holeSize = lastFree - firstFree;
@@ -228,15 +231,15 @@ int cScheduler::FindBiggestGap()
 	return theBest;
 }
 
-void cScheduler::InitFirstLine(int threadId, int firstLine)
+void cScheduler::InitFirstLine(int threadId, int firstLine) const
 {
 	linePendingThreadId[firstLine] = threadId;
 }
 
-QList<int> cScheduler::GetLastRenderedLines(void)
+QList<int> cScheduler::GetLastRenderedLines() const
 {
 	QList<int> list;
-	for (int i = 0; i < numberOfLines; i++)
+	for (int i = startLine; i < endLine; i++)
 	{
 		if (lastLinesDone[i])
 		{
@@ -247,10 +250,10 @@ QList<int> cScheduler::GetLastRenderedLines(void)
 	return list;
 }
 
-double cScheduler::PercentDone()
+double cScheduler::PercentDone() const
 {
 	int count = 0;
-	for (int i = 0; i < numberOfLines; i++)
+	for (int i = startLine; i < endLine; i++)
 	{
 		if (lineDone[i]) count++;
 	}
@@ -263,12 +266,12 @@ double cScheduler::PercentDone()
 
 	if (progressiveEnabled)
 	{
-		percent_done = ((double)count / numberOfLines) * 0.75 / (progressiveStep * progressiveStep)
-									 + progressiveDone;
+		percent_done =
+			double(count) / numberOfLines * 0.75 / (progressiveStep * progressiveStep) + progressiveDone;
 	}
 	else
 	{
-		percent_done = (double)count / numberOfLines;
+		percent_done = double(count) / numberOfLines;
 	}
 
 	return percent_done;
@@ -285,13 +288,13 @@ bool cScheduler::ProgressiveNextStep()
 	}
 	else
 	{
-		memset(linePendingThreadId, 0, sizeof(int) * numberOfLines);
-		memset(lineDone, 0, sizeof(bool) * numberOfLines);
+		memset(linePendingThreadId, 0, sizeof(int) * endLine);
+		memset(lineDone, 0, sizeof(bool) * endLine);
 		return true;
 	}
 }
 
-void cScheduler::MarkReceivedLines(const QList<int> &lineNumbers)
+void cScheduler::MarkReceivedLines(const QList<int> &lineNumbers) const
 {
 	for (int i = 0; i < lineNumbers.size(); i++)
 	{
@@ -302,10 +305,10 @@ void cScheduler::MarkReceivedLines(const QList<int> &lineNumbers)
 	}
 }
 
-QList<int> cScheduler::CreateDoneList()
+QList<int> cScheduler::CreateDoneList() const
 {
 	QList<int> list;
-	for (int i = 0; i < numberOfLines; i++)
+	for (int i = startLine; i < endLine; i++)
 	{
 		if (lineDone[i])
 		{
@@ -329,9 +332,9 @@ void cScheduler::UpdateDoneLines(const QList<int> &done)
 	mutex.unlock();
 }
 
-bool cScheduler::IsLineDoneByServer(int line)
+bool cScheduler::IsLineDoneByServer(int line) const
 {
-	if (line >= 0 && line < numberOfLines)
+	if (line >= startLine && line < endLine)
 	{
 		return linePendingThreadId[line] == LINE_DONE_BY_SERVER;
 	}
