@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -36,42 +36,47 @@
  * while the image renders.
  */
 
-#ifndef SCHEDULER_HPP_
-#define SCHEDULER_HPP_
+#ifndef MANDELBULBER2_SRC_SCHEDULER_HPP_
+#define MANDELBULBER2_SRC_SCHEDULER_HPP_
+
+#include <qvector.h>
 
 #include <QMutex>
-#include <qvector.h>
+
+#include "region.hpp"
 
 class cScheduler
 {
 public:
-	cScheduler(int _numberOfLines, int progressive);
+	cScheduler(cRegion<int> screenRegion, int progressive);
 	~cScheduler();
 	int NextLine(int threadId, int actualLine, bool lastLineWasBroken);
-	bool ShouldIBreak(int threadId, int actualLine);
-	bool ThereIsStillSomethingToDo(int ThreadId);
-	bool AllLinesDone();
-	void InitFirstLine(int threadId, int firstLine);
-	QList<int> GetLastRenderedLines(void);
-	double PercentDone();
+	bool ShouldIBreak(int threadId, int actualLine) const;
+	bool ThereIsStillSomethingToDo(int ThreadId) const;
+	bool AllLinesDone() const;
+	void InitFirstLine(int threadId, int firstLine) const;
+	QList<int> GetLastRenderedLines() const;
+	double PercentDone() const;
 	void Stop() { stopRequest = true; }
-	void MarkReceivedLines(const QList<int> &lineNumbers);
+	void MarkReceivedLines(const QList<int> &lineNumbers) const;
 	void UpdateDoneLines(const QList<int> &done);
 
-	int GetProgressiveStep() { return progressiveStep; }
-	int GetProgressivePass() { return progressivePass; }
+	int GetProgressiveStep() const { return progressiveStep; }
+	int GetProgressivePass() const { return progressivePass; }
 	bool ProgressiveNextStep();
-	QList<int> CreateDoneList();
-	bool IsLineDoneByServer(int line);
+	QList<int> CreateDoneList() const;
+	bool IsLineDoneByServer(int line) const;
 
 private:
-	void Reset(void);
-	int FindBiggestGap();
+	void Reset() const;
+	int FindBiggestGap() const;
 
 	int *linePendingThreadId;
 	bool *lineDone;
 	bool *lastLinesDone;
 	int numberOfLines;
+	int startLine;
+	int endLine;
 	volatile bool stopRequest;
 	int progressiveStep;
 	int progressivePass;
@@ -79,4 +84,4 @@ private:
 	QMutex mutex;
 };
 
-#endif /* SCHEDULER_HPP_ */
+#endif /* MANDELBULBER2_SRC_SCHEDULER_HPP_ */
