@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-18 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -36,11 +36,18 @@
  */
 
 #include "settings.hpp"
+
+#include <QCryptographicHash>
+
+#include "animation_frames.hpp"
 #include "error_message.hpp"
+#include "fractal_container.hpp"
+#include "fractal_enums.h"
 #include "initparameters.hpp"
+#include "keyframes.hpp"
+#include "material.h"
 #include "primitives.h"
 #include "system.hpp"
-#include <QCryptographicHash>
 
 cSettings::cSettings(enumFormat _format)
 {
@@ -51,20 +58,28 @@ cSettings::cSettings(enumFormat _format)
 	fileVersion = 0;
 	quiet = false;
 	csvNoOfColumns = 0;
+	foundAnimSoundParameters = false;
 }
 
 size_t cSettings::CreateText(const cParameterContainer *par, const cFractalContainer *fractPar,
 	cAnimationFrames *frames, cKeyframes *keyframes)
 {
-	WriteLog("Create settings text", 2);
+	WriteLog("Create settings text", 3);
 	settingsText.clear();
 	settingsText += CreateHeader();
+	if ((format == formatFullText || format == formatCondensedText) && par->IfExists("description")
+			&& par->Get<QString>("description") != "")
+	{
+		settingsText += "[description]\n";
+		settingsText += par->Get<QString>("description") + "\n";
+	}
 	settingsText += "[main_parameters]\n";
 
 	// standard parameters
 	QList<QString> parameterList = par->GetListOfParameters();
 	for (int i = 0; i < parameterList.size(); i++)
 	{
+		if (parameterList[i] == "description") continue;
 		settingsText += CreateOneLine(par, parameterList[i]);
 	}
 
@@ -74,11 +89,16 @@ size_t cSettings::CreateText(const cParameterContainer *par, const cFractalConta
 		{
 			for (int f = 0; f < NUMBER_OF_FRACTALS; f++)
 			{
-				settingsText += "[fractal_" + QString::number(f + 1) + "]\n";
 				QList<QString> parameterListFractal = fractPar->at(f).GetListOfParameters();
+				QString fractalSettingsText = "";
 				for (int i = 0; i < parameterListFractal.size(); i++)
 				{
-					settingsText += CreateOneLine(&fractPar->at(f), parameterListFractal[i]);
+					fractalSettingsText += CreateOneLine(&fractPar->at(f), parameterListFractal[i]);
+				}
+				if (fractalSettingsText.length() > 0)
+				{
+					settingsText += "[fractal_" + QString::number(f + 1) + "]\n";
+					settingsText += fractalSettingsText;
 				}
 				parameterListFractal.clear();
 			}
@@ -100,11 +120,11 @@ size_t cSettings::CreateText(const cParameterContainer *par, const cFractalConta
 
 	WriteLogString("Settings text prepared", settingsText, 3);
 
-	return (size_t) settingsText.size();
+	return size_t(settingsText.size());
 }
 
 void cSettings::CreateAnimationString(
-	QString &text, const QString &headerText, const cAnimationFrames *frames)
+	QString &text, const QString &headerText, const cAnimationFrames *frames) const
 {
 	if (frames)
 	{
@@ -216,7 +236,7 @@ void cSettings::CreateAnimationString(
 	}
 }
 
-QString cSettings::CreateHeader()
+QString cSettings::CreateHeader() const
 {
 	QString header("# Mandelbulber settings file\n");
 	header += "# version " + QString::number(appVersion, 'f', 2) + "\n";
@@ -231,7 +251,7 @@ QString cSettings::CreateHeader()
 	return header;
 }
 
-QString cSettings::CreateOneLine(const cParameterContainer *par, QString name)
+QString cSettings::CreateOneLine(const cParameterContainer *par, QString name) const
 {
 	QString text;
 	enumParameterType parType = par->GetParameterType(name);
@@ -241,7 +261,8 @@ QString cSettings::CreateOneLine(const cParameterContainer *par, QString name)
 	{
 		QString value;
 		enumVarType type = par->GetVarType(name);
-		if (!par->isDefaultValue(name) || format == formatFullText || format == formatNetRender)
+		if (!par->isDefaultValue(name) || format == formatFullText || format == formatNetRender
+				|| format == formatAppSettings)
 		{
 			if (type == typeBool)
 			{
@@ -250,7 +271,14 @@ QString cSettings::CreateOneLine(const cParameterContainer *par, QString name)
 			}
 			else
 			{
-				value = par->Get<QString>(name);
+				if (par->GetAsOneParameter(name).IsEnumeration())
+				{
+					value = par->GetAsOneParameter(name).GetValueByEnumeration();
+				}
+				else
+				{
+					value = par->Get<QString>(name);
+				}
 			}
 			text = name + " " + value + ";\n";
 		}
@@ -258,28 +286,28 @@ QString cSettings::CreateOneLine(const cParameterContainer *par, QString name)
 	return text;
 }
 
-bool cSettings::SaveToFile(QString filename)
+bool cSettings::SaveToFile(QString filename) const
 {
 	WriteLogString("Saving settings started", filename, 2);
-	QFile qfile(filename);
-	if (qfile.open(QIODevice::WriteOnly))
+	QFile qFile(filename);
+	if (qFile.open(QIODevice::WriteOnly))
 	{
-		QTextStream outstream(&qfile);
-		outstream << settingsText;
-		outstream.flush();
-		qfile.close();
+		QTextStream outStream(&qFile);
+		outStream << settingsText;
+		outStream.flush();
+		qFile.close();
 		return true;
 	}
 	else
 	{
 		cErrorMessage::showMessage(
-			QString("Settings file not saved!\n") + filename + "\n" + qfile.errorString(),
+			QString("Settings file not saved!\n") + filename + "\n" + qFile.errorString(),
 			cErrorMessage::errorMessage);
 		return false;
 	}
 }
 
-void cSettings::SaveToClipboard()
+void cSettings::SaveToClipboard() const
 {
 	WriteLog("Save settings to clipboard", 2);
 	QClipboard *clipboard = QApplication::clipboard();
@@ -291,12 +319,12 @@ bool cSettings::LoadFromFile(QString filename)
 	settingsText.clear();
 	textPrepared = false;
 	WriteLogString("Loading settings started", filename, 2);
-	QFile qfile(filename);
-	if (qfile.open(QIODevice::ReadOnly))
+	QFile qFile(filename);
+	if (qFile.open(QIODevice::ReadOnly))
 	{
-		QTextStream instream(&qfile);
-		settingsText.append(instream.readAll());
-		qfile.close();
+		QTextStream inStream(&qFile);
+		settingsText.append(inStream.readAll());
+		qFile.close();
 		textPrepared = true;
 
 		// hash code will be needed for generating thumbnails
@@ -314,7 +342,7 @@ bool cSettings::LoadFromFile(QString filename)
 		if (!quiet)
 		{
 			cErrorMessage::showMessage(
-				QString("Settings file not loaded!\n") + filename + "\n" + qfile.errorString(),
+				QString("Settings file not loaded!\n") + filename + "\n" + qFile.errorString(),
 				cErrorMessage::errorMessage);
 		}
 		return false;
@@ -393,7 +421,10 @@ void cSettings::DecodeHeader(QStringList &separatedText)
 		}
 		catch (QString &error)
 		{
-			cErrorMessage::showMessage(error, cErrorMessage::errorMessage);
+			if (!quiet)
+			{
+				cErrorMessage::showMessage(error, cErrorMessage::errorMessage);
+			}
 			textPrepared = false;
 			return;
 		}
@@ -408,7 +439,9 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 		"*frames)",
 		2);
 
-	QStringList separatedText = settingsText.split(QRegExp("[\r\n]"), QString::SkipEmptyParts);
+	QString settingsTextTrimmed = settingsText.trimmed();
+	QStringList separatedText =
+		settingsTextTrimmed.split(QRegExp("\n|\r\n|\r"), QString::KeepEmptyParts);
 	DecodeHeader(separatedText);
 
 	int errorCount = 0;
@@ -428,11 +461,16 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 		listOfLoadedPrimitives.clear();
 		DeleteAllMaterialParams(par);
 
-		if (frames) frames->ClearAll();
+		if (frames)
+		{
+			frames->ClearAll();
+			frames->RemoveAllAudioParameters(par);
+		}
 		if (keyframes)
 		{
 			keyframes->ClearAll();
 			keyframes->ClearMorphCache();
+			keyframes->RemoveAllAudioParameters(par);
 		}
 		// temporary containers to decode frames
 		cParameterContainer parTemp;
@@ -443,10 +481,24 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 			QString line = separatedText[l];
 			bool isNewSection = CheckSection(line, section);
 
-			if (isNewSection) csvLine = 0;
-
-			if (!isNewSection)
+			if (isNewSection)
 			{
+				csvLine = 0;
+				continue;
+			}
+			else if (section == QString("description"))
+			{
+				// concat multi-line description
+				QString description = "";
+				if (par->IfExists("description")) description = par->Get<QString>("description");
+				if (description != "") description += "\n";
+				description += line;
+				par->Set("description", description);
+				continue;
+			}
+			else
+			{
+				if (line == "") continue;
 				bool result = false;
 				if (section == QString("main_parameters"))
 				{
@@ -465,7 +517,7 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 						{
 							CheckIfMaterialsAreDefined(par);
 							parTemp = *par;
-							if (fractPar)	fractTemp = *fractPar;
+							if (fractPar) fractTemp = *fractPar;
 
 							result = DecodeFramesHeader(line, par, fractPar, frames);
 							csvLine++;
@@ -489,7 +541,7 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 						{
 							CheckIfMaterialsAreDefined(par);
 							parTemp = *par;
-							if (fractPar)	fractTemp = *fractPar;
+							if (fractPar) fractTemp = *fractPar;
 
 							result = DecodeFramesHeader(line, par, fractPar, keyframes);
 							csvLine++;
@@ -508,14 +560,20 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 
 				if (!result)
 				{
-					QString errorMessage =
-						QObject::tr("Error in settings file. Line: ") + QString::number(l) + " (" + line + ")";
-					cErrorMessage::showMessage(errorMessage, cErrorMessage::errorMessage);
+					if (!quiet)
+					{
+						QString errorMessage = QObject::tr("Error in settings file. Line: ")
+																	 + QString::number(l) + " (" + line + ")";
+						cErrorMessage::showMessage(errorMessage, cErrorMessage::errorMessage);
+					}
 					errorCount++;
 					if (errorCount > 3)
 					{
-						cErrorMessage::showMessage(
-							QObject::tr("Too many errors in settings file"), cErrorMessage::errorMessage);
+						if (!quiet)
+						{
+							cErrorMessage::showMessage(
+								QObject::tr("Too many errors in settings file"), cErrorMessage::errorMessage);
+						}
 						return false;
 					}
 				}
@@ -527,9 +585,9 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 		{
 			if (keyframes->GetListOfUsedParameters().size() == 0)
 			{
-				keyframes->AddAnimatedParameter("camera", par->GetAsOneParameter("camera"));
-				keyframes->AddAnimatedParameter("target", par->GetAsOneParameter("target"));
-				keyframes->AddAnimatedParameter("camera_top", par->GetAsOneParameter("camera_top"));
+				keyframes->AddAnimatedParameter("camera", par->GetAsOneParameter("camera"), par);
+				keyframes->AddAnimatedParameter("target", par->GetAsOneParameter("target"), par);
+				keyframes->AddAnimatedParameter("camera_top", par->GetAsOneParameter("camera_top"), par);
 			}
 		}
 
@@ -539,8 +597,43 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 			CheckIfMaterialsAreDefined(par);
 		}
 
+		// now when anim sound parameters are already prepared by animation, all animsound parameters
+		// can be processed
+		if (keyframes && linesWithSoundParameters.length() > 0)
+		{
+			foundAnimSoundParameters = true;
+			for (int i = 0; i < linesWithSoundParameters.length(); i++)
+			{
+				bool result;
+				result = DecodeOneLine(par, linesWithSoundParameters[i]);
+
+				if (!result)
+				{
+					if (!quiet)
+					{
+						QString errorMessage =
+							QObject::tr("Error in settings file. Line: ") + linesWithSoundParameters[i];
+						cErrorMessage::showMessage(errorMessage, cErrorMessage::errorMessage);
+					}
+					errorCount++;
+					if (errorCount > 3)
+					{
+						if (!quiet)
+						{
+							cErrorMessage::showMessage(
+								QObject::tr("Too many errors in settings file"), cErrorMessage::errorMessage);
+						}
+						return false;
+					}
+				}
+			}
+		}
+
 		Compatibility2(par, fractPar);
 
+		if (frames) frames->LoadAllAudioFiles(par);
+		if (keyframes) keyframes->LoadAllAudioFiles(par);
+
 		return true;
 	}
 	else
@@ -562,9 +655,20 @@ bool cSettings::CheckIfMaterialsAreDefined(cParameterContainer *par)
 			break;
 		}
 	}
+
+	if (par->IfExists("formula_material_id")) // if there is checked full set of parameters
+	{
+		if (par->Get<int>("formula_material_id") == 1
+				&& !par->IfExists("mat1_is_defined")) // if material 1 is used but still not defined
+		{
+			matParameterFound = false;
+		}
+	}
+
 	if (!matParameterFound)
 	{
 		InitMaterialParams(1, par);
+		par->Set("mat1_is_defined", true);
 	}
 	return matParameterFound;
 }
@@ -600,22 +704,39 @@ bool cSettings::DecodeOneLine(cParameterContainer *par, QString line)
 			if (cMaterial::paramsList.indexOf(shortName) >= 0)
 			{
 				InitMaterialParams(matIndex, par);
+				par->Set(QString("mat%1_is_defined").arg(matIndex), true);
 			}
 			else
 			{
-				cErrorMessage::showMessage(
-					QObject::tr("Unknown parameter: ") + parameterName, cErrorMessage::errorMessage);
+				if (!quiet)
+				{
+					cErrorMessage::showMessage(
+						QObject::tr("Unknown parameter: ") + parameterName, cErrorMessage::errorMessage);
+				}
 				return false;
 			}
 		}
 	}
 
+	if (parameterName.left(9) == "animsound")
+	{
+		if (!foundAnimSoundParameters)
+		{
+			linesWithSoundParameters.append(
+				line);		 // added line for further processing (after animation is loaded)
+			return true; // parameter will be processed later
+		}
+	}
+
 	enumVarType varType = par->GetVarType(parameterName);
 
 	if (varType == typeNull)
 	{
-		cErrorMessage::showMessage(
-			QObject::tr("Unknown parameter: ") + parameterName, cErrorMessage::errorMessage);
+		if (!quiet)
+		{
+			cErrorMessage::showMessage(
+				QObject::tr("Unknown parameter: ") + parameterName, cErrorMessage::errorMessage);
+		}
 		return false;
 	}
 	else
@@ -624,28 +745,39 @@ bool cSettings::DecodeOneLine(cParameterContainer *par, QString line)
 		{
 			if (value.size() == 0)
 			{
-				cErrorMessage::showMessage(QObject::tr("Missing value for parameter %1").arg(parameterName),
-					cErrorMessage::errorMessage);
+				if (!quiet)
+				{
+					cErrorMessage::showMessage(
+						QObject::tr("Missing value for parameter %1").arg(parameterName),
+						cErrorMessage::errorMessage);
+				}
 				return false;
 			}
 		}
 		if (varType == typeBool)
 		{
-			value = (value == QString("true")) ? "1" : "0";
+			value = value == QString("true") ? "1" : "0";
 		}
 		else if (varType == typeDouble || varType == typeVector3 || varType == typeVector4)
 		{
 			value = everyLocaleDouble(value);
 		}
 
-		par->Set(parameterName, value);
+		if (par->GetAsOneParameter(parameterName).IsEnumeration())
+		{
+			par->Set(parameterName, par->GetAsOneParameter(parameterName).GetIndexByEnumeration(value));
+		}
+		else
+		{
+			par->Set(parameterName, value);
+		}
 		return true;
 	}
 }
 
 bool cSettings::CheckSection(QString text, QString &section)
 {
-	if (text.left(1) == "[")
+	if (text.left(1) == "[" && text.right(1) == "]")
 	{
 		section = text.mid(1, text.length() - 2);
 		return true;
@@ -653,7 +785,7 @@ bool cSettings::CheckSection(QString text, QString &section)
 	return false;
 }
 
-void cSettings::Compatibility(QString &name, QString &value)
+void cSettings::Compatibility(QString &name, QString &value) const
 {
 	if (fileVersion <= 2.01)
 	{
@@ -746,17 +878,44 @@ void cSettings::Compatibility(QString &name, QString &value)
 		if (name == QString("fractal_coloring_line_direction"))
 			name.replace("fractal_coloring_line_direction", "mat1_fractal_coloring_line_direction");
 	}
+	if (fileVersion < 2.09)
+	{
+		if (name == QString("delta_DE_function"))
+		{
+			if (value == QString("0"))
+			{
+				value = "2";
+			}
+			else if (value == QString("2"))
+			{
+				value = "0";
+			}
+		}
+	}
+
+	if (fileVersion < 2.12)
+	{
+		name.replace("gpu_", "opencl_");
+	}
+
+	if (fileVersion < 2.13)
+	{
+		if (name.contains("primitive_water"))
+		{
+			name.replace("amplitude", "relative_amplitude");
+		}
+	}
 }
 
 void cSettings::Compatibility2(cParameterContainer *par, cFractalContainer *fract)
 {
 	if (fileVersion <= 2.06)
 	{
-		if ((fractal::enumDEFunctionType)par->Get<int>("delta_DE_function")
+		if (fractal::enumDEFunctionType(par->Get<int>("delta_DE_function"))
 				!= fractal::linearDEFunction)
-			par->Set("delta_DE_function", (int)fractal::logarithmicDEFunction);
+			par->Set("delta_DE_function", int(fractal::logarithmicDEFunction));
 
-		if(fract)
+		if (fract)
 		{
 			for (int i = 0; i < 4; i++)
 			{
@@ -793,6 +952,24 @@ void cSettings::Compatibility2(cParameterContainer *par, cFractalContainer *frac
 			par->Set(listOfLoadedPrimitives[i] + "_material_id", materialId);
 		}
 	}
+
+	if (fileVersion <= 2.12)
+	{
+		if (par->Get<bool>("iteration_fog_enable"))
+		{
+			par->Set("iteration_fog_brightness_boost", 100.0);
+		}
+		for (int i = 0; i < listOfLoadedPrimitives.size(); i++)
+		{
+			if (listOfLoadedPrimitives[i].contains("primitive_water"))
+			{
+				double relativeAmplitude =
+					par->Get<double>(listOfLoadedPrimitives[i] + "_relative_amplitude")
+					/ par->Get<double>(listOfLoadedPrimitives[i] + "_length");
+				par->Set(listOfLoadedPrimitives[i] + "_relative_amplitude", relativeAmplitude);
+			}
+		}
+	}
 }
 
 bool cSettings::DecodeFramesHeader(
@@ -813,19 +990,51 @@ bool cSettings::DecodeFramesHeader(
 				if (fullParameterName.length() > 2)
 				{
 					QString lastTwo = fullParameterName.right(2);
-					if (lastTwo == "_x" || lastTwo == "_y" || lastTwo == "_z")
+					if (lastTwo == "_x") // check if it's CVector4
 					{
-						fullParameterName = fullParameterName.left(fullParameterName.length() - 2);
-						i += 2;
+						// check if there are at least 2 parameters left and they are *_y and *_z
+						bool isCVector4 = false;
+						if (i + 3 < lineSplit.size())
+						{
+							QString lastTwoY = lineSplit[i + 1].right(2);
+							QString lastTwoZ = lineSplit[i + 2].right(2);
+							QString lastTwoW = lineSplit[i + 3].right(2);
+							if (lastTwoY == "_y" && lastTwoZ == "_z" && lastTwoW == "_w")
+							{
+								fullParameterName = fullParameterName.left(fullParameterName.length() - 2);
+								i += 3;
+								isCVector4 = true;
+							}
+						}
+
+						if (!isCVector4 && i + 2 < lineSplit.size()) // check if it's CVector3
+						{
+							QString lastTwoY = lineSplit[i + 1].right(2);
+							QString lastTwoZ = lineSplit[i + 2].right(2);
+							if (lastTwoY == "_y" && lastTwoZ == "_z")
+							{
+								fullParameterName = fullParameterName.left(fullParameterName.length() - 2);
+								i += 2;
+							}
+						}
 					}
-					else if (lastTwo == "_R" || lastTwo == "_G" || lastTwo == "_B")
+					else if (lastTwo == "_R") // check if it's RGB
 					{
-						fullParameterName = fullParameterName.left(fullParameterName.length() - 2);
-						i += 2;
+						// check if there are at least 2 parameters left and they are *_G and *_B
+						if (i + 2 < lineSplit.size())
+						{
+							QString lastTwoG = lineSplit[i + 1].right(2);
+							QString lastTwoB = lineSplit[i + 2].right(2);
+							if (lastTwoG == "_G" && lastTwoB == "_B")
+							{
+								fullParameterName = fullParameterName.left(fullParameterName.length() - 2);
+								i += 2;
+							}
+						}
 					}
 				}
 
-				//compatibility with older settings
+				// compatibility with older settings
 				int firstUnderscore = fullParameterName.indexOf('_');
 				QString containerName = fullParameterName.left(firstUnderscore);
 				QString parameterName = fullParameterName.mid(firstUnderscore + 1);
@@ -974,7 +1183,7 @@ bool cSettings::DecodeFramesLine(
 	return true;
 }
 
-QString cSettings::GetSettingsText()
+QString cSettings::GetSettingsText() const
 {
 	if (textPrepared)
 	{
