@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-18 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -29,7 +29,7 @@
  *
  * Authors: Krzysztof Marczak (buddhi1980@gmail.com)
  *
- * cSSAOWorker class - worker for multithread SSAO effect
+ * cSSAOWorker class - worker for multi thread SSAO effect
  *
  * SSAO shades inner edges and corners. For each pixel the surrounding
  * z-buffer (depth from camera) is scanned and based on the angle of the surface
@@ -37,10 +37,15 @@
  */
 
 #include "ssao_worker.h"
+
+#include "cimage.hpp"
+#include "color_structures.hpp"
 #include "common_math.h"
+#include "fractparams.hpp"
+#include "render_data.hpp"
 
 cSSAOWorker::cSSAOWorker(
-	const cParamRender *_params, sThreadData *_threadData, const sRenderData *_data, cImage *_image)
+	const sParamRender *_params, sThreadData *_threadData, const sRenderData *_data, cImage *_image)
 {
 	params = _params;
 	data = _data;
@@ -56,32 +61,39 @@ cSSAOWorker::~cSSAOWorker()
 void cSSAOWorker::doWork()
 {
 	int quality = threadData->quality;
-	int startLine = threadData->startLine;
-	int width = image->GetWidth();
-	int height = image->GetHeight();
+	int startLineInit = threadData->startLine;
+	int startLine = threadData->region.y1;
+	int endLine = threadData->region.y2;
+	int width = threadData->region.width;
+	int height = threadData->region.height;
+	int startX = threadData->region.x1;
+	int endX = threadData->region.x2;
 
 	double *cosine = new double[quality];
 	double *sine = new double[quality];
 	for (int i = 0; i < quality; i++)
 	{
-		sine[i] = sin((double)i / quality * 2.0 * M_PI);
-		cosine[i] = cos((double)i / quality * 2.0 * M_PI);
+		sine[i] = sin(double(i) / quality * 2.0 * M_PI);
+		cosine[i] = cos(double(i) / quality * 2.0 * M_PI);
 	}
 
-	double scale_factor = (double)width / (quality * quality) / 2.0;
-	double aspectRatio = (double)width / height;
-
 	params::enumPerspectiveType perspectiveType = params->perspectiveType;
+
+	double scale_factor = double(width) / (quality * quality) / 2.0;
+	double aspectRatio = double(width) / height;
+
+	if (perspectiveType == params::perspEquirectangular) aspectRatio = 2.0;
+
 	double fov = params->fov;
 
-	double intensity = params->ambientOcclusion;
+	float intensity = params->ambientOcclusion;
 
 	int listIndex = 0;
 
 	int step = threadData->progressive;
 	if (step == 0) step = 1;
 
-	for (int y = startLine; y < height; y += threadData->noOfThreads)
+	for (int y = startLineInit; y < endLine; y += threadData->noOfThreads)
 	{
 		if (threadData->list)
 		{
@@ -98,21 +110,20 @@ void cSSAOWorker::doWork()
 				listIndex++;
 			}
 		}
-		for (int x = 0; x < width; x += step)
+		for (int x = startX; x < endX; x += step)
 		{
-			double z = image->GetPixelZBuffer(x, y);
+			double z = double(image->GetPixelZBuffer(x, y));
 			unsigned short opacity16 = image->GetPixelOpacity(x, y);
-			double opacity = opacity16 / 65535.0;
-			double total_ambient = 0;
+			float opacity = opacity16 / 65535.0f;
+			float total_ambient = 0.0f;
 
 			if (z < 1e19)
 			{
-				// printf("SSAO point on object\n");
 				double x2, y2;
-				if (perspectiveType == params::perspFishEye)
+				if (perspectiveType == params::perspFishEye || perspectiveType == params::perspFishEyeCut)
 				{
-					x2 = M_PI * ((double)x / width - 0.5) * aspectRatio;
-					y2 = M_PI * ((double)y / height - 0.5);
+					x2 = M_PI * (double(x - startX) / width - 0.5) * aspectRatio;
+					y2 = M_PI * (double(y - startLine) / height - 0.5);
 					double r = sqrt(x2 * x2 + y2 * y2);
 					if (r != 0.0)
 					{
@@ -122,21 +133,21 @@ void cSSAOWorker::doWork()
 				}
 				else if (perspectiveType == params::perspEquirectangular)
 				{
-					x2 = M_PI * ((double)x / width - 0.5) * aspectRatio;
-					y2 = M_PI * ((double)y / height - 0.5);
+					x2 = M_PI * (double(x - startX) / width - 0.5) * aspectRatio;
+					y2 = M_PI * (double(y - startLine) / height - 0.5);
 					x2 = sin(fov * x2) * cos(fov * y2) * z;
 					y2 = sin(fov * y2) * z;
 				}
 				else
 				{
-					x2 = ((double)x / width - 0.5) * aspectRatio;
-					y2 = ((double)y / height - 0.5);
+					x2 = (double(x - startX) / width - 0.5) * aspectRatio;
+					y2 = double(y - startLine) / height - 0.5;
 					x2 = x2 * z * fov;
 					y2 = y2 * z * fov;
 				}
 
-				double ambient = 0;
-				double angleStep = M_PI * 2.0 / (double)quality;
+				float ambient = 0.0f;
+				double angleStep = M_PI * 2.0 / double(quality);
 				int maxRandom = 62831 / quality;
 				double rRandom = 1.0;
 
@@ -145,18 +156,14 @@ void cSSAOWorker::doWork()
 				for (int angleIndex = 0; angleIndex < quality; angleIndex++)
 				{
 					double ca, sa;
+					double angle = angleIndex;
 					if (params->SSAO_random_mode)
 					{
-						double angle = angleStep * angleIndex + Random(maxRandom) / 10000.0;
-						ca = cos(angle);
-						sa = sin(angle);
-					}
-					else
-					{
-						ca = cosine[angleIndex];
-						sa = sine[angleIndex];
+						angle = angleStep * angleIndex + Random(maxRandom) / 10000.0;
 					}
 
+					ca = cosine[(int)angle];
+					sa = sine[(int)angle];
 					double max_diff = -1e50;
 
 					for (double r = 1.0; r < quality; r += rRandom)
@@ -165,15 +172,16 @@ void cSSAOWorker::doWork()
 						double xx = x + rr * ca;
 						double yy = y + rr * sa;
 
-						if ((int)xx == x && (int)yy == y) continue;
-						if (xx < 0 || xx > width - 1 || yy < 0 || yy > height - 1) continue;
-						double z2 = image->GetPixelZBuffer((int)xx, (int)yy);
+						if (int(xx) == x && int(yy) == y) continue;
+						if (xx < startX || xx > endX - 1 || yy < startLine || yy > endLine - 1) continue;
+						double z2 = double(image->GetPixelZBuffer(int(xx), int(yy)));
 
 						double xx2, yy2;
-						if (perspectiveType == params::perspFishEye)
+						if (perspectiveType == params::perspFishEye
+								|| perspectiveType == params::perspFishEyeCut)
 						{
-							xx2 = M_PI * (xx / width - 0.5) * aspectRatio;
-							yy2 = M_PI * (yy / height - 0.5);
+							xx2 = M_PI * ((xx - startX) / width - 0.5) * aspectRatio;
+							yy2 = M_PI * ((yy - startLine) / height - 0.5);
 							double r2 = sqrt(xx2 * xx2 + yy2 * yy2);
 							if (r != 0.0)
 							{
@@ -183,15 +191,15 @@ void cSSAOWorker::doWork()
 						}
 						else if (perspectiveType == params::perspEquirectangular)
 						{
-							xx2 = M_PI * (xx / width - 0.5) * aspectRatio;
-							yy2 = M_PI * (yy / height - 0.5);
+							xx2 = M_PI * ((xx - startX) / width - 0.5) * aspectRatio;
+							yy2 = M_PI * ((yy - startLine) / height - 0.5);
 							xx2 = sin(fov * xx2) * cos(fov * yy2) * z2;
 							yy2 = sin(fov * yy2) * z2;
 						}
 						else
 						{
-							xx2 = (xx / width - 0.5) * aspectRatio;
-							yy2 = (yy / height - 0.5);
+							xx2 = ((xx - startX) / width - 0.5) * aspectRatio;
+							yy2 = (yy - startLine) / height - 0.5;
 							xx2 = xx2 * (z2 * fov);
 							yy2 = yy2 * (z2 * fov);
 						}
@@ -215,14 +223,14 @@ void cSSAOWorker::doWork()
 
 			for (int xx = 0; xx < step; xx++)
 			{
-				if (xx >= width - 1) break;
+				if (x + xx >= endX - 1) break;
 				sRGB8 colour = image->GetPixelColor(x + xx, y);
-				sRGB16 pixel = image->GetPixelImage16(x + xx, y);
-				double shadeFactor = (65535.0 / 256.0) * total_ambient * intensity * (1.0 - opacity);
-				pixel.R = (unsigned short)min(pixel.R + (int)(colour.R * shadeFactor), 65535);
-				pixel.G = (unsigned short)min(pixel.G + (int)(colour.G * shadeFactor), 65535);
-				pixel.B = (unsigned short)min(pixel.B + (int)(colour.B * shadeFactor), 65535);
-				image->PutPixelImage16(x + xx, y, pixel);
+				sRGBFloat pixel = image->GetPixelPostImage(x + xx, y);
+				float shadeFactor = 1.0f / 256.0f * total_ambient * intensity * (1.0f - opacity);
+				pixel.R = pixel.R + colour.R * shadeFactor;
+				pixel.G = pixel.G + colour.G * shadeFactor;
+				pixel.B = pixel.B + colour.B * shadeFactor;
+				image->PutPixelPostImage(x + xx, y, pixel);
 			}
 		}
 
