@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -29,22 +29,28 @@
  *
  * Authors: Krzysztof Marczak (buddhi1980@gmail.com)
  *
- * cSSAOWorker class - worker for multithread SSAO effect
+ * cSSAOWorker class - worker for multi thread SSAO effect
  *
  * SSAO shades inner edges and corners. For each pixel the surrounding
  * z-buffer (depth from camera) is scanned and based on the angle of the surface
  * each pixel will be shaded. This class gets used in render_ssao.cpp as multiple threads.
  */
 
-#ifndef SSAO_WORKER_H_
-#define SSAO_WORKER_H_
+#ifndef MANDELBULBER2_SRC_SSAO_WORKER_H_
+#define MANDELBULBER2_SRC_SSAO_WORKER_H_
 
-#include "cimage.hpp"
-#include "fractparams.hpp"
-#include "render_data.hpp"
-#include <QList>
 #include <qobject.h>
 
+#include <QList>
+#include <QThread>
+
+#include "region.hpp"
+
+// forward declarations
+struct sParamRender;
+struct sRenderData;
+class cImage;
+
 class cSSAOWorker : public QObject
 {
 	Q_OBJECT
@@ -58,25 +64,26 @@ public:
 		unsigned int done;
 		bool stopRequest;
 		QList<int> *list;
+		cRegion<int> region;
 	};
 
-	cSSAOWorker(const cParamRender *_params, sThreadData *_threadData, const sRenderData *_data,
+	cSSAOWorker(const sParamRender *_params, sThreadData *_threadData, const sRenderData *_data,
 		cImage *_image);
 	~cSSAOWorker();
 
 	QThread workerThread;
 
 	// data got from main thread
-	const cParamRender *params;
+	const sParamRender *params;
 	const sRenderData *data;
 	sThreadData *threadData;
 	cImage *image;
 
 public slots:
-	void doWork(void);
+	void doWork();
 
 signals:
 	void finished();
 };
 
-#endif /* SSAO_WORKER_H_ */
+#endif /* MANDELBULBER2_SRC_SSAO_WORKER_H_ */
