@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-18 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -34,32 +34,41 @@
 
 #include "system.hpp"
 
-#include "global_data.hpp"
-#include "initparameters.hpp"
-#include "interface.hpp"
-#include <QTextStream>
-#include <QtGui>
-#include <ctime>
 #include <locale.h>
 #include <qstylefactory.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#ifndef WIN32
+#include <ctime>
+#include <iostream>
+
+#include <QTextStream>
+#include <QtGui>
+
+#include "global_data.hpp"
+#include "initparameters.hpp"
+#include "interface.hpp"
+#include "render_window.hpp"
+
+// custom includes
+#ifndef _WIN32
 #include <signal.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
 #endif
 
-#include <stdio.h>
-
+#if defined(__APPLE__) || defined(__MACOSX)
+#include "CoreFoundation/CoreFoundation.h"
+#endif
+ 
+ 
 //#define CLSUPPORT
 
 sSystem systemData;
 sActualFileNames actualFileNames;
 
-bool InitSystem(void)
+bool InitSystem()
 {
 	setlocale(LC_ALL, "");
 	systemData.locale = QLocale::system();
@@ -70,19 +79,38 @@ bool InitSystem(void)
 
 	systemData.globalTimer.start();
 
-	systemData.homedir = QDir::toNativeSeparators(QDir::homePath() + QDir::separator());
+	systemData.homeDir = QDir::toNativeSeparators(QDir::homePath() + QDir::separator());
+#ifdef _WIN32 /* WINDOWS */
+	systemData.sharedDir = QDir::toNativeSeparators(QDir::currentPath() + QDir::separator());
+	systemData.docDir =
+		QDir::toNativeSeparators(QDir::currentPath() + QDir::separator() + "doc" + QDir::separator());
+#elif SHARED_DIR_IS_APP_DIR
+	/* used for AppImage, which requires fixed data bundled at same location, as the application */
+    QString sharePath;
+    #if defined(__APPLE__) || defined(__MACOSX)
+        CFURLRef appUrlRef = CFBundleCopyBundleURL( CFBundleGetMainBundle() );
+        CFStringRef macPath = CFURLCopyFileSystemPath( appUrlRef, kCFURLPOSIXPathStyle );
+        sharePath = QString::fromCFString( macPath );
+        CFRelease(appUrlRef);
+        CFRelease(macPath);
+    #else
+        sharePath = QDir::currentPath();
+    #endif
+    out << "sharePath directory: " << sharePath << endl;
 
-#ifdef WIN32 /* WINDOWS */
-	systemData.sharedDir = QDir::toNativeSeparators((QDir::currentPath() + QDir::separator()));
+	systemData.sharedDir = QDir::toNativeSeparators(sharePath + QDir::separator());
+	systemData.docDir =
+		QDir::toNativeSeparators(sharePath + QDir::separator() + "doc" + QDir::separator());
 #else
 	systemData.sharedDir = QDir::toNativeSeparators(QString(SHARED_DIR) + QDir::separator());
-#endif /* WINDOWS */
-
+	systemData.docDir = QDir::toNativeSeparators(QString(SHARED_DOC_DIR) + QDir::separator());
+#endif
+ 
 // logfile
-#ifdef WIN32 /* WINDOWS */
-	systemData.logfileName = systemData.homedir + "mandelbulber_log.txt";
+#ifdef _WIN32 /* WINDOWS */
+	systemData.logfileName = systemData.homeDir + "mandelbulber_log.txt";
 #else
-	systemData.logfileName = systemData.homedir + ".mandelbulber_log.txt";
+	systemData.logfileName = systemData.homeDir + ".mandelbulber_log.txt";
 #endif
 	FILE *logfile = fopen(systemData.logfileName.toLocal8Bit().constData(), "w");
 	fclose(logfile);
@@ -102,27 +130,28 @@ bool InitSystem(void)
 #endif
 
 // data directory location
-#ifdef WIN32 /* WINDOWS */
-	systemData.dataDirectory = systemData.homedir + "mandelbulber" + QDir::separator();
+#ifdef _WIN32 /* WINDOWS */
+	systemData.SetDataDirectoryHidden(systemData.homeDir + "mandelbulber" + QDir::separator());
+	systemData.SetDataDirectoryPublic(systemData.homeDir + "mandelbulber" + QDir::separator());
 #else
-	systemData.dataDirectory =
-		QDir::toNativeSeparators(systemData.homedir + ".mandelbulber" + QDir::separator());
+	systemData.SetDataDirectoryHidden(
+		QDir::toNativeSeparators(systemData.homeDir + ".mandelbulber" + QDir::separator()));
+	systemData.SetDataDirectoryPublic(
+		QDir::toNativeSeparators(systemData.homeDir + "mandelbulber" + QDir::separator()));
 #endif
-	out << "Default data directory: " << systemData.dataDirectory << endl;
-	WriteLogString("Default data directory", systemData.dataDirectory, 1);
-
-	systemData.thumbnailDir =
-		QDir::toNativeSeparators(systemData.dataDirectory + "thumbnails" + QDir::separator());
-
-	systemData.autosaveFile = QDir::toNativeSeparators(systemData.dataDirectory + ".autosave.fract");
+	out << "Default data hidden directory: " << systemData.GetDataDirectoryHidden() << endl;
+	WriteLogString("Default data hidden directory", systemData.GetDataDirectoryHidden(), 1);
+	out << "Default data public directory: " << systemData.GetDataDirectoryPublic() << endl;
+	WriteLogString("Default data public directory", systemData.GetDataDirectoryPublic(), 1);
 
 	//*********** temporary set to false ************
 	systemData.noGui = false;
+	systemData.silent = false;
 
 	systemData.lastSettingsFile = QDir::toNativeSeparators(
-		systemData.dataDirectory + "settings" + QDir::separator() + QString("settings.fract"));
+		systemData.GetSettingsFolder() + QDir::separator() + QString("settings.fract"));
 	systemData.lastImageFile = QDir::toNativeSeparators(
-		systemData.dataDirectory + "images" + QDir::separator() + QString("image.jpg"));
+		systemData.GetImagesFolder() + QDir::separator() + QString("image.jpg"));
 	systemData.lastImagePaletteFile = QDir::toNativeSeparators(
 		systemData.sharedDir + "textures" + QDir::separator() + QString("colour palette.jpg"));
 
@@ -134,6 +163,7 @@ bool InitSystem(void)
 	systemData.supportedLanguages.insert("pl_PL", "Polski");
 	systemData.supportedLanguages.insert("de_DE", "Deutsch");
 	systemData.supportedLanguages.insert("it_IT", "Italiano");
+	systemData.supportedLanguages.insert("nl_NL", "Nederlands");
 
 	// get number of columns of console
 	systemData.terminalWidth = 80;
@@ -154,7 +184,7 @@ bool InitSystem(void)
 void handle_winch(int sig)
 {
 	(void)sig;
-#ifndef WIN32
+#ifndef _WIN32
 	signal(SIGWINCH, SIG_IGN);
 	struct winsize w;
 	ioctl(STDOUT_FILENO, TIOCGWINSZ, &w);
@@ -169,32 +199,44 @@ int get_cpu_count()
 	return QThread::idealThreadCount();
 }
 
+void WriteLogCout(QString text, int verbosityLevel)
+{
+	// output to console
+	cout << text.toStdString();
+	cout.flush();
+	WriteLog(text, verbosityLevel);
+}
+
 void WriteLog(QString text, int verbosityLevel)
 {
+	// verbosity level:
+	// 1 - only errors
+	// 2 - main events / actions
+	// 3 - detailed events / actions
+
 	if (verbosityLevel <= systemData.loggingVerbosity)
 	{
 		FILE *logfile = fopen(systemData.logfileName.toLocal8Bit().constData(), "a");
-#ifdef WIN32
-		QString logtext = QString("PID: %1, time: %2, %3\n")
+#ifdef _WIN32
+		QString logText = QString("PID: %1, time: %2, %3\n")
 												.arg(QCoreApplication::applicationPid())
 												.arg(QString::number(clock() / 1.0e3, 'f', 3))
 												.arg(text);
 #else
-		QString logtext =
+		QString logText =
 			QString("PID: %1, time: %2, %3\n")
 				.arg(QCoreApplication::applicationPid())
 				.arg(QString::number((systemData.globalTimer.nsecsElapsed()) / 1.0e9, 'f', 9))
 				.arg(text);
 #endif
 
-		fputs(logtext.toLocal8Bit().constData(), logfile);
+		fputs(logText.toLocal8Bit().constData(), logfile);
 		fclose(logfile);
 
 		// write to log in window
-		if (gMainInterface && gMainInterface->mainWindow != NULL)
+		if (gMainInterface && gMainInterface->mainWindow != nullptr)
 		{
-			// FIXME: AppendToLog consumes too much memory! may be fixed, please test
-			gMainInterface->mainWindow->AppendToLog(logtext);
+			gMainInterface->mainWindow->AppendToLog(logText);
 		}
 	}
 }
@@ -209,22 +251,32 @@ void WriteLogDouble(QString text, double value, int verbosityLevel)
 	WriteLog(text + ", value = " + QString::number(value), verbosityLevel);
 }
 
-bool CreateDefaultFolders(void)
+void WriteLogInt(QString text, int value, int verbosityLevel)
+{
+	WriteLog(text + ", value = " + QString::number(value), verbosityLevel);
+}
+
+void WriteLogSizeT(QString text, size_t value, int verbosityLevel)
+{
+	WriteLog(text + ", value = " + QString::number(value), verbosityLevel);
+}
+
+bool CreateDefaultFolders()
 {
 	// create data directory if not exists
 	bool result = true;
 
-	result &= CreateFolder(systemData.dataDirectory);
-	result &= CreateFolder(systemData.dataDirectory + "images");
-	result &= CreateFolder(systemData.dataDirectory + "keyframes");
-	result &= CreateFolder(systemData.dataDirectory + "paths");
-	result &= CreateFolder(systemData.dataDirectory + "undo");
-	result &= CreateFolder(systemData.dataDirectory + "paths");
-	result &= CreateFolder(systemData.dataDirectory + "thumbnails");
-	result &= CreateFolder(systemData.dataDirectory + "toolbar");
-	result &= CreateFolder(systemData.dataDirectory + "settings");
-	result &= CreateFolder(systemData.dataDirectory + "slices");
-	result &= CreateFolder(systemData.dataDirectory + "materials");
+	result &= CreateFolder(systemData.GetDataDirectoryHidden());
+	result &= CreateFolder(systemData.GetDataDirectoryPublic());
+	result &= CreateFolder(systemData.GetImagesFolder());
+	result &= CreateFolder(systemData.GetThumbnailsFolder());
+	result &= CreateFolder(systemData.GetToolbarFolder());
+	result &= CreateFolder(systemData.GetHttpCacheFolder());
+	result &= CreateFolder(systemData.GetCustomWindowStateFolder());
+	result &= CreateFolder(systemData.GetSettingsFolder());
+	result &= CreateFolder(systemData.GetSlicesFolder());
+	result &= CreateFolder(systemData.GetMaterialsFolder());
+	result &= CreateFolder(systemData.GetAnimationFolder());
 
 	RetrieveToolbarPresets(false);
 	RetrieveExampleMaterials(false);
@@ -267,33 +319,34 @@ bool CreateDefaultFolders(void)
 	return result;
 }
 
-bool CreateFolder(QString qname)
+bool CreateFolder(QString qName)
 {
-	if (QDir(qname).exists())
+	if (QDir(qName).exists())
 	{
-		WriteLogString("Directory already exists", qname, 1);
+		WriteLogString("Directory already exists", qName, 1);
 		return true;
 	}
 	else
 	{
-		if (QDir().mkdir(qname))
+		if (QDir().mkdir(qName))
 		{
-			WriteLogString("Directory created", qname, 2);
+			WriteLogString("Directory created", qName, 2);
 			return true;
 		}
 		else
 		{
-			WriteLogString("Directory can't be created", qname, 1);
-			qCritical() << "error: directory " << qname << " cannot be created" << endl;
+			WriteLogString("Directory can't be created", qName, 1);
+			qCritical() << "error: directory " << qName << " cannot be created" << endl;
 			return false;
 		}
 	}
 }
 
-void DeleteAllFilesFromDirectory(QString folder, QString filterExpression)
+void DeleteAllFilesFromDirectory(
+	QString folder, QString filterExpression, QRegExp::PatternSyntax pattern)
 {
 	QRegExp rx(filterExpression);
-	rx.setPatternSyntax(QRegExp::Wildcard);
+	rx.setPatternSyntax(pattern);
 
 	if (QDir(folder).exists())
 	{
@@ -325,13 +378,10 @@ int fcopy(QString source, QString dest)
 {
 	// ------ file reading
 
-	FILE *pFile;
-	long int lSize;
 	char *buffer;
-	size_t result;
 
-	pFile = fopen(source.toLocal8Bit().constData(), "rb");
-	if (pFile == NULL)
+	FILE *pFile = fopen(source.toLocal8Bit().constData(), "rb");
+	if (pFile == nullptr)
 	{
 		qCritical() << "Can't open source file for copying: " << source << endl;
 		WriteLogString("Can't open source file for copying", source, 1);
@@ -340,7 +390,7 @@ int fcopy(QString source, QString dest)
 
 	// obtain file size:
 	fseek(pFile, 0, SEEK_END);
-	lSize = ftell(pFile);
+	const long int lSize = ftell(pFile);
 	rewind(pFile);
 
 	// allocate memory to contain the whole file:
@@ -349,8 +399,8 @@ int fcopy(QString source, QString dest)
 		buffer = new char[lSize];
 
 		// copy the file into the buffer:
-		result = fread(buffer, 1, lSize, pFile);
-		if (result != (size_t)lSize)
+		const size_t result = fread(buffer, 1, lSize, pFile);
+		if (result != size_t(lSize))
 		{
 			qCritical() << "Can't read source file for copying: " << source << endl;
 			WriteLogString("Can't read source file for copying", source, 1);
@@ -370,7 +420,7 @@ int fcopy(QString source, QString dest)
 	// ----- file writing
 
 	pFile = fopen(dest.toLocal8Bit().constData(), "wb");
-	if (pFile == NULL)
+	if (pFile == nullptr)
 	{
 		qCritical() << "Can't open destination file for copying: " << dest << endl;
 		WriteLogString("Can't open destination file for copying", dest, 1);
@@ -427,14 +477,15 @@ void myMessageOutput(QtMsgType type, const QMessageLogContext &context, const QS
 				context.line, context.function);
 			text = QString("Fatal: ") + QString(localMsg.constData()) + " (" + context.file + ":"
 						 + QString::number(context.line) + ", " + context.function;
+			abort();
 			break;
 	}
 	WriteLog(text, 1);
 
-	if(type == QtFatalMsg) abort();
+	if (type == QtFatalMsg) abort();
 }
 
-void UpdateDefaultPaths(void)
+void UpdateDefaultPaths()
 {
 	systemData.lastSettingsFile =
 		gPar->Get<QString>("default_settings_path") + QDir::separator() + QString("settings.fract");
@@ -448,7 +499,7 @@ void UpdateDefaultPaths(void)
 		gPar->Get<QString>("default_textures_path") + QDir::separator() + "lightmap.jpg");
 }
 
-void UpdateUIStyle(void)
+void UpdateUIStyle()
 {
 	// set ui style
 	if (gPar->Get<int>("ui_style_type") >= 0
@@ -459,53 +510,91 @@ void UpdateUIStyle(void)
 	}
 }
 
-void UpdateUISkin(void)
+void UpdateUISkin()
 {
 	static QPalette defaultPalette; // cache "normal" skin on first call
 	QPalette palette;
 
+	QColor colorBackground1 = palette.color(QPalette::Window);
+	QColor colorBackground2 = palette.color(QPalette::Base);
+	QColor colorText1 = palette.color(QPalette::WindowText);
+	QColor colorText2 = palette.color(QPalette::HighlightedText);
+
 	switch (gPar->Get<int>("ui_skin"))
 	{
 		case 1: // dark skin
-			palette.setColor(QPalette::Window, QColor(53, 53, 53));
-			palette.setColor(QPalette::WindowText, Qt::white);
-			palette.setColor(QPalette::Base, QColor(25, 25, 25));
-			palette.setColor(QPalette::AlternateBase, QColor(53, 53, 53));
-			palette.setColor(QPalette::ToolTipBase, Qt::white);
-			palette.setColor(QPalette::ToolTipText, Qt::white);
-			palette.setColor(QPalette::Text, Qt::white);
-			palette.setColor(QPalette::Button, QColor(53, 53, 53));
-			palette.setColor(QPalette::ButtonText, Qt::white);
-			palette.setColor(QPalette::BrightText, Qt::red);
-			palette.setColor(QPalette::Link, QColor(42, 130, 218));
-
-			palette.setColor(QPalette::Highlight, QColor(42, 130, 218));
-			palette.setColor(QPalette::HighlightedText, Qt::black);
-			palette.setColor(QPalette::ToolTipBase, Qt::black);
-			palette.setColor(QPalette::ToolTipText, QColor(42, 130, 218));
+			colorBackground1 = QColor(53, 53, 53);
+			colorBackground2 = QColor(25, 25, 25);
+			colorText1 = Qt::white;
+			colorText2 = Qt::black;
 			break;
 		case 2: // light skin (only roughly inverted dark skin)
-			palette.setColor(QPalette::Window, QColor(240, 240, 240));
-			palette.setColor(QPalette::WindowText, Qt::black);
-			palette.setColor(QPalette::Base, QColor(250, 250, 250));
-			palette.setColor(QPalette::AlternateBase, QColor(240, 240, 240));
-			palette.setColor(QPalette::ToolTipBase, Qt::black);
-			palette.setColor(QPalette::ToolTipText, Qt::black);
-			palette.setColor(QPalette::Text, Qt::black);
-			palette.setColor(QPalette::Button, QColor(240, 240, 240));
-			palette.setColor(QPalette::ButtonText, Qt::black);
-			palette.setColor(QPalette::BrightText, Qt::red);
-			palette.setColor(QPalette::Link, QColor(42, 130, 218));
-
-			palette.setColor(QPalette::Highlight, QColor(42, 130, 218));
-			palette.setColor(QPalette::HighlightedText, Qt::white);
-			palette.setColor(QPalette::ToolTipBase, Qt::white);
-			palette.setColor(QPalette::ToolTipText, QColor(42, 130, 218));
+			colorBackground1 = QColor(240, 240, 240);
+			colorBackground2 = QColor(250, 250, 250);
+			colorText1 = Qt::black;
+			colorText2 = Qt::white;
+			break;
+		case 3: // Nasa Font light
+			QFontDatabase::addApplicationFont(":/fonts/fonts/nasalization-rg.ttf");
+			gApplication->setFont(QFont("nasalization"));
+			colorBackground1 = QColor(167, 173, 187);
+			colorBackground2 = QColor(192, 196, 207);
+			colorText1 = Qt::black;
+			colorText2 = Qt::white;
+			break;
+		case 4: // Nasa Font dark
+			QFontDatabase::addApplicationFont(":/fonts/fonts/nasalization-rg.ttf");
+			gApplication->setFont(QFont("nasalization"));
+			colorBackground1 = QColor(52, 61, 70);
+			colorBackground2 = QColor(79, 91, 102);
+			colorText1 = Qt::white;
+			colorText2 = Qt::black;
+			break;
+		case 5: // Nasa Font dark green
+			QFontDatabase::addApplicationFont(":/fonts/fonts/nasalization-rg.ttf");
+			gApplication->setFont(QFont("nasalization"));
+			colorBackground1 = QColor(20, 40, 10);
+			colorBackground2 = QColor(30, 50, 0);
+			colorText1 = Qt::green;
+			colorText2 = Qt::black;
+			break;
+		case 6: // Nasa Font dark blue
+			QFontDatabase::addApplicationFont(":/fonts/fonts/nasalization-rg.ttf");
+			gApplication->setFont(QFont("nasalization"));
+			colorBackground1 = QColor(10, 10, 40);
+			colorBackground2 = QColor(20, 20, 60);
+			colorText1 = QColor(50, 150, 255);
+			colorText2 = Qt::black;
+			break;
+		case 7: // Nasa Font dark brown
+			QFontDatabase::addApplicationFont(":/fonts/fonts/nasalization-rg.ttf");
+			gApplication->setFont(QFont("nasalization"));
+			colorBackground1 = QColor(40, 10, 10);
+			colorBackground2 = QColor(60, 20, 20);
+			colorText1 = QColor(255, 150, 50);
+			colorText2 = Qt::black;
 			break;
 		default: // normal skin
 			palette = defaultPalette;
 			break;
 	}
+	if (gPar->Get<int>("ui_skin") != 0)
+	{
+		palette.setColor(QPalette::Window, colorBackground1);
+		palette.setColor(QPalette::WindowText, colorText1);
+		palette.setColor(QPalette::Base, colorBackground2);
+		palette.setColor(QPalette::AlternateBase, colorBackground1);
+		palette.setColor(QPalette::ToolTipBase, colorBackground2);
+		palette.setColor(QPalette::ToolTipText, colorText1);
+		palette.setColor(QPalette::Text, colorText1);
+		palette.setColor(QPalette::Button, colorBackground1);
+		palette.setColor(QPalette::ButtonText, colorText1);
+		palette.setColor(QPalette::BrightText, Qt::red);
+		palette.setColor(QPalette::Link, QColor(42, 130, 218));
+
+		palette.setColor(QPalette::Highlight, QColor(42, 130, 218));
+		palette.setColor(QPalette::HighlightedText, colorText2);
+	}
 	// set ui skin
 	gApplication->setPalette(palette);
 }
@@ -530,7 +619,7 @@ void UpdateLanguage(QCoreApplication *app)
 	WriteLogString("locale", locale, 2);
 	mandelbulberMainTranslator.load(locale, systemData.sharedDir + QDir::separator() + "language");
 	mandelbulberFractalUiTranslator.load(
-		"qt_data_" + locale, systemData.sharedDir + QDir::separator() + "language");
+		"formula_" + locale, systemData.sharedDir + QDir::separator() + "language");
 
 	WriteLog("Installing translator", 2);
 	app->installTranslator(&mandelbulberMainTranslator);
@@ -548,7 +637,7 @@ void UpdateLanguage(QCoreApplication *app)
 
 void RetrieveToolbarPresets(bool force)
 {
-	if (QDir(systemData.dataDirectory + "toolbar")
+	if (QDir(systemData.GetToolbarFolder())
 					.entryInfoList(QDir::NoDotAndDotDot | QDir::AllEntries)
 					.count()
 				== 0
@@ -560,15 +649,15 @@ void RetrieveToolbarPresets(bool force)
 		{
 			toolbarFiles.next();
 			if (toolbarFiles.fileName() == "." || toolbarFiles.fileName() == "..") continue;
-			fcopy(
-				toolbarFiles.filePath(), systemData.dataDirectory + "toolbar/" + toolbarFiles.fileName());
+			fcopy(toolbarFiles.filePath(),
+				systemData.GetToolbarFolder() + QDir::separator() + toolbarFiles.fileName());
 		}
 	}
 }
 
 void RetrieveExampleMaterials(bool force)
 {
-	if (QDir(systemData.dataDirectory + "materials")
+	if (QDir(systemData.GetMaterialsFolder())
 					.entryInfoList(QDir::NoDotAndDotDot | QDir::AllEntries)
 					.count()
 				== 0
@@ -581,7 +670,73 @@ void RetrieveExampleMaterials(bool force)
 			materialFiles.next();
 			if (materialFiles.fileName() == "." || materialFiles.fileName() == "..") continue;
 			fcopy(materialFiles.filePath(),
-				systemData.dataDirectory + "materials/" + materialFiles.fileName());
+				systemData.GetMaterialsFolder() + QDir::separator() + materialFiles.fileName());
 		}
 	}
 }
+
+QThread::Priority GetQThreadPriority(enumRenderingThreadPriority priority)
+{
+	switch (priority)
+	{
+		case renderingPriorityLowest: return QThread::LowestPriority;
+		case renderingPriorityLow: return QThread::LowPriority;
+		case renderingPriorityNormal: return QThread::NormalPriority;
+		case renderingPriorityHigh: return QThread::HighPriority;
+		default: return QThread::NormalPriority;
+	}
+}
+
+void CalcPreferredFontSize(bool noGui)
+{
+	if (!noGui)
+	{
+		const int fontSize = gApplication->font().pointSizeF();
+		systemData.SetPreferredFontPointSize(fontSize);
+
+		QFontMetrics fm(gApplication->font());
+		const int pixelFontSize = fm.height();
+
+		const int thumbnailSize = (pixelFontSize * 8);
+		systemData.SetPreferredThumbnailSize(thumbnailSize);
+	}
+	else
+	{
+		systemData.SetPreferredFontPointSize(10);
+		systemData.SetPreferredFontSize(10);
+		systemData.SetPreferredThumbnailSize(80);
+	}
+}
+
+QString sSystem::GetIniFile() const
+{
+	double version = MANDELBULBER_VERSION;
+	int versionInt = int(version * 100);
+
+	QString iniFileName = QString("mandelbulber_%1.ini").arg(versionInt);
+	QString fullIniFileName = dataDirectoryHidden + iniFileName;
+
+	// if setting file doesn't exist then look for older files
+	if (!QFile::exists(fullIniFileName))
+	{
+		QString tempFileName;
+		for (int ver = versionInt; ver >= 212; ver--)
+		{
+			if (ver == 212)
+			{
+				tempFileName = QString("mandelbulber.ini");
+			}
+			else
+			{
+				tempFileName = QString("mandelbulber_%1.ini").arg(ver);
+			}
+
+			if (QFile::exists(dataDirectoryHidden + tempFileName))
+			{
+				fcopy(dataDirectoryHidden + tempFileName, fullIniFileName);
+				WriteLogString("Found older settings file", dataDirectoryHidden + tempFileName, 1);
+			}
+		}
+	}
+	return fullIniFileName;
+}
