@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -39,31 +39,49 @@
  */
 
 #include "texture.hpp"
+
 #include "common_math.h"
 #include "error_message.hpp"
 #include "files.h"
+#include "qimage.h"
 
 // constructor
 cTexture::cTexture(QString filename, enumUseMipmaps mode, bool beQuiet)
 {
-	bitmap = NULL;
-	QImage qimage;
-	qimage.load(filename);
-	qimage = qimage.convertToFormat(QImage::Format_RGB888);
+	bitmap = nullptr;
 
-	if (!qimage.isNull())
+	filename = FilePathHelperTextures(filename);
+
+	// try to load image if it's PNG format (this one supports 16-bit depth images)
+	bitmap = LoadPNG(filename, width, height);
+
+	// if not try to use Qt image loader
+	if (!bitmap)
 	{
-		width = qimage.width();
-		height = qimage.height();
-		bitmap = new sRGB8[width * height];
-
-		for (int y = 0; y < height; y++)
+		QImage qImage;
+		qImage.load(filename);
+		qImage = qImage.convertToFormat(QImage::Format_RGB888);
+		if (!qImage.isNull())
 		{
-			memcpy(&bitmap[y * width], qimage.scanLine(y), sizeof(sRGB8) * width);
+			width = qImage.width();
+			height = qImage.height();
+			bitmap = new sRGBA16[width * height];
+			for (int y = 0; y < height; y++)
+			{
+				sRGB8 *line = reinterpret_cast<sRGB8 *>(qImage.scanLine(y));
+				for (int x = 0; x < width; x++)
+				{
+					const sRGBA16 pixel(static_cast<unsigned short>(line[x].R) * 256,
+						static_cast<unsigned short>(line[x].G) * 256,
+						static_cast<unsigned short>(line[x].B) * 256, 65535);
+					bitmap[x + y * width] = pixel;
+				}
+			}
 		}
+	}
 
-		// qDebug() << "cTexture::cTexture(QString filename, bool beQuiet): "
-		// 				 << "(sRGB8*)(qimage.bits());:" << width * height * sizeof(sRGB8);
+	if (bitmap)
+	{
 		loaded = true;
 		originalFileName = filename;
 		if (mode == useMipmaps)
@@ -79,10 +97,8 @@ cTexture::cTexture(QString filename, enumUseMipmaps mode, bool beQuiet)
 		width = 100;
 		height = 100;
 		loaded = false;
-		bitmap = new sRGB8[100 * 100];
-		memset(bitmap, 255, sizeof(sRGB8) * 100 * 100);
-		// qDebug() << "cTexture::cTexture(QString filename, bool beQuiet): "
-		// 				 << "new sRGB8[100 * 100];:" << width * height * sizeof(sRGB8);
+		bitmap = new sRGBA16[100 * 100];
+		memset(bitmap, 255, sizeof(sRGBA16) * 100 * 100);
 	}
 	invertGreen = false;
 }
@@ -94,10 +110,8 @@ cTexture::cTexture(const cTexture &tex)
 	height = tex.height;
 	loaded = tex.loaded;
 	originalFileName = tex.originalFileName;
-	bitmap = new sRGB8[width * height];
-	// qDebug() << "cTexture::cTexture(const cTexture &tex): "
-	// 				 << "new sRGB8[width * height]:" << width * height * sizeof(sRGB8);
-	memcpy(bitmap, tex.bitmap, sizeof(sRGB8) * width * height);
+	bitmap = new sRGBA16[width * height];
+	memcpy(bitmap, tex.bitmap, sizeof(sRGBA16) * width * height);
 	mipmaps = tex.mipmaps;
 	mipmapSizes = tex.mipmapSizes;
 	invertGreen = tex.invertGreen;
@@ -108,18 +122,14 @@ cTexture &cTexture::operator=(const cTexture &tex)
 	if (bitmap)
 	{
 		delete[] bitmap;
-		bitmap = NULL;
-		// qDebug() << "cTexture& cTexture::operator=(const cTexture &tex): "
-		// 				 << "delete[] bitmap;:" << width * height * sizeof(sRGB8);
+		bitmap = nullptr;
 	}
 	width = tex.width;
 	height = tex.height;
 	loaded = tex.loaded;
 	originalFileName = tex.originalFileName;
-	bitmap = new sRGB8[width * height];
-	// qDebug() << "cTexture& cTexture::operator=(const cTexture &tex): "
-	// 				 << "new sRGB8[width * height];:" << width * height * sizeof(sRGB8);
-	memcpy(bitmap, tex.bitmap, sizeof(sRGB8) * width * height);
+	bitmap = new sRGBA16[width * height];
+	memcpy(bitmap, tex.bitmap, sizeof(sRGBA16) * width * height);
 	mipmaps = tex.mipmaps;
 	mipmapSizes = tex.mipmapSizes;
 	invertGreen = tex.invertGreen;
@@ -127,28 +137,58 @@ cTexture &cTexture::operator=(const cTexture &tex)
 	return *this;
 }
 
+cTexture &cTexture::operator=(cTexture &&tex)
+{
+	if (bitmap)
+	{
+		delete[] bitmap;
+		bitmap = nullptr;
+	}
+	width = tex.width;
+	height = tex.height;
+	loaded = tex.loaded;
+	originalFileName = tex.originalFileName;
+	mipmaps = tex.mipmaps;
+	mipmapSizes = tex.mipmapSizes;
+	invertGreen = tex.invertGreen;
+
+	// move
+	bitmap = tex.bitmap;
+	tex.bitmap = nullptr;
+	tex.loaded = false;
+
+	return *this;
+}
+
 void cTexture::FromQByteArray(QByteArray *buffer, enumUseMipmaps mode)
 {
 	if (bitmap)
 	{
-		// qDebug() << "void cTexture::FromQByteArray(QByteArray buffer):"
-		// 				 << "delete[] bitmap;:" << width * height * sizeof(sRGB8);
 		delete[] bitmap;
 	}
 
-	bitmap = NULL;
-	QImage qimage(*buffer);
-	qimage.loadFromData(*buffer);
-	qimage = qimage.convertToFormat(QImage::Format_RGB888);
-	if (!qimage.isNull())
+	bitmap = nullptr;
+	QImage qImage(*buffer);
+	qImage.loadFromData(*buffer);
+	qImage = qImage.convertToFormat(QImage::Format_RGB888);
+
+	if (!qImage.isNull())
 	{
-		width = qimage.width();
-		height = qimage.height();
-		bitmap = new sRGB8[width * height];
-		memcpy(bitmap, qimage.constBits(), sizeof(sRGB8) * width * height);
-		// qDebug() << "void cTexture::FromQByteArray(QByteArray buffer): "
-		// 				 << "(sRGB8*)(qimage.bits()):" << width * height * sizeof(sRGB8);
-		CreateMipMaps();
+		width = qImage.width();
+		height = qImage.height();
+		bitmap = new sRGBA16[width * height];
+		for (int y = 0; y < height; y++)
+		{
+			sRGB8 *line = reinterpret_cast<sRGB8 *>(qImage.scanLine(y));
+			for (int x = 0; x < width; x++)
+			{
+				const sRGBA16 pixel(static_cast<unsigned short>(line[x].R) * 256,
+					static_cast<unsigned short>(line[x].G) * 256,
+					static_cast<unsigned short>(line[x].B) * 256, 65535);
+				bitmap[x + y * width] = pixel;
+			}
+		}
+
 		loaded = true;
 
 		if (mode == useMipmaps)
@@ -163,41 +203,33 @@ void cTexture::FromQByteArray(QByteArray *buffer, enumUseMipmaps mode)
 		width = 100;
 		height = 100;
 		loaded = false;
-		bitmap = new sRGB8[100 * 100];
-		memset(bitmap, 255, sizeof(sRGB8) * 100 * 100);
+		bitmap = new sRGBA16[100 * 100];
+		memset(bitmap, 255, sizeof(sRGBA16) * 100 * 100);
 	}
 }
 
-cTexture::cTexture(void)
+cTexture::cTexture()
 {
 	width = 100;
 	height = 100;
 	loaded = false;
-	bitmap = new sRGB8[100 * 100];
-	memset(bitmap, 255, sizeof(sRGB8) * 100 * 100);
-	// qDebug() << "cTexture::cTexture(void):"
-	// 				 << "new sRGB8[100 * 100]" << width * height * sizeof(sRGB8);
+	bitmap = new sRGBA16[100 * 100];
+	memset(bitmap, 255, sizeof(sRGBA16) * 100 * 100);
 	invertGreen = false;
 }
 
 // destructor
-cTexture::~cTexture(void)
+cTexture::~cTexture()
 {
 	if (bitmap)
 	{
 		delete[] bitmap;
-		bitmap = NULL;
-		// qDebug() << "cTexture::~cTexture(void):"
-		// 				 << "delete[] bitmap:" << width * height * sizeof(sRGB8);
-	}
-	else
-	{
-		qCritical() << "cTexture::~cTexture(void): bitmap was not allocated before";
+		bitmap = nullptr;
 	}
 }
 
 // read pixel
-sRGBfloat cTexture::Pixel(double x, double y, double pixelSize)
+sRGBFloat cTexture::Pixel(double x, double y, double pixelSize) const
 {
 	if (x >= 0 && x < width && y >= 0 && y < height - 1.0)
 	{
@@ -205,54 +237,53 @@ sRGBfloat cTexture::Pixel(double x, double y, double pixelSize)
 	}
 	else
 	{
-		return sRGBfloat(0.0, 0.0, 0.0);
+		return sRGBFloat(0.0, 0.0, 0.0);
 	}
 }
 
-sRGBfloat cTexture::Pixel(CVector2<double> point, double pixelSize) const
+sRGBFloat cTexture::Pixel(CVector2<double> point, double pixelSize) const
 {
-  if(point.x > 0)
-    point.x = fmod(point.x, 1.0);
-  else
-    point.x = 1.0 + fmod(point.x, 1.0);
+	if (point.x > 0)
+		point.x = fmod(point.x, 1.0);
+	else
+		point.x = 1.0 + fmod(point.x, 1.0);
 
-  if(point.y > 0)
-    point.y = fmod(point.y, 1.0);
-  else
-    point.y = 1.0 + fmod(point.y, 1.0);
+	if (point.y > 0)
+		point.y = fmod(point.y, 1.0);
+	else
+		point.y = 1.0 + fmod(point.y, 1.0);
 
-	point.x *= (double)width;
-	point.y *= (double)height;
+	point.x *= double(width);
+	point.y *= double(height);
 	return MipMap(point.x, point.y, pixelSize);
 }
 
-sRGB8 cTexture::LinearInterpolation(double x, double y)
+sRGBA16 cTexture::LinearInterpolation(double x, double y) const
 {
-	sRGB8 color;
-	int ix = (int)x;
-	int iy = (int)y;
-	double rx = (x - (int)x);
-	double ry = (y - (int)y);
-	sRGB8 k1 = bitmap[iy * width + ix];
-	sRGB8 k2 = bitmap[iy * width + ix + 1];
-	sRGB8 k3 = bitmap[(iy + 1) * width + ix];
-	sRGB8 k4 = bitmap[(iy + 1) * width + ix + 1];
-	color.R = (unsigned char)(k1.R * (1.0 - rx) * (1.0 - ry) + k2.R * (rx) * (1.0 - ry)
-														+ k3.R * (1.0 - rx) * ry + k4.R * (rx * ry));
-	color.G = (unsigned char)(k1.G * (1.0 - rx) * (1.0 - ry) + k2.G * (rx) * (1.0 - ry)
-														+ k3.G * (1.0 - rx) * ry + k4.G * (rx * ry));
-	color.B = (unsigned char)(k1.B * (1.0 - rx) * (1.0 - ry) + k2.B * (rx) * (1.0 - ry)
-														+ k3.B * (1.0 - rx) * ry + k4.B * (rx * ry));
+	sRGBA16 color;
+	const int ix = int(x);
+	const int iy = int(y);
+	const double rx = x - int(x);
+	const double ry = y - int(y);
+	const sRGBA16 k1 = bitmap[iy * width + ix];
+	const sRGBA16 k2 = bitmap[iy * width + ix + 1];
+	const sRGBA16 k3 = bitmap[(iy + 1) * width + ix];
+	const sRGBA16 k4 = bitmap[(iy + 1) * width + ix + 1];
+	color.R = static_cast<unsigned short>(k1.R * (1.0 - rx) * (1.0 - ry) + k2.R * rx * (1.0 - ry)
+																				+ k3.R * (1.0 - rx) * ry + k4.R * (rx * ry));
+	color.G = static_cast<unsigned short>(k1.G * (1.0 - rx) * (1.0 - ry) + k2.G * rx * (1.0 - ry)
+																				+ k3.G * (1.0 - rx) * ry + k4.G * (rx * ry));
+	color.B = static_cast<unsigned short>(k1.B * (1.0 - rx) * (1.0 - ry) + k2.B * rx * (1.0 - ry)
+																				+ k3.B * (1.0 - rx) * ry + k4.B * (rx * ry));
 	return color;
 }
 
-sRGBfloat cTexture::BicubicInterpolation(
-	double x, double y, const sRGB8 *bitmap, int w, int h) const
+sRGBFloat cTexture::BicubicInterpolation(double x, double y, const sRGBA16 *bitmap, int w, int h)
 {
-	int ix = (int)x;
-	int iy = (int)y;
-	double rx = (x - ix);
-	double ry = (y - iy);
+	const int ix = int(x);
+	const int iy = int(y);
+	const double rx = x - ix;
+	const double ry = y - iy;
 
 	double R[4][4], G[4][4], B[4][4];
 
@@ -264,8 +295,8 @@ sRGBfloat cTexture::BicubicInterpolation(
 			int iyy = iy + yy - 1;
 			ixx = (ixx + w) % w;
 			iyy = (iyy + h) % h;
-			int address2 = ixx + iyy * w;
-			sRGB8 pixel = bitmap[address2];
+			const int address2 = ixx + iyy * w;
+			const sRGBA16 pixel = bitmap[address2];
 			R[xx][yy] = pixel.R;
 			G[xx][yy] = pixel.G;
 			B[xx][yy] = pixel.B;
@@ -276,24 +307,24 @@ sRGBfloat cTexture::BicubicInterpolation(
 	double dG = bicubicInterpolate(G, rx, ry);
 	double dB = bicubicInterpolate(B, rx, ry);
 	if (dR < 0) dR = 0;
-	if (dR > 255) dR = 255;
+	if (dR > 65535) dR = 65535;
 	if (dG < 0) dG = 0;
-	if (dG > 255) dG = 255;
+	if (dG > 65535) dG = 65535;
 	if (dB < 0) dB = 0;
-	if (dB > 255) dB = 255;
+	if (dB > 65535) dB = 65535;
 
-	return sRGBfloat((float)(dR / 256.0), (float)(dG / 256.0), (float)(dB / 256.0));
+	return sRGBFloat(float(dR / 65536.0), float(dG / 65536.0), float(dB / 65536.0));
 }
 
-sRGB8 cTexture::FastPixel(int x, int y)
+sRGBA16 cTexture::FastPixel(int x, int y) const
 {
 	return bitmap[x + y * width];
 }
 
 CVector3 cTexture::NormalMapFromBumpMap(CVector2<double> point, double bump, double pixelSize) const
 {
-	int intX = (int)point.x;
-	int intY = (int)point.y;
+	const int intX = int(point.x);
+	const int intY = int(point.y);
 	point.x = point.x - intX;
 	point.y = point.y - intY;
 	if (point.x < 0.0) point.x += 1.0;
@@ -317,13 +348,13 @@ CVector3 cTexture::NormalMapFromBumpMap(CVector2<double> point, double bump, dou
 
 CVector3 cTexture::NormalMap(CVector2<double> point, double bump, double pixelSize) const
 {
-	int intX = (int)point.x;
-	int intY = (int)point.y;
+	const int intX = int(point.x);
+	const int intY = int(point.y);
 	point.x = point.x - intX;
 	point.y = point.y - intY;
 	if (point.x < 0.0) point.x += 1.0;
 	if (point.y < 0.0) point.y += 1.0;
-	sRGBfloat normalPixel = MipMap(point.x * width, point.y * height, pixelSize);
+	const sRGBFloat normalPixel = MipMap(point.x * width, point.y * height, pixelSize);
 	CVector3 normal(normalPixel.R * 2.0 - 1.0, normalPixel.G * 2.0 - 1.0, normalPixel.B);
 	normal.x *= -bump;
 	normal.y *= -bump;
@@ -333,9 +364,9 @@ CVector3 cTexture::NormalMap(CVector2<double> point, double bump, double pixelSi
 	return normal;
 }
 
-sRGBfloat cTexture::MipMap(double x, double y, double pixelSize) const
+sRGBFloat cTexture::MipMap(double x, double y, double pixelSize) const
 {
-	pixelSize /= (double)max(width, height);
+	pixelSize /= double(max(width, height));
 	if (mipmaps.size() > 0 && pixelSize > 0)
 	{
 		if (pixelSize < 1e-20) pixelSize = 1e-20;
@@ -343,14 +374,14 @@ sRGBfloat cTexture::MipMap(double x, double y, double pixelSize) const
 		if (dMipLayer < 0) dMipLayer = 0;
 		if (dMipLayer + 1 >= mipmaps.size() - 1) dMipLayer = mipmaps.size() - 1;
 
-		int layerBig = (int)dMipLayer;
-		int layerSmall = (int)(dMipLayer + 1);
-		double sizeMultipBig = pow(2.0, (double)layerBig);
-		double sizeMultipSmall = pow(2.0, (double)layerSmall);
-		double trans = dMipLayer - layerBig;
-		double transN = 1.0 - trans;
+		const int layerBig = int(dMipLayer);
+		const int layerSmall = int(dMipLayer + 1);
+		const double sizeMultipleBig = pow(2.0, double(layerBig));
+		const double sizeMultipleSmall = pow(2.0, double(layerSmall));
+		const double trans = dMipLayer - layerBig;
+		const double transN = 1.0 - trans;
 
-		const sRGB8 *bigBitmap, *smallBitmap;
+		const sRGBA16 *bigBitmap, *smallBitmap;
 		CVector2<int> bigBitmapSize, smallBitmapSize;
 		if (layerBig >= 0 && layerBig <= mipmaps.length() && layerSmall >= 0
 				&& layerSmall <= mipmaps.length())
@@ -370,20 +401,20 @@ sRGBfloat cTexture::MipMap(double x, double y, double pixelSize) const
 				bigBitmapSize = mipmapSizes[layerBig - 1];
 				smallBitmapSize = mipmapSizes[layerSmall - 1];
 			}
-			sRGBfloat pixelFromBig = BicubicInterpolation(
-				x / sizeMultipBig, y / sizeMultipBig, bigBitmap, bigBitmapSize.x, bigBitmapSize.y);
-			sRGBfloat pixelFromSmall = BicubicInterpolation(x / sizeMultipSmall, y / sizeMultipSmall,
-				smallBitmap, smallBitmapSize.x, smallBitmapSize.y);
+			const sRGBFloat pixelFromBig = BicubicInterpolation(
+				x / sizeMultipleBig, y / sizeMultipleBig, bigBitmap, bigBitmapSize.x, bigBitmapSize.y);
+			const sRGBFloat pixelFromSmall = BicubicInterpolation(x / sizeMultipleSmall,
+				y / sizeMultipleSmall, smallBitmap, smallBitmapSize.x, smallBitmapSize.y);
 
-			sRGBfloat pixel;
-			pixel.R = (float)(pixelFromSmall.R * trans + pixelFromBig.R * transN);
-			pixel.G = (float)(pixelFromSmall.G * trans + pixelFromBig.G * transN);
-			pixel.B = (float)(pixelFromSmall.B * trans + pixelFromBig.B * transN);
+			sRGBFloat pixel;
+			pixel.R = float(pixelFromSmall.R * trans + pixelFromBig.R * transN);
+			pixel.G = float(pixelFromSmall.G * trans + pixelFromBig.G * transN);
+			pixel.B = float(pixelFromSmall.B * trans + pixelFromBig.B * transN);
 			return pixel;
 		}
 		else
 		{
-			return sRGBfloat();
+			return sRGBFloat();
 		}
 	}
 	else
@@ -398,24 +429,25 @@ void cTexture::CreateMipMaps()
 	int prevH = height;
 	int w = width / 2;
 	int h = height / 2;
-	sRGB8 *prevBitmap = bitmap;
+	sRGBA16 *prevBitmap = bitmap;
 	while (w > 0 && h > 0)
 	{
-		QVector<sRGB8> newMipmapV(w * h);
-		sRGB8 *newMipmap = newMipmapV.data();
+		QVector<sRGBA16> newMipmapV(w * h);
+		sRGBA16 *newMipmap = newMipmapV.data();
 
 		for (int y = 0; y < h; y++)
 		{
 			for (int x = 0; x < w; x++)
 			{
-				sRGB8 newPixel;
-				sRGB8 p1 = prevBitmap[WrapInt(x * 2, prevW) + WrapInt(y * 2, prevH) * prevW];
-				sRGB8 p2 = prevBitmap[WrapInt(x * 2 + 1, prevW) + WrapInt(y * 2, prevH) * prevW];
-				sRGB8 p3 = prevBitmap[WrapInt(x * 2, prevW) + WrapInt(y * 2 + 1, prevH) * prevW];
-				sRGB8 p4 = prevBitmap[WrapInt(x * 2 + 1, prevW) + WrapInt(y * 2 + 1, prevH) * prevW];
-				newPixel.R = (unsigned char)(((int)p1.R + p2.R + p3.R + p4.R) / 4);
-				newPixel.G = (unsigned char)(((int)p1.G + p2.G + p3.G + p4.G) / 4);
-				newPixel.B = (unsigned char)(((int)p1.B + p2.B + p3.B + p4.B) / 4);
+				sRGBA16 newPixel;
+				const sRGBA16 p1 = prevBitmap[WrapInt(x * 2, prevW) + WrapInt(y * 2, prevH) * prevW];
+				const sRGBA16 p2 = prevBitmap[WrapInt(x * 2 + 1, prevW) + WrapInt(y * 2, prevH) * prevW];
+				const sRGBA16 p3 = prevBitmap[WrapInt(x * 2, prevW) + WrapInt(y * 2 + 1, prevH) * prevW];
+				const sRGBA16 p4 =
+					prevBitmap[WrapInt(x * 2 + 1, prevW) + WrapInt(y * 2 + 1, prevH) * prevW];
+				newPixel.R = static_cast<unsigned short>((int(p1.R) + p2.R + p3.R + p4.R) / 4);
+				newPixel.G = static_cast<unsigned short>((int(p1.G) + p2.G + p3.G + p4.G) / 4);
+				newPixel.B = static_cast<unsigned short>((int(p1.B) + p2.B + p3.B + p4.B) / 4);
 				newMipmap[x + y * w] = newPixel;
 			}
 		}
