@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -38,8 +38,8 @@
  * more information on Mipmaps:  https://en.wikipedia.org/wiki/Mipmap
  */
 
-#ifndef TEXTURE_HPP_
-#define TEXTURE_HPP_
+#ifndef MANDELBULBER2_SRC_TEXTURE_HPP_
+#define MANDELBULBER2_SRC_TEXTURE_HPP_
 
 #include <qbytearray.h>
 #include <qlist.h>
@@ -61,34 +61,35 @@ public:
 	cTexture();
 	cTexture(const cTexture &tex);
 	cTexture &operator=(const cTexture &tex);
+	cTexture &operator=(cTexture &&tex);
 
-	~cTexture(void);
-	int Height(void) { return height; }
-	int Width(void) { return width; }
-	sRGBfloat Pixel(double x, double y, double pixelSize = 0.0);
-	sRGBfloat Pixel(CVector2<double> point, double pixelSize = 0.0) const;
-	sRGB8 FastPixel(int x, int y);
-	bool IsLoaded(void) const { return loaded; }
-	QString GetFileName() { return originalFileName; }
+	~cTexture();
+	int Height() const { return height; }
+	int Width() const { return width; }
+	sRGBFloat Pixel(double x, double y, double pixelSize = 0.0) const;
+	sRGBFloat Pixel(CVector2<double> point, double pixelSize = 0.0) const;
+	sRGBA16 FastPixel(int x, int y) const;
+	bool IsLoaded() const { return loaded; }
+	QString GetFileName() const { return originalFileName; }
 	void FromQByteArray(QByteArray *buffer, enumUseMipmaps mode);
 	CVector3 NormalMapFromBumpMap(CVector2<double> point, double bump, double pixelSize = 0.0) const;
 	CVector3 NormalMap(CVector2<double> point, double bump, double pixelSize = 0.0) const;
 	void SetInvertGreen(bool invert) { invertGreen = invert; }
 
 private:
-	sRGB8 LinearInterpolation(double x, double y);
-	sRGBfloat BicubicInterpolation(double x, double y, const sRGB8 *bitmap, int w, int h) const;
-	sRGBfloat MipMap(double x, double y, double pixelSize) const;
+	sRGBA16 LinearInterpolation(double x, double y) const;
+	static sRGBFloat BicubicInterpolation(double x, double y, const sRGBA16 *bitmap, int w, int h);
+	sRGBFloat MipMap(double x, double y, double pixelSize) const;
 	void CreateMipMaps();
-	inline int WrapInt(int a, int size) { return (a + size) % size; }
-	sRGB8 *bitmap;
+	static int WrapInt(int a, int size) { return (a + size) % size; }
+	sRGBA16 *bitmap;
 	int width;
 	int height;
 	bool loaded;
 	QString originalFileName;
-	QList<QVector<sRGB8>> mipmaps;
+	QList<QVector<sRGBA16>> mipmaps;
 	QList<CVector2<int>> mipmapSizes;
 	bool invertGreen;
 };
 
-#endif /* TEXTURE_HPP_ */
+#endif /* MANDELBULBER2_SRC_TEXTURE_HPP_ */
