@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014 Krzysztof Marczak        §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -14,7 +14,7 @@
  * either version 3 of the License,    TWw [.j"5=~N[=§%=%W,T ]R,"=="Y[LFT ]N
  * or (at your option)                   TW=,-#"%=;[  =Q:["V""  ],,M.m == ]N
  * any later version.                      J§"mr"] ,=,," =="""J]= M"M"]==ß"
- *                                          §= "=C=4 §"eM "=B:m\4"]#F,§~
+ *                                          §= "=C=4 §"eM "=B:m|4"]#F,§~
  * Mandelbulber is distributed in            "9w=,,]w em%wJ '"~" ,=,,ß"
  * the hope that it will be useful,                 . "K=  ,=RMMMßM"""
  * but WITHOUT ANY WARRANTY;                            .'''
@@ -39,9 +39,11 @@
  */
 
 #include "undo.h"
+
 #include "error_message.hpp"
 #include "initparameters.hpp"
 #include "settings.hpp"
+#include "system.hpp"
 
 cUndo gUndo;
 
@@ -63,7 +65,7 @@ void cUndo::Store(cParameterContainer *par, cFractalContainer *parFractal, cAnim
 	WriteLog("Autosave started", 2);
 	cSettings parSettings(cSettings::formatCondensedText);
 	parSettings.CreateText(gPar, gParFractal, gAnimFrames, gKeyframes);
-	parSettings.SaveToFile(systemData.autosaveFile);
+	parSettings.SaveToFile(systemData.GetAutosaveFile());
 	WriteLog("Autosave finished", 2);
 
 	WriteLog("cUndo::Store() started", 2);
@@ -77,6 +79,7 @@ void cUndo::Store(cParameterContainer *par, cFractalContainer *parFractal, cAnim
 	else
 	{
 		record.hasFrames = false;
+		record.animationFrames = cAnimationFrames();
 	}
 
 	if (keyframes)
@@ -87,6 +90,7 @@ void cUndo::Store(cParameterContainer *par, cFractalContainer *parFractal, cAnim
 	else
 	{
 		record.hasKeyframes = false;
+		record.animationKeyframes = cKeyframes();
 	}
 
 	if (undoBuffer.size() > level)
@@ -122,6 +126,7 @@ bool cUndo::Undo(cParameterContainer *par, cFractalContainer *parFractal, cAnima
 			if (keyframes && record.hasKeyframes)
 			{
 				*keyframes = record.animationKeyframes;
+				keyframes->RegenerateAudioTracks(par);
 				*refreshKeyframes = true;
 			}
 		}
@@ -152,6 +157,7 @@ bool cUndo::Redo(cParameterContainer *par, cFractalContainer *parFractal, cAnima
 		if (keyframes && record.hasKeyframes)
 		{
 			*keyframes = record.animationKeyframes;
+			keyframes->RegenerateAudioTracks(par);
 			*refreshKeyframes = true;
 		}
 		return true;
